/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"Geo2Geo.h"

/* Geo2Geotializing the properties of the vertices ******************* */

void
Geo2GeoVerQty (struct GEO GeoA, struct GEO *pGeoB)
{
  (*pGeoB).VerQty = GeoA.VerQty;

  return;
}

void
Geo2GeoVerCoo (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).VerQty; i++)
    for (j = 0; j <= 2; j++)
      (*pGeoB).VerCoo[i][j] = GeoA.VerCoo[i][j];

  return;
}

void
Geo2GeoVerEdge (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).VerQty; i++)
  {
    (*pGeoB).VerEdgeQty[i] = GeoA.VerEdgeQty[i];

    for (j = 0; j <= (*pGeoB).VerEdgeQty[i] - 1; j++)
      (*pGeoB).VerEdgeNb[i][j] = GeoA.VerEdgeNb[i][j];
  }

  return;
}

void
Geo2GeoVerBound (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).VerQty; i++)
    for (j = 0; j <= 3; j++)
      (*pGeoB).VerBound[i][j] = GeoA.VerBound[i][j];

  return;
}

void
Geo2GeoVerState (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).VerQty; i++)
    (*pGeoB).VerState[i] = GeoA.VerState[i];

  return;
}


/* Initializing the properties of the edges ******************* */

void
Geo2GeoEdgeQty (struct GEO GeoA, struct GEO *pGeoB)
{
  (*pGeoB).EdgeQty = GeoA.EdgeQty;

  return;
}

void
Geo2GeoEdgeVerNb (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).EdgeQty; i++)
    for (j = 0; j <= 1; j++)
      (*pGeoB).EdgeVerNb[i][j] = GeoA.EdgeVerNb[i][j];

  return;
}

void
Geo2GeoEdgeFace (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= GeoA.EdgeQty; i++)
  {
    (*pGeoB).EdgeFaceQty[i] = GeoA.EdgeFaceQty[i];

    for (j = 0; j <= (*pGeoB).EdgeFaceQty[i] - 1; j++)
      (*pGeoB).EdgeFaceNb[i][j] = GeoA.EdgeFaceNb[i][j];
  }

  return;
}

void
Geo2GeoEdgeLength (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).EdgeQty; i++)
    (*pGeoB).EdgeLength[i] = GeoA.EdgeLength[i];

  return;
}

void
Geo2GeoEdgeDel (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).EdgeQty; i++)
    (*pGeoB).EdgeDel[i] = GeoA.EdgeDel[i];

  return;
}

void
Geo2GeoEdgeState (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).EdgeQty; i++)
    (*pGeoB).EdgeState[i] = GeoA.EdgeState[i];

  return;
}

/* Geo2Geotializing the properties of the faces ******************* */

void
Geo2GeoFaceQty (struct GEO GeoA, struct GEO *pGeoB)
{
  (*pGeoB).FaceQty = GeoA.FaceQty;

  return;
}

void
Geo2GeoFacePoly (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
    for (j = 0; j <= 1; j++)
      (*pGeoB).FacePoly[i][j] = GeoA.FacePoly[i][j];

  return;
}

void
Geo2GeoFaceVer (struct GEO GeoA, struct GEO *pGeoB, int alloc)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
  {
    (*pGeoB).FaceVerQty[i] = GeoA.FaceVerQty[i];

    if (alloc == 1)
      (*pGeoB).FaceVerNb[i] =
	ut_alloc_1d_int ((*pGeoB).FaceVerQty[i] + 1);

    for (j = 1; j <= (*pGeoB).FaceVerQty[i]; j++)
      (*pGeoB).FaceVerNb[i][j] = GeoA.FaceVerNb[i][j];
  }

  return;
}

/* a preliminar Geo2GeoFaceVer is needed! */
void
Geo2GeoFaceEdge (struct GEO GeoA, struct GEO *pGeoB, int alloc)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
  {
    if (alloc == 1)
    {
      (*pGeoB).FaceEdgeNb[i] =
	ut_alloc_1d_int ((*pGeoB).FaceVerQty[i] + 1);
      (*pGeoB).FaceEdgeOri[i] =
	ut_alloc_1d_int ((*pGeoB).FaceVerQty[i] + 1);
    }

    for (j = 1; j <= (*pGeoB).FaceVerQty[i]; j++)
    {
      (*pGeoB).FaceEdgeNb[i][j] = GeoA.FaceEdgeNb[i][j];
      (*pGeoB).FaceEdgeOri[i][j] = GeoA.FaceEdgeOri[i][j];
    }
  }

  return;
}

void
Geo2GeoFaceFF (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
    (*pGeoB).FaceFF[i] = GeoA.FaceFF[i];

  return;
}

void
Geo2GeoFaceEq (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
    for (j = 0; j <= 3; j++)
      (*pGeoB).FaceEq[i][j] = GeoA.FaceEq[i][j];

  return;
}

void
Geo2GeoFaceBound (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
    (*pGeoB).FaceBound[i] = GeoA.FaceBound[i];

  return;
}

void
Geo2GeoFaceState (struct GEO GeoA, struct GEO *pGeoB)
{
  int i;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
    (*pGeoB).FaceState[i] = GeoA.FaceState[i];

  return;
}

void
Geo2GeoFacePt (struct GEO GeoA, struct GEO *pGeoB)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).FaceQty; i++)
  {
    (*pGeoB).FacePt[i] = GeoA.FacePt[i];
    for (j = 0; j <= 2; j++)
      (*pGeoB).FacePtCoo[i][j] = GeoA.FacePtCoo[i][j];
  }

  return;
}

/* Geo2Geotializing the properties of the faces ******************* */

void
Geo2GeoPolyQty (struct GEO GeoA, struct GEO *pGeoB)
{
  (*pGeoB).PolyQty = GeoA.PolyQty;

  return;
}

void
Geo2GeoPolyFace (struct GEO GeoA, struct GEO *pGeoB, int alloc)
{
  int i, j;

  for (i = 1; i <= (*pGeoB).PolyQty; i++)
  {
    /*sprintf(message,"  poly %d: ",i); */
    /*ut_print_message(-1,message); */
    (*pGeoB).PolyFaceQty[i] = GeoA.PolyFaceQty[i];
    /*sprintf(message,"%d faces\n",GeoA.PolyFaceQty[i]); */
    /*ut_print_message(-1,message); */
    if (alloc == 1)
    {
      (*pGeoB).PolyFaceNb[i] =
	ut_alloc_1d_int ((*pGeoB).PolyFaceQty[i] + 1);
      (*pGeoB).PolyFaceOri[i] =
	ut_alloc_1d_int ((*pGeoB).PolyFaceQty[i] + 1);
    }

    for (j = 1; j <= (*pGeoB).PolyFaceQty[i]; j++)
    {
      /*LineHeader(-1); printf("   face %d/%d\n",j,(*pGeoB).PolyFaceQty[i]); */
      (*pGeoB).PolyFaceNb[i][j] = GeoA.PolyFaceNb[i][j];
      (*pGeoB).PolyFaceOri[i][j] = GeoA.PolyFaceOri[i][j];
    }
  }

  return;
}

void
Geo2GeoDomainSize (struct GEO GeoA, struct GEO *pGeoB, int alloc)
{
  int i, j;

  if (alloc == 1)
    (*pGeoB).DomainSize = ut_alloc_2d (3, 2);

  for (i = 0; i < 3; i++)
    for (j = 0; j < 2; j++)
      (*pGeoB).DomainSize[i][j] = GeoA.DomainSize[i][j];

  return;
}
