/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"DeleteEdge.h"

/* Updating of EdgeState */
void
UpdateEdgeState (struct GEO *pGeo, int edge)
{
  /* edge is set as useless and deleted. */
  (*pGeo).EdgeState[edge] = -1;

  return;
}

/* Searching the vertex to delete and the new one (its coo will change) */
void
SearchDelNNewVer (struct GEO *pGeo, int edge, int *pdelver, int *pnewver)
{
  (*pdelver) =
    IntMax ((*pGeo).EdgeVerNb[edge][0], (*pGeo).EdgeVerNb[edge][1]);
  (*pnewver) =
    IntMin ((*pGeo).EdgeVerNb[edge][0], (*pGeo).EdgeVerNb[edge][1]);

  return;
}


void
UpdateVerCoo (struct GEO *pGeo, int delver, int newver, int verbosity)
{

  UpdateVerCooBary (pGeo, delver, newver);
  UpdateVerCooMiniFF (pGeo, newver, verbosity);

  return;
}

/* UpdateEdgeVerNb updates the edge ver nb
 * the delver vertex is replaced by the new one
 */
void
UpdateEdgeVerNb (struct GEO *pGeo, int delver, int newver, int verbosity)
{
  int i;
  int tmpedge;
  int pos;

  if (verbosity >= 3)
  {
    ut_print_lineheader (-1);
    printf ("    Changing ver %d to ver %d in edges of ver %d\n", newver,
	    delver, delver);
  }

  for (i = 0; i <= (*pGeo).VerEdgeQty[delver] - 1; i++)
  {
    tmpedge = (*pGeo).VerEdgeNb[delver][i];

    if (verbosity >= 3)
    {
      ut_print_lineheader (-1);
      printf ("      ver of edge %d: ", tmpedge);
    }

    pos = oneDIntEltPos ((*pGeo).EdgeVerNb[tmpedge], 0, 1, delver, 0);

    if (verbosity >= 3)
      printf ("%d %d become ", (*pGeo).EdgeVerNb[tmpedge][0],
	      (*pGeo).EdgeVerNb[tmpedge][1]);

    (*pGeo).EdgeVerNb[tmpedge][pos] = newver;

    if (verbosity >= 3)
      printf ("%d %d\n", (*pGeo).EdgeVerNb[tmpedge][0],
	      (*pGeo).EdgeVerNb[tmpedge][1]);
  }

  return;
}

/* UpdateFace updates the properties of the parent faces of the edge */
int
UpdateFaceVerNEdge (struct GEO *pGeo, int edge, int delver, int newver,
		    int verbosity)
{
  int i, err = 0;

  if (verbosity >= 3)
  {
    ut_print_lineheader (-1);
    printf ("  %d parent faces: ", (*pGeo).EdgeFaceQty[edge]);
    for (i = 0; i <= (*pGeo).EdgeFaceQty[edge] - 1; i++)
      printf (" %d", (*pGeo).EdgeFaceNb[edge][i]);
    printf ("\n");

    ut_print_lineheader (-1);
    printf ("Updating face vertices\n");
  }

  UpdateFaceState (pGeo, delver, newver);

  UpdateFaceVer (pGeo, delver, newver, verbosity);

  if (verbosity >= 3)
  {
    printf ("\n");
    ut_print_lineheader (-1);
    printf ("Updating face edges\n");
  }

  err = UpdateFaceEdge (pGeo, edge, verbosity);

  return err;
}

int
UpdateVer (struct GEO *pGeo, int edge, int delver, int newver, int verbosity)
{
  int err = 0;

  /* Updating of VerEdgeQty & VerEdgeNb */
  UpdateVerEdge (pGeo, edge, delver, newver, verbosity);

  /* Updating of VerBound */
  err = UpdateVerBound (pGeo, delver, newver, verbosity);

  /* Updating of VerState */
  UpdateVerState (pGeo, delver, newver, verbosity);

  return err;
}

/* UpdateFaceBound updates the bound of the faces of ver */
void
UpdateFaceBound (struct GEO *pGeo, int ver)
{
  int i, face;
  int *NewVerFace = ut_alloc_1d_int (100);

  neut_geo_ver_face (*pGeo, ver, NewVerFace);

  for (i = 1; i <= NewVerFace[0]; i++)
  {
    face = NewVerFace[i];
    (*pGeo).FaceBound[face] = ThisFaceBound (*pGeo, face);
  }

  ut_free_1d_int (NewVerFace);

  return;
}
