/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"MergeGeo.h"

int
MergeGeo (struct GEO *pGeo, struct GEOPARA GeoPara, struct IN In, int
    *pDelId, int loop)
{
  int i, trash, testg1o;
  int edge, edgeb, newver;
  double lratio1;
  double tmp;
  double maxff, minmaxff = 0;
  int minid = 0;
  int ThisDelQty, DelQty = 0;
  struct GEO GeoC;
  struct GEO GeoCC;
  double lratio2;

  neut_geo_set_zero (&GeoC);
  neut_geo_set_zero (&GeoCC);

  edge = NextEdgeToDel (*pGeo, GeoPara, &lratio1);

  ut_print_message (0, 3, "                        ");
  fflush (stdout);
    
  printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b %d     %4.0f%%    %6d", loop, 0., DelQty);
  while (edge != -1)
  {
    ThisDelQty = 0;

    neut_geo_geo (*pGeo, &GeoC);

    testg1o = DeleteEdge (&GeoC, In, edge, &newver, &maxff, pDelId);

    /* if (testg1o == 0 and) maxff < max, the edge is deleted */
    /* else (maxff > max), we try with another edge */
    /* even if testg1o != 0, the deletion continues:  we let the
     * program a chance to find a suitable solution, and generally it
     * does.
     */

    /* if the max ff is acceptable, we record the modif into Geo
     * else, we re-initialize it.
     */
    if (maxff < GeoPara.maxff)	/* also means that testg1o == 0 */
    {
      ThisDelQty = 1;
      neut_geo_geo (GeoC, pGeo);
    }
    else
    {
      if (In.verbosity >= 2)
      {
	printf ("Deleting neighbouring edges, in turn: %d: ",
		GeoC.VerEdgeQty[newver]);
	for (i = 0; i <= GeoC.VerEdgeQty[newver] - 1; i++)
	  printf ("%d ", GeoC.VerEdgeNb[newver][i]);
	printf ("\n");
      }

      minmaxff = 180;
      minid = 0;
      for (i = 0; i < GeoC.VerEdgeQty[newver]; i++)
      {
	edgeb = GeoC.VerEdgeNb[newver][i];
	
	neut_geo_edge_selratio (*pGeo, GeoPara, edgeb, &lratio2);

	if (lratio1 + lratio2 < 2)
	{
	  neut_geo_geo (GeoC, &GeoCC);
	  testg1o =
	    DeleteEdge (&GeoCC, In, edgeb, &trash, &tmp, pDelId);

	  if (testg1o == 0 && tmp < minmaxff)
	  {
	    minmaxff = tmp;
	    minid = edgeb;
	  }
	}
	else if (In.verbosity >= 3)
	{
	  ut_print_lineheader (0);
	  printf ("edgedel %d=-1 --> skipping\n", edgeb);
	}
      }

      if (minmaxff < GeoPara.maxff)
      {
	ThisDelQty = 2;

	DeleteEdge (pGeo, In, edge, &newver, &tmp, pDelId);
	DeleteEdge (pGeo, In, minid, &newver, &tmp, pDelId);

	/* this node has been deleted: edgedel <- 1 */
	(*pGeo).EdgeDel[minid] = 1;
      }
    }

    /* in all cases, edgedel <- 1 */
    (*pGeo).EdgeDel[edge] = 1;

    DelQty += ThisDelQty;

    if (In.verbosity >= 1)
      PrintDeletion (loop, lratio1, ThisDelQty, edge, minid, maxff,
		     minmaxff);

    /* Searching next edge to delete */
    edge = NextEdgeToDel (*pGeo, GeoPara, &lratio1);
    
    printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b %d     %4.0f%%    %6d", loop, 100 * lratio1, DelQty);
    fflush (stdout);

    neut_geo_free (&GeoC);
    neut_geo_free (&GeoCC);
  }
  printf ("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b %d     %4.0f%%    %6d", loop, 100., DelQty);
  printf ("\n");

  /* ForceMerge deletes the edges specified manually 
   * ForceMerge(pGeo,GeoPara);
   */

  /* at the end,  edgedel is reset to 0. */
  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    (*pGeo).EdgeDel[i] = 0;

  /* printf("end of MergeGeo\n"); */
  neut_geo_free (&GeoC);
  neut_geo_free (&GeoCC);

  return DelQty;
}
