/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"WriteMeshFev.h"

void
neut_mesh_fprintf_fev_parms (FILE* file, struct NODES Nodes, struct MESH Mesh)
{
  /* fev1 = nb of elts and nodes */
  fprintf (file, "%d %d 9\n", Mesh.EltQty, Nodes.NodeQty);
}

void
neut_mesh_fprintf_fev_nodes (FILE * file, struct NODES Nodes)
{
  int i, j;

  for (i = 1; i <= Nodes.NodeQty; i++)
  {
    fprintf (file, "%d", i - 1);
    for (j = 0; j < 3; j++)
      fprintf (file, "  %.12f", Nodes.NodeCoo[i][j]);
    fprintf (file, "\n");
  }

  return;
}

void
neut_mesh_fprintf_fev_elts (FILE * file, struct MESH Mesh, int **FoD)
{
  int i, j, fod;
  char **fodnames = ut_alloc_2d_char (7, 3);

  sprintf (fodnames[1], "x0");
  sprintf (fodnames[2], "x1");
  sprintf (fodnames[3], "y0");
  sprintf (fodnames[4], "y1");
  sprintf (fodnames[5], "z0");
  sprintf (fodnames[6], "z1");

  if (Mesh.Dimension == 2 && Mesh.EltOrder == 1)
  {
    fprintf (file, "1\n      %d\n", Mesh.EltQty);
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", i - 1);
      for (j = 0; j < 3; j++)
	fprintf (file, " %d", Mesh.EltNodes[i][j] - 1);

      /* searching fod via elset */
      if (neut_fod_facefod (FoD, Mesh.EltElset[i], &fod) < 0)
	abort ();

      fprintf (file, " %s\n", fodnames[fod]);
    }
  }
  else if (Mesh.Dimension == 2 && Mesh.EltOrder == 2)
  {
    fprintf (file, "1\n%8d\n", Mesh.EltQty);
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", i - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][0] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][3] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][1] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][4] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][2] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][5] - 1);

      /* searching fod via elset */
      if (neut_fod_facefod (FoD, Mesh.EltElset[i], &fod) < 0)
	abort ();

      fprintf (file, " %s\n", fodnames[fod]);
    }
  }
  else if (Mesh.Dimension == 3 && Mesh.EltOrder == 1)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", i - 1);
      for (j = 0; j < 4; j++)
	fprintf (file, " %d", Mesh.EltNodes[i][j] - 1);
      fprintf (file, "\n");
    }
  else if (Mesh.Dimension == 3 && Mesh.EltOrder == 2)
    for (i = 1; i <= Mesh.EltQty; i++)
    {
      fprintf (file, "%d ", i - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][0] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][4] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][1] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][5] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][2] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][6] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][7] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][9] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][8] - 1);
      fprintf (file, " %d", Mesh.EltNodes[i][3] - 1);
      fprintf (file, "\n");
    }

  return;
}

void
neut_mesh_fprintf_fev_skinelts (FILE * file, struct MESH Mesh2D,
				struct MESH Mesh3D, struct NODES Nodes,
			        char* surflist, int **FoD)
{
  double res;
  int i, j, k, l, eltnb, eltqty, elt3dqty;
  int* elt3d = ut_alloc_1d_int (2);
  char **fodnames = ut_alloc_2d_char (7, 3);
  double *n = ut_alloc_1d (3);
  double *n0 = ut_alloc_1d (3);
  int seqo1[3] = { 0, 1, 2 };
  int seqo2[6] = { 0, 3, 1, 4, 2, 5 };
  char** name = NULL;
  int qty;
  
  ut_string_separate (surflist, ',', &name, &qty);

  if (FoD == NULL)
  {
    ut_print_message (2,0,"neut_mesh_fprintf_fev_skinelts: FoD == NULL\n");
    ut_error_reportbug ();
  }

  sprintf (fodnames[1], "x0");
  sprintf (fodnames[2], "x1");
  sprintf (fodnames[3], "y0");
  sprintf (fodnames[4], "y1");
  sprintf (fodnames[5], "z0");
  sprintf (fodnames[6], "z1");

  fprintf (file, "%d\n", qty);

  for (i = 1; i <= 6; i++)
  {
    if (ut_string_inlist (surflist, ',', fodnames[i]) == 0)
      continue;

    // calculating number of elements
    eltqty = 0;
    for (j = 1; j <= FoD[i][0]; j++)
      eltqty += Mesh2D.Elsets[FoD[i][j]][0];
  
    fprintf (file, "%d\n", eltqty);

    switch (i)
    {
      case 1:
	ol_r_set_this (n0,-1, 0, 0);
	break;
      case 2:
	ol_r_set_this (n0, 1, 0, 0);
	break;
      case 3:
	ol_r_set_this (n0, 0,-1, 0);
	break;
      case 4:
	ol_r_set_this (n0, 0, 1, 0);
	break;
      case 5:
	ol_r_set_this (n0, 0, 0,-1);
	break;
      case 6:
	ol_r_set_this (n0, 0, 0, 1);
	break;
      default:
	abort ();
    }

    for (j = 1; j <= FoD[i][0]; j++)
    {
      for (k = 1; k <= Mesh2D.Elsets[FoD[i][j]][0]; k++)
      {
	eltnb = Mesh2D.Elsets[FoD[i][j]][k];
	neut_mesh_eltnormal (Mesh2D, Nodes, eltnb, n);

	neut_mesh_elt2delts3d (Mesh2D, eltnb, Mesh3D, elt3d, &elt3dqty);

	if (elt3dqty != 1)
	{
	  printf ("Nodes:\n");
	  neut_debug_nodes (stdout, Nodes);
	  printf ("Mesh2D:\n");
	  neut_debug_mesh  (stdout, Mesh2D);
	  printf ("Mesh3D:\n");
	  neut_debug_mesh  (stdout, Mesh3D);

	  printf ("elt = %d\n", eltnb);
	  printf ("elt3dqty != 1 ( = %d)\n", elt3dqty);
	  ut_array_1d_int_fprintf (stdout, elt3d, elt3dqty, "%d");
	  abort ();
	}

	fprintf (file, "%d", elt3d[0] - 1);

	res = ut_vector_scalprod (n0, n);
	if (Mesh2D.EltOrder == 1)
	{
	  if (res > 0)
	    for (l = 0; l <= 2; l++)
	      fprintf (file, " %d", Mesh2D.EltNodes[eltnb][seqo1[l]] - 1);
	  else
	    for (l = 2; l >= 0; l--)
	      fprintf (file, " %d", Mesh2D.EltNodes[eltnb][seqo1[l]] - 1);
	}
	else if (Mesh2D.EltOrder == 2)
	{
	  if (res > 0)
	  {
	    for (l = 0; l < 6; l++)
	      fprintf (file, " %d", Mesh2D.EltNodes[eltnb][seqo2[l]] - 1);
	  }
	  else
	  {
	    /* reverse order from 0 (must be from a 1st order node) */
	    fprintf (file, " %d", Mesh2D.EltNodes[eltnb][seqo2[0]] - 1);
	    for (l = 5; l >= 1; l--)
	      fprintf (file, " %d", Mesh2D.EltNodes[eltnb][seqo2[l]] - 1);
	  }
	}

	fprintf (file, " \n");
      }
    }
  }

  ut_free_1d (n);
  ut_free_1d (n0);
  ut_free_2d_char (fodnames, 7);
  ut_free_1d_int (elt3d);

  return;
}

void
neut_mesh_fprintf_fev_elsets (FILE * file, struct MESH Mesh3D)
{
  int i;

  fprintf (file, "grain-input\n");
  fprintf (file, "%d %d\n", Mesh3D.EltQty, Mesh3D.ElsetQty);
  
  for (i = 1; i <= Mesh3D.EltQty; i++)
    fprintf (file, "%d 1\n", Mesh3D.EltElset[i]); /* 1 if for the phase */

  fprintf (file, "end-options\n");

  return;
}


void
neut_mesh_fprintf_fev_nsets (FILE* file, char* nset, int** NSets, char** NSetNames)
{
  int i, j;
  char** name;
  int qty, bcqty;

  ut_string_separate (nset, ',', &name, &qty);

  /* first pass, for calculating total number of bcs */
  bcqty = 0;
  for (i = 0; i < qty; i++)
    for (j = 1; j <= 44; j++)
      if (strcmp (name[i], NSetNames[j]) == 0)
      {
	bcqty += NSets[j][0];
	break;
      }

  /* header */
  fprintf (file, "0\n%d\n", bcqty);

  for (i = 0; i < qty; i++)
    for (j = 1; j <= 44; j++)
      if (strcmp (name[i], NSetNames[j]) == 0)
      {
	neut_mesh_fprintf_fev_nset (file, NSetNames[j], NSets[j]);
	break;
      }

  /* foot */
  fprintf (file, "0\n");

  return;
}

/*
void
neut_mesh_bcsfile_update (FILE* filein, FILE* file, struct PART Part)
{
  int i, j, id, newid, tmp, qty, colqty;
  char* string = ut_alloc_1d_char (1000);

  colqty = ut_file_nbwords_pointer (filein) - 3;
  colqty /= tmp;
  colqty--;

  fscanf (filein, "%d\n", &tmp);
  fprintf (file, "%d\n", tmp);
  fscanf (filein, "%d\n", &qty);
  fprintf (file, "%d\n", qty);

  for (i = 0; i < qty; i++)
  {
    fscanf (filein, "%d", &id);
    id++;
    if (Part.qty > 0)
      newid = Part.node_old_new[id];
    else
      newid = id;

    fprintf (file, "%d", newid - 1);

    for (j = 0; j < colqty; j++)
    {
      fscanf (filein, "%s", string);
      fprintf (file, " %s", string);
    }

    fprintf (file, "\n");
  }
  
  fscanf (filein, "%d\n", &tmp);
  fprintf (file, "%d\n", tmp);

  ut_free_1d_char (string);

  return;
}
*/

/*
void
neut_mesh_surffile_update (FILE* filein, FILE* file, struct PART Part)
{
  int i, j, st, eltnodeqty, id, newid, tmp, qty;
  char* string = ut_alloc_1d_char (1000);

  eltnodeqty = ut_file_nbwords_pointer (filein) - 2; 

  fscanf (filein, "%d", &tmp);
  fprintf (file, "%d\n", tmp);

  fscanf (filein, "%d", &qty);
  fprintf (file, "%d\n", qty);

  eltnodeqty /= qty;

  st = 0;
  switch (eltnodeqty)
  {
  case 8:
    st = 1;
    eltnodeqty = 6;
    break;
  case 7:
    st = 0;
    eltnodeqty = 6;
    break;
  case 5:
    st = 1;
    eltnodeqty = 3;
    break;
  case 4:
    st = 0;
    eltnodeqty = 3;
    break;
  default:
    {
      ut_print_message (2, 0, "surf file has a bad format.\n");
      abort ();
    }
  }

  for (i = 0; i < qty; i++)
  {
    fscanf (filein, "%d", &id);
    id++;
    if (Part.qty > 0)
      newid = Part.elts_old_new[id];
    else
      newid = id;

    fprintf (file, "%d", newid - 1); 

    for (j = 0; j < eltnodeqty; j++)
    {
      fscanf (filein, "%d", &id);
      id++;
      if (Part.qty > 0)
	newid = Part.nodes_old_new[id];
      else
	newid = id;
      fprintf (file, " %d", newid - 1);
    }

    if (st == 1)
    {
      fscanf (filein, "%s", string);
      fprintf (file, " %s", string);
    }
    fprintf (file, "\n");
  }

  ut_free_1d_char (string);

  return;
}
*/
