/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_mm.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_mm (struct IN *pIn, struct DNEIGH *pDNeigh)
{
  (*pIn).format = ut_alloc_1d_char (100);
  sprintf ((*pIn).format, "msh");

  (*pDNeigh).morpho = ut_alloc_1d_char (10);
  sprintf ((*pDNeigh).morpho, "poisson");

  (*pIn).ingeo = NULL;
  (*pIn).body = NULL;
  (*pIn).geof = NULL;
  (*pIn).msh = NULL;
  (*pIn).centrecoo = NULL;

  (*pIn).verbosity = 0;
  (*pIn).input = -1;
  (*pIn).nset = ut_alloc_1d_char (1000);
  sprintf ((*pIn).nset, "faces");
  (*pIn).faset = ut_alloc_1d_char (1000);
  sprintf ((*pIn).faset, "faces");

  (*pIn).nsite = 0;
  (*pIn).nsitesize = -1;

  (*pDNeigh).ttype = 0;
  (*pDNeigh).NDensity = -1;
  (*pDNeigh).N = -1;
  (*pDNeigh).nN = 0;
  (*pDNeigh).Id = -1;

  (*pDNeigh).morpho = ut_alloc_1d_char (100);
  sprintf ((*pDNeigh).morpho, "poisson");

  (*pIn).msizetype = 1;
  (*pIn).msize = 20;
  (*pIn).msize3 = ut_alloc_1d_int (3);
  (*pIn).msize3[0] = 20;
  (*pIn).msize3[1] = 20;
  (*pIn).msize3[2] = 20;
  (*pIn).morder = 1;

  return;
}

void
SetOptions_mm (struct IN *pIn, struct DNEIGH *pDNeigh,
	       int argc, char **argv)
{
  int i, ArgQty, Res;
  char **ArgList = ut_alloc_2d_char (101, 101);
  char *Arg = ut_alloc_1d_char (101);

  /* This is the possible argument list. */
  ArgQty = 0;
  // Input data --------------------------------------------------------
  sprintf (ArgList[++ArgQty], "-n");
  sprintf (ArgList[++ArgQty], "-id");
  sprintf (ArgList[++ArgQty], "-centrecoo");
  sprintf (ArgList[++ArgQty], "-centercoo");
  sprintf (ArgList[++ArgQty], "-gcoo");
  sprintf (ArgList[++ArgQty], "-regular");
  // General options ---------------------------------------------------
  sprintf (ArgList[++ArgQty], "-o");
  sprintf (ArgList[++ArgQty], "-ttype");
  sprintf (ArgList[++ArgQty], "-nset");
  sprintf (ArgList[++ArgQty], "-msize");
  sprintf (ArgList[++ArgQty], "-morder");
  sprintf (ArgList[++ArgQty], "-order");
  sprintf (ArgList[++ArgQty], "-v");
  sprintf (ArgList[++ArgQty], "-nsite");
  sprintf (ArgList[++ArgQty], "-nsitewidth");
  sprintf (ArgList[++ArgQty], "-faset");
  sprintf (ArgList[++ArgQty], "-msize3");
  sprintf (ArgList[++ArgQty], "-format");

  for (i = 1; i < argc; i++)
  {
    if (argv[i][0] != '-')
    {
      (*pIn).input = 0;
      (*pIn).ingeo = ut_alloc_1d_char (strlen (argv[i]) + 1);
      sprintf ((*pIn).ingeo, "%s", argv[i]);
    }
    else
    {
      /* Searching option name (string completion stuff) */
      Res = ut_string_comp (argv[i], ArgList, ArgQty, Arg);
      if (Res == 1)
      {
	ut_print_lineheader (2);
	printf ("Several possibilities for option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }
      else if (Res == -1)
      {
	ut_print_lineheader (2);
	printf ("Unknown option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }

      if (strcmp (Arg, "-o") == 0 && i < argc - 1)
      {
	(*pIn).body = ut_arg_nextaschar (argv, &i, Arg);
	ut_string_body ((*pIn).body, (*pIn).body);
      }
      else if (strcmp (Arg, "-n") == 0 && i < argc - 1)
      {
	(*pDNeigh).NDensity = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pIn).input = 0;
      }
      else if (strcmp (Arg, "-id") == 0 && i < argc - 1)
	(*pDNeigh).Id = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      else if (strcmp (Arg, "-centrecoo") == 0
            || strcmp (Arg, "-centercoo") == 0
            || strcmp (Arg,   "-gcoo"   ) == 0)
      {
	if ((*pDNeigh).Id > 0)
	{
	  ut_print_message (2, 0,
			    "Options `-id' and `-centrecoo' are mutually exclusive!");
	  abort ();
	}
	(*pDNeigh).Id = 0;
	if ((*pIn).centrecoo != NULL)
	  ut_free_1d_char ((*pIn).centrecoo);
	(*pIn).centrecoo = ut_arg_nextaschar (argv, &i, Arg);
	sprintf ((*pDNeigh).morpho, "custom");
      }
      else if (strcmp (Arg, "-regular") == 0 && i < argc - 2)
      {
	if ((*pDNeigh).Id > 0)
	{
	  ut_print_message (2, 0,
			    "Options `-id' and `-regular' are mutually exclusive!");
	  abort ();
	}
	(*pDNeigh).Id = 0;
	
	if ((*pDNeigh).morpho != NULL)
	  ut_free_1d_char ((*pDNeigh).morpho);
	(*pDNeigh).morpho = ut_arg_nextaschar (argv, &i, Arg);
	(*pDNeigh).N1d = ut_arg_nextasint (argv, &i, Arg, 0, INT_MAX);
	(*pIn).input = 2;
      }
      else if (strcmp (Arg, "-ttype") == 0 && i < argc - 1)
	(*pDNeigh).ttype = ut_arg_nextasint (argv, &i, Arg, 0, 3);
      else if (strcmp (Arg, "-format") == 0 && i < argc - 1)
	(*pIn).format = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-nset") == 0 && i < argc - 1)
	(*pIn).nset = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-faset") == 0 && i < argc - 1)
	(*pIn).faset = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-nsite") == 0 && i < argc - 1)
	(*pIn).nsite = ut_arg_nextasint (argv, &i, Arg, 0, 4);
      else if (strcmp (Arg, "-nsitewidth") == 0 && i < argc - 1)
	(*pIn).nsitesize = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
      else if (strcmp (Arg, "-msize") == 0 && i < argc - 1)
      {
	(*pIn).msizetype = 1;
	(*pIn).msize = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      }
      else if (strcmp (Arg, "-msize3") == 0 && i < argc - 1)
      {
	(*pIn).msizetype = 3;
	(*pIn).msize3[0] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pIn).msize3[1] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pIn).msize3[2] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      }
      else if (strcmp (Arg, "-morder") == 0 && i < argc - 1)
	(*pIn).morder = ut_arg_nextasint (argv, &i, Arg, 1, 2);
      else if (strcmp (Arg, "-order") == 0 && i < argc - 1)
	(*pIn).morder = ut_arg_nextasint (argv, &i, Arg, 1, 2);
      else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	(*pIn).verbosity = ut_arg_nextasint (argv, &i, Arg, 0, 2);
      else
      {
	ut_arg_badarg ();
	abort ();
      }
    }
  }

  ut_free_2d_char (ArgList, 101);
  ut_free_1d_char (Arg);

  return;
}
