/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "RMeshing.h"

void
RMeshing (struct NODES *pNodes, struct MESH *pMesh,
	  int ***pFoDNodes, struct IN In, struct DNEIGH *pDNeigh)
{
  int i;
  struct GEO Geo;
  FILE *in = NULL;
  // char *expandnset = NULL;
  // char *expandfaset = NULL;
  int* msize = ut_alloc_1d_int (3);

  neut_geo_set_zero (&Geo);

  /* Loading tessellation ------------------------------------------ */

  if (In.ingeo != NULL)
  {
    ut_print_message (0, 2, "Loading tessellation ...\n");

    in = ut_file_open (In.ingeo, "r");
    neut_geo_fscanf (in, &Geo);
    ut_file_close (in, In.ingeo, "r");
  }

  if (In.ingeo != NULL)
    nemm_in_msize (In, Geo.DomainSize, msize);
  else
  {
    double** dsize = ut_alloc_2d (3, 2);
    dsize[0][1] = dsize[1][1] = dsize[2][1] = 1;
    nemm_in_msize (In, dsize, msize);
    ut_free_2d (dsize, 3);
  }

  // neut_utils_nset_expand (In.nset, &expandnset);
  // neut_utils_nset_expand (In.faset, &expandfaset);

  /* Creating mapped mesh ------------------------------------------ */

  ut_print_message (0, 2, "Generating mesh ...\n");
  BuildMMesh3D (msize, In.morder, pNodes, pMesh, pFoDNodes);

  /* Searching elsets ---------------------------------------------- */

  if (In.ingeo != NULL)
  {
    ut_print_message (0, 2, "Searching elsets ... ");

    neut_nodes_scale (pNodes, Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
		      Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
		      Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    neut_nodes_shift (pNodes, Geo.DomainSize[0][0],
		      Geo.DomainSize[1][0], Geo.DomainSize[2][0]);
    neut_mesh_scale (pMesh, (*pMesh).msize, Geo.DomainSize[0][1] - Geo.DomainSize[0][0],
		      Geo.DomainSize[1][1] - Geo.DomainSize[1][0],
		      Geo.DomainSize[2][1] - Geo.DomainSize[2][0]);
    neut_mesh_shift (pMesh, (*pMesh).msize, Geo.DomainSize[0][0],
		      Geo.DomainSize[1][0], Geo.DomainSize[2][0]);

    RMeshingFGeo (Geo, pMesh, pDNeigh);

    (*pMesh).ElsetQty = Geo.PolyQty;
  }
  else
  {
    ut_print_message (0, 2, "Searching elsets ...\n");

    RMeshingFNId (In, pMesh, pDNeigh);

    if ((*pDNeigh).ttype == 0 || (*pDNeigh).ttype == 1)
      (*pMesh).ElsetQty = (*pDNeigh).N;
    else if ((*pDNeigh).ttype == 2 || (*pDNeigh).ttype == 3)
      (*pMesh).ElsetQty = (*pDNeigh).N + (*pDNeigh).nN;
    else
      abort ();
  }

  /* Calculation of Elsets (they are sorted) */
  int elset;
  (*pMesh).Elsets = ut_alloc_2d_int ((*pMesh).ElsetQty + 1, 1); 
  for (i = 1; i <= (*pMesh).EltQty; i++)
  {
    elset = (*pMesh).EltElset[i];
    if (elset > (*pMesh).ElsetQty)
    {
      fprintf (stderr, "elset = %d > elsetqty = %d\n", elset, (*pMesh).ElsetQty);
      ut_error_reportbug ();
    }
    (*pMesh).Elsets[elset][0]++;
    (*pMesh).Elsets[elset] =
      ut_realloc_1d_int ((*pMesh).Elsets[elset], (*pMesh).Elsets[elset][0] + 1);
    (*pMesh).Elsets[elset][(*pMesh).Elsets[elset][0]] = i;
  }

  // ut_free_1d_char (expandnset);
  // ut_free_1d_char (expandfaset);
  ut_free_1d_int (msize);

  return;
}

void
nemm_in_msize (struct IN In, double** domainsize, int* msize)
{
  int i;
  double* dsize = ut_alloc_1d (3);
  double avdsize;

  if (In.msizetype == 1)
  {
    for (i = 0; i < 3; i++)
      dsize[i] = domainsize[i][1] - domainsize[i][0];
    avdsize = pow (dsize[0] * dsize[1] * dsize[2], 0.3333333333);

    for (i = 0; i < 3; i++)
      msize[i] = ut_num_d2ri (In.msize * dsize[i] / avdsize);
  }
  else
    ut_array_1d_int_memcpy (msize, 3, In.msize3);

  ut_free_1d (dsize);

  return;
}
