/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#define frand() ((double) rand() / (RAND_MAX+1.0))

#include"DnGDistrib.h"

void
DnGDistrib (struct IN In, struct GERMSET *pGermSet, struct POLY *pDomain,
	    double *size)
{
  int i, j;
  double eps; 
  double* r = ol_r_alloc ();

  /* Building the domain */
  CubeDomain (size, pDomain);

  /* Distributing the germs within the domain */
  if (strcmp ((*pGermSet).morpho, "custom") == 0)
    DistribFromFile (In.centrecoo, (*pGermSet).N, &((*pGermSet).GermsCoo));
  else if (strcmp ((*pGermSet).morpho, "tocta") == 0)
    TruOctaDistrib (pGermSet, size);
  else // (strcmp ((*pGermSet).morpho, "poisson") == 0)
    RandDistrib (pGermSet, size);

  if ((*pGermSet).randomize > 0)
  {
    gsl_rng *rr;
    rr = gsl_rng_alloc (gsl_rng_ranlxd2);
    gsl_rng_set (rr, (*pGermSet).randomize2);

    for (i = 1; i <= (*pGermSet).N; i++)
    {
      ol_nb_max_rtheta_mis (gsl_rng_uniform (rr), gsl_rng_uniform (rr),
	         gsl_rng_uniform (rr), (*pGermSet).randomize, r, &eps);
      for (j = 1; j <= 3; j++)
      {
	(*pGermSet).GermsCoo[i][j] += eps * r[j - 1] * (size[2 * j] - size[2 * j - 1]);
	if ((*pGermSet).GermsCoo[i][j] < size[2 * j - 1])
	  (*pGermSet).GermsCoo[i][j] = size[2 * j - 1] + 1e-6;
	if ((*pGermSet).GermsCoo[i][j] > size[2 * j])
	  (*pGermSet).GermsCoo[i][j] = size[2 * j] - 1e-6;
      }
    }
  }

  return;
}
