/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"InputData_t.h"

/* SetDefaultOptions set the options to their default values */
void
SetDefaultOptions_t (struct IN *pIn, struct GERMSET *pGermSet)
{
  (*pIn).libpath = NULL;

  (*pGermSet).morpho = ut_alloc_1d_char (10);
  sprintf ((*pGermSet).morpho, "poisson");

  (*pIn).size = ut_alloc_1d (7);	/* size of the domain to tesselate */

  (*pIn).size[1] = 0;
  (*pIn).size[2] = 1;
  (*pIn).size[3] = 0;
  (*pIn).size[4] = 1;
  (*pIn).size[5] = 0;
  (*pIn).size[6] = 1;

  (*pIn).F = ut_alloc_1d (3);	/* size of the domain to tesselate */

  (*pIn).F[0] = 1;
  (*pIn).F[1] = 1;
  (*pIn).F[2] = 1;

  (*pIn).verbosity = 0;
  (*pIn).body = NULL;

  (*pGermSet).N = -1;
  (*pGermSet).Id = -1;
  (*pGermSet).ttype = 0;
  (*pGermSet).randomize = 0;
  (*pGermSet).randomize2 = 0;

  (*pIn).input = -1;
  (*pIn).geo = NULL;
  (*pIn).format = ut_alloc_1d_char (100);
  sprintf ((*pIn).format, "tess,oin");
  (*pIn).load = NULL;
  (*pIn).centrecoo = ut_alloc_1d_char (100);
  (*pIn).stt0 = ut_alloc_1d_char (100);
  (*pIn).stt3 = ut_alloc_1d_char (100);

  (*pIn).printpointpoly = 0;
  (*pIn).printneighbour = 0;
  (*pIn).point = ut_alloc_1d_char (100);
  (*pIn).polyid = ut_alloc_1d_char (100);
  
  (*pIn).stattess = 0;
  (*pIn).sorttess_qty = 0;

  return;
}

void
SetOptions_t (struct IN *pIn, struct GERMSET *pGermSet,
	      int argc, char **argv)
{
  int i, ArgQty, Res;
  char **ArgList = ut_alloc_2d_char (101, 101);
  char *Arg = ut_alloc_1d_char (101);
  char* load_type = NULL;

  // Argument list =====================================================

  ArgQty = 0;
  // Input data --------------------------------------------------------
  sprintf (ArgList[++ArgQty], "-n");
  sprintf (ArgList[++ArgQty], "-id");
  sprintf (ArgList[++ArgQty], "-morpho");
  sprintf (ArgList[++ArgQty], "-centrecoo");
  sprintf (ArgList[++ArgQty], "-centercoo");
  sprintf (ArgList[++ArgQty], "-gcoo");
  sprintf (ArgList[++ArgQty], "-regular");
  // General options ---------------------------------------------------
  sprintf (ArgList[++ArgQty], "-o");
  sprintf (ArgList[++ArgQty], "-v");
  // Tessellation options ----------------------------------------------
  sprintf (ArgList[++ArgQty], "-dsize");
  sprintf (ArgList[++ArgQty], "-scale");
  sprintf (ArgList[++ArgQty], "-randomize");
  // Output options ----------------------------------------------------
  sprintf (ArgList[++ArgQty], "-format");
  // Post-processing ---------------------------------------------------
  sprintf (ArgList[++ArgQty], "-stat");
  sprintf (ArgList[++ArgQty], "-stattess");
  sprintf (ArgList[++ArgQty], "-sort");
  sprintf (ArgList[++ArgQty], "-sorttess");
  sprintf (ArgList[++ArgQty], "-pointpoly");
  sprintf (ArgList[++ArgQty], "-neighbour");
  sprintf (ArgList[++ArgQty], "-neighbor");
  // Restart a job -----------------------------------------------------
  sprintf (ArgList[++ArgQty], "-load");

  // Treating arguments ================================================
  
  for (i = 1; i < argc; i++)
  {
    if (argv[i][0] != '-')
      ut_arg_badarg ();
    else
    {
      /* Searching option name (string completion stuff) */
      Res = ut_string_comp (argv[i], ArgList, ArgQty, Arg);
      if (Res == 1)
      {
	ut_print_lineheader (2);
	printf ("Several possibilities for option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }
      else if (Res == -1)
      {
	ut_print_lineheader (2);
	printf ("Unknown option `%s'.\n", argv[i]);
	ut_arg_badarg ();
      }

/*---------------------------------------------------------------------- 
 * input data */
      else if (strcmp (Arg, "-n") == 0 && i < argc - 1)
      {
	if ((*pIn).input == -1)
	{
	  (*pIn).input = 0;
	  (*pGermSet).N = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	  
	  if ((strcmp ((*pGermSet).morpho, "cube"  ) == 0)
	   || (strcmp ((*pGermSet).morpho, "dodeca") == 0)
	   || (strcmp ((*pGermSet).morpho, "tocta" ) == 0))
	  {
	    (*pGermSet).N1d = (*pGermSet).N;
	    (*pGermSet).N   = -1;
	    (*pIn).input = 2;
	  }
	}
	else
	  ut_arg_badarg ();
      }
      else if (strcmp (Arg, "-id") == 0 && i < argc - 1)
      {
	if ((*pGermSet).Id == 0)
	{
	  ut_print_message (2, 0,
			    "Functions `-id' and `-centrecoo' are mutually exclusive!");
	  abort ();
	}
	(*pGermSet).Id = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      }
      else if (strcmp (Arg, "-centrecoo") == 0
            || strcmp (Arg, "-centercoo") == 0
            || strcmp (Arg,   "-gcoo"   ) == 0)
      {
	if ((*pGermSet).Id > 0)
	{
	  ut_print_message (2, 0,
			    "Options `-id' and `-centrecoo' are mutually exclusive!");
	  abort ();
	}
	(*pGermSet).Id = 0;
	if ((*pIn).centrecoo != NULL)
	  ut_free_1d_char ((*pIn).centrecoo);
	(*pIn).centrecoo = ut_arg_nextaschar (argv, &i, Arg);
	sprintf ((*pGermSet).morpho, "custom");
      }
      else if (strcmp (Arg, "-morpho") == 0)
      {
	(*pGermSet).morpho = ut_arg_nextaschar (argv, &i, Arg);
	if ((*pGermSet).N > 0)
	  if ((strcmp ((*pGermSet).morpho, "cube"  ) == 0)
	   || (strcmp ((*pGermSet).morpho, "dodeca") == 0)
	   || (strcmp ((*pGermSet).morpho, "tocta" ) == 0))
	  {
	    (*pGermSet).N1d = (*pGermSet).N;
	    (*pGermSet).N   = -1;
	    (*pIn).input    =  2;
	  }
      }
      else if (strcmp (Arg, "-regular") == 0 && i < argc - 2)
      {
	ut_print_messagewnc (1, 72, "Option -regular is deprecated and will be removed from future neper versions.  Use -morpho instead.");
	if ((*pGermSet).morpho != NULL)
	  ut_free_1d_char ((*pGermSet).morpho);
	(*pGermSet).morpho = ut_arg_nextaschar (argv, &i, Arg);
	(*pGermSet).N1d = ut_arg_nextasint (argv, &i, Arg, 0, INT_MAX);
	(*pIn).input = 2;
      }
/*---------------------------------------------------------------------- 
 * general options */
      else if (strcmp (Arg, "-o") == 0 && i < argc - 1)
      {
	(*pIn).body = ut_arg_nextaschar (argv, &i, Arg);
	ut_string_body ((*pIn).body, (*pIn).body);
      }
      else if (strcmp (Arg, "-v") == 0 && i < argc - 1)
	(*pIn).verbosity = ut_arg_nextasint (argv, &i, Arg, 0, 1);

/*---------------------------------------------------------------------- 
 * tessellation options */
      else if (strcmp (Arg, "-dsize") == 0 && i < argc - 1)
      {
	(*pIn).size[2] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
	(*pIn).size[4] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
	(*pIn).size[6] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
      }
      else if (strcmp (Arg, "-scale") == 0 && i < argc - 1)
      {
	(*pIn).F[0] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
	(*pIn).F[1] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
	(*pIn).F[2] = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
      }
      else if (strcmp (Arg, "-randomize") == 0 && i < argc - 2)
      {
	(*pGermSet).randomize = ut_arg_nextasreal (argv, &i, Arg, 0, DBL_MAX);
	(*pGermSet).randomize2 = ut_arg_nextasint (argv, &i, Arg, 0, INT_MAX);
      }

/*---------------------------------------------------------------------- 
 * post-processing options */
      else if ((strcmp (Arg, "-neighbour") == 0 && i < argc - 1)
            || (strcmp (Arg, "-neighbor")  == 0 && i < argc - 1))
	(*pIn).printneighbour = ut_arg_nextasint (argv, &i, Arg, 0, 1);
      else if (strcmp (Arg, "-pointpoly") == 0 && i < argc - 1)
      {
	(*pIn).printpointpoly = 1;
	(*pIn).point = ut_arg_nextaschar (argv, &i, Arg);
      }
      /*
      else if (strcmp (Arg, "-print") == 0 && i < argc - 1)
      {
	(*pIn).printentity = ut_arg_nextaschar (argv, &i, Arg);
	tmp = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
	(*pIn).printlist = ut_alloc_1d_int (tmp + 1);
	(*pIn).printlist[0] = tmp;
	for (j = 1; j <= (*pIn).printlist[0]; j++)
	  (*pIn).printlist[j] = ut_arg_nextasint (argv, &i, Arg, 1, INT_MAX);
      }
      else if (strcmp (Arg, "-printformat") == 0 && i < argc - 1)
	(*pIn).printformat = ut_arg_nextaschar (argv, &i, Arg);
      */
      else if (strcmp (Arg, "-format") == 0 && i < argc - 1)
	(*pIn).format = ut_arg_nextaschar (argv, &i, Arg);
      else if (strcmp (Arg, "-stattess") == 0 || strcmp (Arg, "-stat") == 0)
	(*pIn).stattess = ut_arg_nextasint (argv, &i, Arg, 0, 1);
      else if ((strcmp (Arg, "-sorttess") == 0 ||
	        strcmp (Arg, "-sort") == 0   ) && i < argc - 2) 
      {
	(*pIn).sorttess_qty = 1;
	(*pIn).sorttess = ut_alloc_1d_pchar (2);
	(*pIn).sorttess[0] = ut_arg_nextaschar (argv, &i, Arg);
	(*pIn).sorttess[1] = ut_arg_nextaschar (argv, &i, Arg);
      }

/*---------------------------------------------------------------------- 
 * load capability */
      else if (strcmp (Arg, "-load") == 0 && i < argc - 2)
      {
	load_type = ut_alloc_1d_char (strlen (argv[i + 1]));
	load_type = ut_arg_nextaschar (argv, &i, Arg);
	if (strcmp (load_type, "tess") == 0)
	  (*pIn).input = 1;
	else
	{
	  ut_print_message (2, 0, "Unknown load type.\n");
	  abort ();
	}
	ut_free_1d_char (load_type);
	(*pIn).load = ut_alloc_1d_char (strlen (argv[i + 1]));
	(*pIn).load = ut_arg_nextaschar (argv, &i, Arg);
      }
      else
      {
	ut_arg_badarg ();
	abort ();
      }
    }
  }

  ut_free_1d_char (Arg);
  ut_free_2d_char (ArgList, 101);

  return;
}
