/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neper_t.h"

int
neper_t (int fargc, char **fargv, int argc, char **argv)
{
/* *********************************************************
 * STRUCTURE AND VARIABLE DEFINITIONS  */
  /* See the structure files for details. */
  FILE *file;
  struct INTEPARA IntePara;
  struct POLY Domain;
  struct GERMSET GermSet;
  struct POLYMOD Polymod;
  struct POLY *Poly;
  struct TESS Tess;
  struct GEO Geo;
  struct IN In;
  int* perm = NULL;

  /* initialization */
  neut_geo_set_zero (&Geo);
  neut_tess_set_zero (&Tess);

  ut_print_moduleheader ("-T", fargc, fargv, argc, argv);

/* *********************************************************
 * PARAMETER, INPUT DATA AND VARIABLES INITIALIZATION  */
  /* InputData initializes all the parameters, input data and related
   * variables. */
  ut_print_message (0, 1, "Reading input data ...\n");
  InputData_t (&In, &GermSet, fargc, fargv, argc, argv);

  // Input = n,id for tessellation, or regular / tocta
  if (In.input == 0
  || (In.input == 2 && strcmp (GermSet.morpho, "tocta") == 0))
  {
    /* IniInternalPara initializes the internal parameters -those
     * of the IntePara structure. */
    InternalPara (&IntePara);

    ut_print_message (0, 1, "Creating tessellation ...\n");

    /* *********************************************************
     * STUDY OF THE GERM DISTRIBUTION  */
    /* GermsNei analyses the neighbourhood of every germ: neighbour
     * numbers and distances. */
    DnGDistrib (In, &GermSet, &Domain, In.size);

    /* *********************************************************
     * POLYEDRA DETERMINATION  */
    /* PolyComp decomposes the cube into the set of Voronoi polyhedra
     * and records them into Poly (which is allocated in the routine). */
    PolyComp (&IntePara, Domain, GermSet, &Polymod, &Poly, In.verbosity);

    /* *********************************************************
     * TESSELATION CONSTRUCTION  */
    /* Tessellation lies the polyhedra by suppressing vertices and faces
     * defined several times. That results in the Tess structure, which
     * contains all the information about the Voronoi tessellation. */
    Tessellation (GermSet, Poly, &Tess);
  }
  // Importing tessellation
  else if (In.input == 1)
  {
    ut_print_message (0, 1, "Importing tessellation ...\n");
    file = ut_file_open (In.load, "r");
    neut_geo_fscanf (file, &Geo);
    ut_file_close (file, In.load, "r");
  }

  if (ut_num_equal (In.F[0], 1, 1e-6) == 0
   || ut_num_equal (In.F[1], 1, 1e-6) == 0
   || ut_num_equal (In.F[2], 1, 1e-6) == 0)
  {
    if (Tess.PolyQty != 0)
    {
      ut_print_message (0, 1, "Deforming tessellation ...\n");
      neut_tess_deform (&Tess, In.F);
    }
    if (Geo.PolyQty != 0)
    {
      ut_print_message (0, 1, "Deforming geometry ...\n");
      neut_geo_scale (&Geo, In.F[0], In.F[1], In.F[2]);
    }
  }
  
  if (Geo.PolyQty == 0)
    neut_tess_geo (Tess, &Geo);

  // from here, only geo.
  
  if (In.sorttess_qty > 0)
  {
    ut_print_message (0, 1, "Sorting tessellation ...\n");
    net_geo_sort (&Geo, In.sorttess[0], In.sorttess[1], &perm);
  }

  /* PolyFree(Poly,GermSet.N); */

/* *********************************************************
 * RESULTS WRITING  */
  ut_print_message (0, 1, "Writing results ...\n");
  neut_geo_init_boundingbox (&Geo);
  Res_t (In, GermSet, Tess, Geo, perm);

  // net_in_free (In);
  // net_germset_free (GermSet);

  return 0;
}
