/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neper_vs.h"

int
neper_vs (int fargc, char **fargv, int argc, char **argv)
{
  int i;
  struct PRINT Print;
  struct GEO Geo;
  struct NODES Nodes;
  struct MESH Mesh0D;
  struct MESH Mesh1D;
  struct MESH Mesh2D;
  struct MESH Mesh3D;
  struct DATA Data;
  int expargc;
  char** expargv = NULL;

  ut_arg_expand (argc, argv, &expargc, &expargv);

  neut_print_set_default (&Print);
  neut_geo_set_zero   (&Geo);
  neut_nodes_set_zero (&Nodes);
  neut_mesh_set_zero  (&Mesh0D);
  neut_mesh_set_zero  (&Mesh1D);
  neut_mesh_set_zero  (&Mesh2D);
  neut_mesh_set_zero  (&Mesh3D);
  neut_data_set_default (&Data);

  ut_print_moduleheader ("-VS", fargc, fargv, argc, argv);

  for (i = 1; i < expargc; i++)
  {
    // data loading ----------------------------------------------------
    if (strncmp (expargv[i], "-load", 5) == 0)
    {
      nevs_load (expargv, &i, &Nodes, &Mesh0D, &Mesh1D, &Mesh2D, &Mesh3D,
	  &Geo);

      if (strcmp (expargv[i-1], "-loadmesh") == 0)
      {
	neut_mesh_volume (Nodes, Mesh3D, &(Print.eltedgerad), NULL, NULL);
	Print.elt1dedgerad = Print.eltedgerad;
	Print.eltedgerad /= Mesh3D.EltQty;
	Print.eltedgerad = pow (Print.eltedgerad, 0.3333333333333) * 0.01;
	Print.elt1dedgerad /= Geo.PolyQty;
	Print.elt1dedgerad = pow (Print.elt1dedgerad, 0.3333333333333) * 0.01;
      }
      
      if (strcmp (expargv[i-1], "-loadtess") == 0)
      {
	neut_geo_volume (Geo, &(Print.edgerad));
	Print.edgerad /= Geo.PolyQty;
	Print.edgerad = pow (Print.edgerad, 0.3333333333333) * 0.01;
      }
    }

    // show settings ---------------------------------------------------
    else if (strncmp (expargv[i], "-show", 5) == 0)
    {
      nevs_show (expargv, &i, Geo, Nodes, Mesh0D, Mesh1D, Mesh2D, Mesh3D, &Print);
    }

    // data settings ---------------------------------------------------
    else if (strncmp (expargv[i], "-data", 5) == 0)
    {
      nevs_data (expargv, &i,  Geo, Nodes, Mesh3D, &Data);
      if (Data.nodecoo != NULL)
	ut_array_2d_memcpy (Nodes.NodeCoo + 1, Nodes.NodeQty, 3, Data.nodecoo + 1);
    }

    // camera settings -------------------------------------------------
    else if (strncmp (expargv[i], "-camera", 7) == 0)
      nevs_camera (expargv, &i, Geo, Nodes, Mesh3D, &Print);
    
    // image settings --------------------------------------------------
    else if (strncmp (expargv[i], "-image", 6) == 0)
      nevs_image (expargv, &i, &Print);


    // image printing --------------------------------------------------
    else if (strncmp (expargv[i], "-print", 6) == 0)
    {
      nevs_show_init (Geo, Nodes, Mesh0D, Mesh1D, Mesh2D, Mesh3D, &Print);
      nevs_camera_init (Geo, Nodes, Mesh3D, &Print);
      nevs_print (expargv, &i, &Print, Data, &Geo, &Nodes, Mesh0D, Mesh1D,
	  Mesh2D, Mesh3D);
    }

    // process image(s) ------------------------------------------------
    else if (strncmp (expargv[i], "-processimage", 13) == 0)
      nevs_processimage (expargv, &i);

    else
    {
      ut_print_message (2, 0, "Unknown option %s\n", expargv[i]);
      abort ();
    }
  }

  neut_print_free (&Print);
  neut_geo_free (&Geo);
  neut_nodes_free (&Nodes);
  neut_mesh_free (&Mesh0D);
  neut_mesh_free (&Mesh1D);
  neut_mesh_free (&Mesh2D);
  neut_mesh_free (&Mesh3D);

  return 0;
}
