/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_data.h"

void
nevs_data (char** argv, int* pi, struct GEO Geo, struct NODES Nodes, struct MESH Mesh, struct DATA* pData)
{
  int i, j, nbl;
  FILE* file;

  // load node coordinates ---------------------------------------------
  if (strcmp (argv[(*pi)], "-datanodecoo") == 0)
  {
    (*pi)++;
    if ((*pData).nodecoo == NULL)
      (*pData).nodecoo = ut_alloc_2d (Nodes.NodeQty + 1, 3);
    nbl = ut_file_nbwords (argv[(*pi)]) / 3;
    if (nbl != Nodes.NodeQty)
    {
      ut_print_message (2, 0, "-datanodecoo: number of data != number of nodes");
      abort ();
    }

    file = ut_file_open (argv[(*pi)], "r");
    ut_array_2d_fscanf (file, (*pData).nodecoo + 1, Nodes.NodeQty, 3);
    ut_file_close (file, argv[(*pi)], "r");
  }

  // load element colours ----------------------------------------------
  else if (strcmp (argv[(*pi)], "-dataeltcolour") == 0 
        || strcmp (argv[(*pi)], "-dataeltcolor") == 0)
  {
    (*pi)++;
    if ((*pData).eltcolour == NULL)
      (*pData).eltcolour = ut_alloc_2d_int (Mesh.EltQty + 1, 3);

    nbl = ut_file_nbwords (argv[(*pi)]) / 3;
    if (nbl != Mesh.EltQty)
    {
      ut_print_message (2, 0, "-dataeltcolour: number of data != number of elements");
      abort ();
    }

    file = ut_file_open (argv[(*pi)], "r");
    ut_array_2d_int_fscanf (file, (*pData).eltcolour + 1, Mesh.EltQty, 3);
    ut_file_close (file, argv[(*pi)], "r");
  }

  // load element set colours ------------------------------------------
  else if (strcmp (argv[(*pi)], "-datapolycolour")  == 0
        || strcmp (argv[(*pi)], "-datapolycolor" )  == 0
        || strcmp (argv[(*pi)], "-dataelsetcolour") == 0
        || strcmp (argv[(*pi)], "-dataelsetcolor" ) == 0)
  {
    int* rgb = ut_alloc_1d_int (3);
    if ((*pData).polycolour == NULL)
      (*pData).polycolour = ut_alloc_2d_int (Geo.PolyQty + 1, 3);

    (*pi)++;
    
    nbl = ut_file_nbwords (argv[(*pi)]) / 3;
    if (nbl != Geo.PolyQty)
    {
      ut_print_message (2, 0, "-data[poly,elset]colour: number of data != number of polys");
      abort ();
    }

    file = ut_file_open (argv[(*pi)], "r");
    for (i = 1; i <= nbl; i++)
      ut_array_1d_int_fscanf (file, (*pData).polycolour[i], 3);
    ut_file_close (file, argv[(*pi)], "r");

    if ((*pData).eltcolour == NULL && Mesh.EltQty > 0)
      (*pData).eltcolour = ut_alloc_2d_int (Mesh.EltQty + 1, 3);
    for (i = 1; i <= Mesh.ElsetQty; i++)
      for (j = 1; j <= Mesh.Elsets[i][0]; j++)
	ut_array_1d_int_memcpy ((*pData).eltcolour[Mesh.Elsets[i][j]],
				3, (*pData).polycolour[i]);

    ut_free_1d_int (rgb);
  }

  // load edge colours -------------------------------------------------
  else if (strcmp (argv[(*pi)], "-dataedgecolour") == 0
        || strcmp (argv[(*pi)], "-dataedgecolor") == 0)
  {
    int* rgb = ut_alloc_1d_int (3);
    if ((*pData).edgecolour == NULL)
      (*pData).edgecolour = ut_alloc_2d_int (Geo.EdgeQty + 1, 3);

    (*pi)++;
    
    nbl = ut_file_nbwords (argv[(*pi)]) / 3;
    if (nbl != Geo.EdgeQty)
    {
      ut_print_message (2, 0, "-dataedgecolour: number of data != number of edges\n");
      printf ("number of data = %d\n", nbl);
      printf ("number of edges = %d\n", Geo.EdgeQty);
      abort ();
    }

    file = ut_file_open (argv[(*pi)], "r");
    for (i = 1; i <= nbl; i++)
      ut_array_1d_int_fscanf (file, (*pData).edgecolour[i], 3);
    ut_file_close (file, argv[(*pi)], "r");

    ut_free_1d_int (rgb);
  }
  
  // error message -----------------------------------------------------
  else
  {
    ut_print_message (2, 0, "Unknown command %s\n", argv[(*pi)]);
    abort ();
  }

  return;
}
