/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_print.h"

void
nevs_print_init (struct PRINT* pPrint, struct DATA Data)
{
  if ((*pPrint).showelt[0] > 0)
  {
    if (Data.eltcolour == NULL)
    {
      ut_print_message (2, 1, "Element colours not set.\n");
      abort ();
    }
  }
  else if ((*pPrint).showpoly[0] > 0 && Data.polycolour == NULL)
  {
    ut_print_message (2, 1, "Poly colours not set.\n");
    abort ();
  }

  return;
}

void
nevs_print (char** expargv, int* pi,
            struct PRINT* pPrint,
            struct DATA Data,
            struct GEO* pGeo,   struct NODES* pNodes,
            struct MESH Mesh0D, struct MESH Mesh1D,
            struct MESH Mesh2D, struct MESH Mesh3D)
{
  double factor;
  char* outfilename = ut_alloc_1d_char (1000);

  if (strcmp (expargv[(*pi)], "-print") != 0)
  {
    if (strcmp (expargv[(*pi)], "-printedgerad") == 0)
    {
      (*pi)++;
      if (strncmp (expargv[(*pi)], "*=", 2) == 0)
      {
	sscanf (expargv[(*pi)] + 2, "%lf", &factor);
	(*pPrint).edgerad *= factor;
      }
      else
	sscanf (expargv[(*pi)], "%lf", &((*pPrint).edgerad));
    }
    else if (strcmp (expargv[(*pi)], "-printeltedgerad") == 0)
    {
      (*pi)++;
      if (strncmp (expargv[(*pi)], "*=", 2) == 0)
      {
	sscanf (expargv[(*pi)] + 2, "%lf", &factor);
	(*pPrint).eltedgerad *= factor;
      }
      else
	sscanf (expargv[(*pi)], "%lf", &((*pPrint).eltedgerad));
    }
    else if (strcmp (expargv[(*pi)], "-printelt1dedgerad") == 0)
    {
      (*pi)++;
      if (strncmp (expargv[(*pi)], "*=", 2) == 0)
      {
	sscanf (expargv[(*pi)] + 2, "%lf", &factor);
	(*pPrint).elt1dedgerad *= factor;
      }
      else
	sscanf (expargv[(*pi)], "%lf", &((*pPrint).elt1dedgerad));
    }
  }
  else
  {
    nevs_print_init (pPrint, Data);

    (*pi)++;
    FILE* file = NULL;
    char* filename = ut_alloc_1d_char (strlen (expargv[(*pi)]) + 10);
    char* command = ut_alloc_1d_char (1000);
      
    sprintf (filename, "%s.pov", expargv[(*pi)]);
    sprintf (outfilename, "%s.png", expargv[(*pi)]);

    // tessellation ------------------------------------------------------
    if ((*pPrint).showedge[0] > 0 || (*pPrint).showpoly[0] > 0)
    {
      ut_print_message (0, 1, "Printing tessellation ...\n");
      file = ut_file_open (filename, "w");
      nevs_print_header (file, (*pPrint));
      nevs_print_tess (file, (*pPrint), Data, *pGeo);
    }

    // mesh --------------------------------------------------------------
    if ((*pPrint).showelt[0]   > 0 || (*pPrint).showelt2d[0] > 0
     || (*pPrint).showelt1d[0] > 0 || (*pPrint).showelt0d[0] > 0)
    {
      ut_print_message (0, 1, "Printing mesh ...\n");
      if (file == NULL)
      {
	file = ut_file_open (filename, "w");
	nevs_print_header (file, (*pPrint));
      }
      nevs_print_mesh (file, (*pPrint), Data, (*pNodes), Mesh0D,
	               Mesh1D, Mesh2D, Mesh3D);
    }

    // foot --------------------------------------------------------------

    nevs_print_foot (file);

    ut_file_close (file, filename, "w");

    if (ut_string_inlist ((*pPrint).format, ',', "png"))
    {
      ut_print_message (0, 2, "Generating png file (%dx%d pixels)...\n",
	  (*pPrint).imagewidth, (*pPrint).imageheight);
      ut_file_openmessage (outfilename, "w");
      if ((*pPrint).imageantialias > 0)
	sprintf (command, "povray +I%s +O%s +W%d +H%d -D +A0.%d 2>/dev/null", filename, outfilename, (*pPrint).imagewidth, (*pPrint).imageheight, (*pPrint).imageantialias);
      else
	sprintf (command, "povray +I%s +O%s +W%d +H%d -D 2>/dev/null", filename, outfilename, (*pPrint).imagewidth, (*pPrint).imageheight);
      system (command);
      ut_file_closemessage (outfilename, "w");
    }

    if (ut_string_inlist ((*pPrint).format, ',', "pov") == 0)
      remove (filename);

    ut_free_1d_char (command);
    ut_free_1d_char (filename);
  }

  ut_free_1d_char (outfilename);

  return;
}
