/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_print_mesh.h"

void
nevs_print_mesh (FILE* file, struct PRINT Print, struct DATA Data,
    struct NODES Nodes, struct MESH Mesh0D, struct MESH
    Mesh1D, struct MESH Mesh2D, struct MESH Mesh3D)
{
  int i, j, k, elt3d, elt2dqty, eltinv;
  double *pos = ut_alloc_1d (3);
  int* elts = NULL;
  int elt_qty = 0;
  int** rgbs = NULL;
  int* elt2delts3d = ut_alloc_1d_int (2);
  int* eltsinv = NULL;
  int elt_max;
  int* nodes = ut_alloc_1d_int (3);
  int* elts2d = ut_alloc_1d_int (1000);
  int* elts3d = ut_alloc_1d_int (1000);
  int elt3dqty;
  int* printelt2d = ut_alloc_1d_int (Mesh2D.EltQty + 1);
  int* printelt3d = ut_alloc_1d_int (Mesh3D.EltQty + 1);
  int** printelt3dface = ut_alloc_2d_int (Mesh3D.EltQty + 1, 4);
  int* tmp = ut_alloc_1d_int (10);
  int** seq2 = ut_alloc_2d_int (3, 2);
  seq2[0][0] = 0; 
  seq2[0][1] = 1; 
  seq2[1][0] = 1; 
  seq2[1][1] = 2; 
  seq2[2][0] = 0; 
  seq2[2][1] = 2; 
  int** seq = ut_alloc_2d_int (4, 3);
  seq[0][0] = 0; 
  seq[0][1] = 1; 
  seq[0][2] = 2; 
  seq[1][0] = 1; 
  seq[1][1] = 2; 
  seq[1][2] = 3; 
  seq[2][0] = 2; 
  seq[2][1] = 3; 
  seq[2][2] = 0; 
  seq[3][0] = 0; 
  seq[3][1] = 1; 
  seq[3][2] = 3; 

  Mesh0D.EltQty = Mesh0D.EltQty;
      
  ut_print_message (0, 2, "Preparing mesh data ...\n");
  fflush (stdout);

  elt_qty = ut_array_1d_int_sum (Print.showelt + 1, Mesh3D.EltQty);
  elts = ut_alloc_1d_int (elt_qty);
  rgbs = ut_alloc_2d_int (elt_qty, 3);

  elt_qty = 0;
  for (i = 1; i <= Mesh3D.EltQty; i++)
  {
    if (Print.showelt[i] == 1)
    {
      elts[elt_qty] = i;
      ut_array_1d_int_memcpy (rgbs[elt_qty], 3, Data.eltcolour[elts[elt_qty]]);
      elt_qty++;
    }
  }

  for (i = 0; i < elt_qty; i++)
    if (elts[i] > Mesh3D.EltQty)
    {
      ut_print_message (1, 0, "color file: elt id exceeds number of mesh elts.\n");
      abort ();
    }
  if (elt_qty > 0)
    ut_array_1d_int_inv (elts, elt_qty, &eltsinv, &elt_max);

  // Compressing input data --------------------------------------------
  
  ut_print_message (0, 2, "Reducing data ...\n");
  fflush (stdout);

  char* texture = ut_alloc_1d_char (100);
  double* coo1 = NULL;
  double* coo2 = NULL;
  double* coo3 = NULL;
  double* coo4 = NULL;
  int* leftelts = NULL;
  int lefteltqty = 0;
  int* elset_full = ut_alloc_1d_int (Mesh3D.ElsetQty + 1);

  /*
  if (Print.datareduction == 1)
  {
    if (strcmp (Print.showtype, "elset") == 0)
    {
      neut_geo_polys_skinfaces (Geo, elsets, elset_qty, &faces, &facepoly,
	  &face_qty);
      ut_print_message (0, 3, "%d 3D elsets reduced to %d faces.\n", elset_qty, face_qty);
    }
  }
  */

  if (Print.datareduction == 1)
  {
    // elsets = ut_alloc_1d_int (Mesh3D.ElsetQty);
    // neut_mesh_elts_to_elsets (Mesh3D, elts, elt_qty, elsets,
    // &elset_qty, &leftelts, &lefteltqty);
    
    ut_array_1d_int_set (elset_full + 1, Mesh3D.ElsetQty, 1);
    for (i = 1; i <= Mesh3D.ElsetQty; i++)
      for (j = 1; j <= Mesh3D.Elsets[i][0]; j++)
	if (Print.showelt[Mesh3D.Elsets[i][j]] == 0)
	{
	  elset_full[i] = 0;
	  break;
	}

    neut_mesh_elts_skinelts (Mesh3D, elts, elt_qty, &leftelts, &lefteltqty);

    for (i = 0; i < lefteltqty; i++)
      printelt3d[leftelts[i]] = 1;

    ut_print_message (0, 3,
	"Number of elements   reduced by %2.0f\%% (to %d).\n", \
    100 * (double) (elt_qty - lefteltqty) / (double) elt_qty, lefteltqty);
  }
  else
  {
    lefteltqty = elt_qty;
    leftelts = elts;
  }
    
  // Writing pov file --------------------------------------------------
  nevs_print_mesh_header (file, Print);

  int triangle_qty = 0;
  int** triangles = ut_alloc_2d_int (1, 4);
  if (lefteltqty > 0)
  {
    for (i = 0; i < lefteltqty; i++)
    {
      elt3d = leftelts[i];
      eltinv = eltsinv[elt3d];

      coo1 = Nodes.NodeCoo[Mesh3D.EltNodes[elt3d][0]];
      coo2 = Nodes.NodeCoo[Mesh3D.EltNodes[elt3d][1]];
      coo3 = Nodes.NodeCoo[Mesh3D.EltNodes[elt3d][2]];
      coo4 = Nodes.NodeCoo[Mesh3D.EltNodes[elt3d][3]];

      for (j = 0; j < 4; j++)
      {
	nodes[0] = Mesh3D.EltNodes[elt3d][seq[j][0]];
	nodes[1] = Mesh3D.EltNodes[elt3d][seq[j][1]];
	nodes[2] = Mesh3D.EltNodes[elt3d][seq[j][2]];

	neut_nodes_commonelts (Mesh2D, nodes, 3, elts2d, &elt2dqty);
	neut_nodes_commonelts (Mesh3D, nodes, 3, elts3d, &elt3dqty);


	int print;

	print = 0;

	if (elt3dqty == 1)
	  print = 1;

	if (elt3dqty == 2)
	{
	  // keep the first condition, or combine w the second
	  // (because of the `else if')
	  if ((Mesh3D.EltElset[elts3d[0]] == Mesh3D.EltElset[elts3d[1]]))
	  {
	    if (elset_full[Mesh3D.EltElset[elts3d[0]]] == 1)
	      print = 0;
	    else if (printelt3d[elts3d[0]] == 0
		  || printelt3d[elts3d[1]] == 0)
	      print = 1;
	  }
	  else
	  {
	    if (elset_full[Mesh3D.EltElset[elts3d[0]]] == 1
	     && elset_full[Mesh3D.EltElset[elts3d[1]]] == 1)
	      print = 0;
	    else
	      print = 1;
	  }
	}

	if (print)
	{
	  printelt3dface[elt3d][j] = 1;
	  triangle_qty++;
	  triangles = ut_realloc_2d_int_addline (triangles, triangle_qty, 4);
	  triangles[triangle_qty - 1][0] = elt3d;
	  ut_array_1d_int_memcpy (triangles[triangle_qty - 1] + 1, 3, nodes);

	  if (elt2dqty == 1)
	    printelt2d[elts2d[0]] = 1;
	}
      }
    }

    ut_print_message (0, 3, "Number of elt faces  reduced by %2.0f\%% (to %d).\n", \
  100 * (double) (lefteltqty * 4 - triangle_qty) / (double) (lefteltqty * 4),
    triangle_qty);
  }


  // Element faces
  for (i = 0; i < triangle_qty; i++)
  {
    elt3d = triangles[i][0];
    eltinv = eltsinv[triangles[i][0]];

    fprintf (file,
 "#declare elt3d%d =\n  texture { pigment { rgb <%f,%f,%f> } finish {ambient 0.6} }\n",
	     elt3d, rgbs[eltinv][0] / 255., rgbs[eltinv][1] / 255., rgbs[eltinv][2] / 255.);

    sprintf (texture, "elt3d%d", elt3d);

    nevs_print_triangle (file,      Nodes.NodeCoo[triangles[i][1]],
    Nodes.NodeCoo[triangles[i][2]], Nodes.NodeCoo[triangles[i][3]],
	  texture, NULL, NULL);
  }

  int edge_qty = 0;
  int** edges = ut_alloc_2d_int (1, 3);
  // Element edges
  if (triangle_qty > 0)
    for (i = 0; i < triangle_qty; i++)
      for (j = 0; j < 3; j++)
      {
	for (k = 0; k < 2; k++)
	  tmp[k] = triangles[i][seq2[j][k] + 1];

	// add condition
	edge_qty++;
	edges = ut_realloc_2d_int_addline (edges, edge_qty, 2);
	ut_array_1d_int_memcpy (edges[edge_qty - 1], 2, tmp);
      }

  if (triangle_qty > 0)
    ut_print_message (0, 3, "Number of face edges reduced by %2.0f\%% (to %d).\n", \
  100 * (double) (triangle_qty * 3 - edge_qty) / (double) (triangle_qty * 3),
    edge_qty);
  
  for (i = 0; i < edge_qty; i++)
  {
    nevs_print_segment (file,
      Nodes.NodeCoo[edges[i][0]],
      Nodes.NodeCoo[edges[i][1]],
      "elt_edge_rad", "elt_edge_texture");
  }

  // Element 1D
  //
  int elt1d_qty = ut_array_1d_int_sum (Print.showelt1d + 1, Mesh1D.EltQty);
  int printelt1d_qty = 0;

  if (ut_array_1d_int_sum (Print.showelt1d + 1, Mesh1D.EltQty) > 0)
  {
    for (i = 1; i <= Mesh1D.EltQty; i++)
      if (Print.showelt1d[i] == 1)
      {
	neut_nodes_commonelts (Mesh3D, Mesh1D.EltNodes[i], 2,
	                       elts3d, &elt3dqty);

	int l, print = 0;
	for (j = 0; j < elt3dqty; j++)
	{
	  elt3d = elts3d[j];

	  for (k = 0; k < 4; k++)
	    if (printelt3dface[elt3d][k] == 1)
	    {
	      for (l = 0; l < 3; l++)
		tmp[l] = Mesh3D.EltNodes[elt3d][seq[k][l]];

	      if (ut_array_1d_int_eltpos (tmp, 3, 
		                 Mesh1D.EltNodes[i][0]) != -1
	       && ut_array_1d_int_eltpos (tmp, 3, 
		                 Mesh1D.EltNodes[i][1]) != -1)
	      {
		print = 1;
		break;
	      }
	    }
	}

	if (print == 1)
	{
	  nevs_print_segment (file,
	      Nodes.NodeCoo[Mesh1D.EltNodes[i][0]],
	      Nodes.NodeCoo[Mesh1D.EltNodes[i][1]],
	      "elt1d_edge_rad", "elt1d_edge_texture");

	  printelt1d_qty++;
	}
      }

    ut_print_message (0, 3, "Number of 1d elts    reduced by %2.0f\%% (to %d).\n", \
    100 * (double) (elt1d_qty - printelt1d_qty) / (double) elt1d_qty, printelt1d_qty);
  }

  // Element 1Dall
  //
  for (i = 0; i < Print.showelt1dallqty; i++)
  {
    nevs_print_segment (file,
      Nodes.NodeCoo[Print.showelt1dall[i][0]],
      Nodes.NodeCoo[Print.showelt1dall[i][1]],
      "elt1d_edge_rad", "elt1d_edge_texture");
  }

  ut_free_1d (pos);
  ut_free_1d_int (elts);
  ut_free_1d_int (elt2delts3d);
  ut_free_1d_int (nodes);
  ut_free_1d_int (elts2d);
  ut_free_1d_int (elts3d);
  ut_free_1d_int (printelt2d);
  ut_free_1d_int (printelt3d);
  ut_free_2d_int (printelt3dface, Mesh3D.EltQty + 1);
  ut_free_2d_int (seq, 4);
  ut_free_1d_int (tmp);

  return;
}
