/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_print_tess.h"

void
nevs_print_tess (FILE* file, struct PRINT Print, struct DATA Data,
    struct GEO Geo)
{
  int i, j;
  double** coo = NULL;
  int* polylist = NULL;
  int* facelist = NULL;
  int* edgelist = NULL;
  int* facepolylist = NULL;
  int* neigh = NULL;
  int poly, allneigh, show, facepoly, face, edge;
  char* texture = ut_alloc_1d_char (100);

  // Writing pov file --------------------------------------------------
  nevs_print_tess_header (file, Print);

  // compressing poly to print
  polylist = ut_alloc_1d_int (Geo.PolyQty + 1);
  neigh = ut_alloc_1d_int (Geo.PolyQty + 1);

  if (Print.showpoly[0] > 0)
    for (i = 1; i <= Geo.PolyQty; i++)
      if (Print.showpoly[i] == 1)
      {
	neut_geo_poly_neighpoly (Geo, i, neigh + 1, &(neigh[0]));
	
	if (neigh[0] < Geo.PolyFaceQty[i])
	  polylist[++polylist[0]] = i;
	else
	{
	  allneigh = 1;
	  for (j = 1; j <= neigh[0]; j++)
	    if (Print.showpoly[neigh[j]] == 0)
	    {
	      allneigh = 0;
	      break;
	    }
	  if (allneigh == 0)
	    polylist[++polylist[0]] = i;
	}
      }

  // from the polylist, determining the face list
  facelist = ut_alloc_1d_int (Geo.FaceQty + 1);
  facepolylist = ut_alloc_1d_int (Geo.FaceQty + 1);

    for (i = 1; i <= Geo.FaceQty; i++)
    {
      show = 0;
      facepoly = 0;
      for (j = 0; j < 2; j++)
	if (Geo.FacePoly[i][j] > 0 && ut_array_1d_int_eltpos
		      (polylist + 1, polylist[0], Geo.FacePoly[i][j]) != -1)
	{
	  show++;
	  facepoly = Geo.FacePoly[i][j];
	}

      if (show == 1)
      {
	facelist[++facelist[0]] = i;
	facepolylist[++facepolylist[0]] = facepoly;
      }
    }

  edgelist = ut_alloc_1d_int (Geo.EdgeQty + 1);
  
  if (Print.showedge[0] > 0)
    for (i = 1; i <= Geo.EdgeQty; i++)
      if (Print.showedge[i] == 1)
	edgelist[++edgelist[0]] = i;

  double* p = ut_alloc_1d (3);
  double* p2 = NULL;

  // Writing faces
  for (i = 1; i <= facelist[0]; i++)
  {
    face = facelist[i];
    poly = facepolylist[i];

    fprintf (file,
 "#declare grain%d =\n  texture { pigment { rgb <%f,%f,%f> } finish {ambient 0.6} }\n",
      poly, Data.polycolour[poly][0] / 255., \
      Data.polycolour[poly][1] / 255., \
      Data.polycolour[poly][2] / 255.);

    sprintf (texture, "grain%d", poly);

    // vertex coordinates
    coo = ut_alloc_2d (Geo.FaceVerQty[face], 3);

    for (j = 1; j <= Geo.FaceVerQty[face]; j++)
      ut_array_1d_memcpy (coo[j - 1], 3, Geo.VerCoo[Geo.FaceVerNb[face][j]]);

    // interpolation point
    if (Print.showfaceinter == 1 && Geo.FaceState[face] > 0)
    {
      if (Geo.FacePt[face] == 0)
      {
	ut_array_1d_set (p, 3, 0);
	for (j = 0; j < Geo.FaceVerQty[face]; j++)
	  ut_array_1d_add (p, coo[j], 3, p);
	ut_array_1d_scale (p, 3, 1./Geo.FaceVerQty[face]);
      }
      else if (Geo.FacePt[face] > 0)
	ut_array_1d_memcpy (p, 3, Geo.FacePtCoo[face]);

      p2 = p;
    }
    else
      p2 = NULL;

    fprintf (file, "// face %d\n", face);

    if (Print.showedge[0] == -1)
      nevs_print_polygon (file, Geo.FaceVerQty[face], coo, texture, \
			  "edge_rad", "edge_texture", p2, NULL, NULL);
    else
      nevs_print_polygon (file, Geo.FaceVerQty[face], coo, texture, \
			  NULL, NULL, p2, NULL, NULL);

    if (Print.showfaceinter == 1 && Geo.FaceState[face] > 0)
      for (j = 0; j < Geo.FaceVerQty[face]; j++)
	if (j +1 != Geo.FacePt[face])
	  nevs_print_segment (file, p, coo[j], "faceinter_rad", "faceinter_texture");

    ut_free_2d (coo, Geo.FaceVerQty[face]);
  }

  ut_free_1d (p);

  // Writing edges
  for (i = 1; i <= edgelist[0]; i++)
  {
    edge = edgelist[i];

    fprintf (file,
 "#declare edge%d =\n  texture { pigment { rgb <%f,%f,%f> } finish {ambient 0.6} }\n",
      edge, Data.edgecolour[edge][0] / 255., \
      Data.edgecolour[edge][1] / 255., Data.edgecolour[edge][2] / 255.);
    
    sprintf (texture, "edge%d", edge);

    coo = ut_alloc_2d (2, 3);

    ut_array_1d_memcpy (coo[0], 3, Geo.VerCoo[Geo.EdgeVerNb[edge][0]]);
    ut_array_1d_memcpy (coo[1], 3, Geo.VerCoo[Geo.EdgeVerNb[edge][1]]);

    nevs_print_segment (file, coo[0], coo[1],
			  "edge_rad", texture);

    ut_free_2d (coo, 2);
  }

  ut_free_1d_char (texture);

  return;
}


