/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_print_utils.h"

void
nevs_print_segment (FILE* file, double* coo1,
    double* coo2, char* edge_rad, char* edge_texture)
{
  if (edge_rad != NULL)
  {
    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              %s\n", edge_rad);

    if (edge_texture != NULL)
      fprintf (file, "    texture { %s }\n", edge_texture);
    fprintf (file, "  }\n");
  }
  
  return;
}

void
nevs_print_triangle_edge (FILE* file, double* coo1, double* coo2, double*
    coo3, char* edge_rad, char* edge_texture)
{
  if (edge_rad != NULL)
  {
    fprintf (file, "  union {\n");

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo3[0], coo3[1], coo3[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo3[0], coo3[1], coo3[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              %s }\n", edge_rad);

    if  (edge_texture != NULL)
      fprintf (file, "    texture { %s }\n", edge_texture);
    fprintf (file, "  }\n");
  }
  
  return;
}

void
nevs_print_triangle (FILE* file, double* coo1, double* coo2, double*
    coo3, char* texture, char* edge_rad, char* edge_texture)
{
  fprintf (file, "triangle {<%18.15g,%18.15g,%18.15g>,\n",
	   coo1[0], coo1[1], coo1[2]);
  fprintf (file, "          <%18.15g,%18.15g,%18.15g>,\n",
	   coo2[0], coo2[1], coo2[2]);
  fprintf (file, "          <%18.15g,%18.15g,%18.15g>\n",
	   coo3[0], coo3[1], coo3[2]);

  fprintf (file, "            texture { %s }\n", texture);
  fprintf (file, "  }\n");

  nevs_print_triangle_edge (file, coo1, coo2, coo3, edge_rad, edge_texture);
  
  return;
}
	
void
nevs_print_tet_edge (FILE* file, double* coo1, double* coo2, double*
    coo3, double* coo4, char* edge_rad, char* edge_texture)
{
  if (edge_rad != NULL)
  {
    fprintf (file, "  union {\n");

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo3[0], coo3[1], coo3[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo3[0], coo3[1], coo3[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo4[0], coo4[1], coo4[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo1[0], coo1[1], coo1[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo4[0], coo4[1], coo4[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo2[0], coo2[1], coo2[2]);
    fprintf (file, "              %s }\n", edge_rad);

    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo4[0], coo4[1], coo4[2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
             coo3[0], coo3[1], coo3[2]);
    fprintf (file, "              %s }\n", edge_rad);

    if  (edge_texture != NULL)
      fprintf (file, "    texture { %s }\n", edge_texture);
    fprintf (file, "  }\n");
  }
  
  return;
}
	
void
nevs_print_polygon_edge (FILE* file, int qty, double** coo,
    char* edge_rad, char* edge_texture)
{
  int i;

  if (edge_rad != NULL)
  {
    fprintf (file, "  union {\n");

    for (i = 0; i < qty - 1; i++)
    {
      fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	       coo[i][0], coo[i][1], coo[i][2]);
      fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
	       coo[i + 1][0], coo[i + 1][1], coo[i + 1][2]);
      fprintf (file, "              %s }\n", edge_rad);
    }
    fprintf (file, "    cylinder {<%18.15g,%18.15g,%18.15g>,\n",
	     coo[qty - 1][0], coo[qty - 1][1], coo[qty - 1][2]);
    fprintf (file, "              <%18.15g,%18.15g,%18.15g>,\n",
	     coo[0][0], coo[0][1], coo[0][2]);
    fprintf (file, "              %s }\n", edge_rad);

    if  (edge_texture != NULL)
      fprintf (file, "    texture { %s }\n", edge_texture);
    fprintf (file, "  }\n");
  }
  
  return;
}
	
void
nevs_print_polygon (FILE* file, int qty, double** coo,
    char* texture, char* edge_rad, char* edge_texture,
    double* p, char* iedge_rad, char* iedge_texture)
{
  int i;
  double* c = ut_alloc_1d (3);

  if (p == NULL)
  {
    for (i = 0; i < qty; i++)
      ut_array_1d_add (c, coo[i], 3, c);
    ut_array_1d_scale (c, 3, 1. / qty);
  }
  else
    ut_array_1d_memcpy (c, 3, p);

  // drawing face
  for (i = 0; i < qty - 1; i++)
    nevs_print_triangle (file, c, coo[i], coo[i + 1], texture, NULL, NULL);
  nevs_print_triangle (file, c, coo[qty - 1], coo[0], texture, NULL, NULL);

  nevs_print_polygon_edge (file, qty, coo, edge_rad, edge_texture);

  // drawing segment face interpolation 
  if (iedge_rad != NULL)
    for (i = 0; i < qty; i++)
      nevs_print_segment (file, c, coo[i], iedge_rad, iedge_texture);

  ut_free_1d (c);

  return;
}

	
void
nevs_print_tet (FILE* file, double* coo1, double* coo2, double*
    coo3, double* coo4, char* texture, char* edge_rad, char* edge_texture)
{
  nevs_print_triangle (file, coo1, coo2, coo3, texture, NULL, NULL);
  nevs_print_triangle (file, coo1, coo3, coo4, texture, NULL, NULL);
  nevs_print_triangle (file, coo1, coo4, coo2, texture, NULL, NULL);
  nevs_print_triangle (file, coo2, coo3, coo4, texture, NULL, NULL);

  nevs_print_tet_edge (file, coo1, coo2, coo3, coo4, edge_rad, edge_texture);

  return;
}
