/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"nevs_show.h"
#include<unistd.h>

void
nevs_show_init (struct GEO Geo, struct NODES Nodes,
	   struct MESH Mesh0D, struct MESH Mesh1D, struct MESH Mesh2D,
	   struct MESH Mesh, struct PRINT *pPrint)
{
  if ((*pPrint).showinit != 0)
    return;

  Nodes.NodeQty = Nodes.NodeQty;
  (*pPrint).showtype = ut_alloc_1d_char (100);

  (*pPrint).showedge =
    ut_realloc_1d_int ((*pPrint).showedge, Geo.EdgeQty + 1);

  (*pPrint).showface =
    ut_realloc_1d_int ((*pPrint).showface, Geo.FaceQty + 1);

  (*pPrint).showpoly =
    ut_realloc_1d_int ((*pPrint).showpoly, Geo.PolyQty + 1);

  (*pPrint).showelt0d =
    ut_realloc_1d_int ((*pPrint).showelt0d, Mesh0D.EltQty + 1);

  (*pPrint).showelt1d =
    ut_realloc_1d_int ((*pPrint).showelt1d, Mesh1D.EltQty + 1);

  (*pPrint).showelt2d =
    ut_realloc_1d_int ((*pPrint).showelt2d, Mesh2D.EltQty + 1);

  (*pPrint).showelt = ut_realloc_1d_int ((*pPrint).showelt, Mesh.EltQty + 1);
  
  (*pPrint).showelt1dallqty = 0;

  if (Mesh.EltQty > 0)
  {
    if ((*pPrint).showelt[0] == -1)
    {
      sprintf ((*pPrint).showtype, "elset");
      ut_array_1d_int_set ((*pPrint).showelt + 1, Mesh.EltQty, 1);
      (*pPrint).showelt[0] = Mesh.EltQty;
    }
  }
  else if (Geo.PolyQty > 0 && (*pPrint).showpoly[0] == -1)
  {
    sprintf ((*pPrint).showtype, "poly");
    ut_array_1d_int_set ((*pPrint).showpoly + 1, Geo.PolyQty, 1);
    (*pPrint).showpoly[0] = Geo.PolyQty;
  }

  return;
}

void
nevs_show (char **argv, int *pi, struct GEO Geo, struct NODES Nodes,
	   struct MESH Mesh0D, struct MESH Mesh1D, struct MESH Mesh2D,
	   struct MESH Mesh, struct PRINT *pPrint)
{
  int i, j, k, l, m, n;
  FILE *file;
  double *coo = ut_alloc_1d (3);
  int id;

  (*pPrint).showinit = 1;

  Nodes.NodeQty = Nodes.NodeQty;
  (*pPrint).showtype = ut_alloc_1d_char (100);

  (*pPrint).showedge =
    ut_realloc_1d_int ((*pPrint).showedge, Geo.EdgeQty + 1);
  (*pPrint).showface =
    ut_realloc_1d_int ((*pPrint).showface, Geo.FaceQty + 1);

  (*pPrint).showpoly =
    ut_realloc_1d_int ((*pPrint).showpoly, Geo.PolyQty + 1);

  (*pPrint).showelt0d =
    ut_realloc_1d_int ((*pPrint).showelt0d, Mesh0D.EltQty + 1);

  (*pPrint).showelt1d =
    ut_realloc_1d_int ((*pPrint).showelt1d, Mesh1D.EltQty + 1);

  (*pPrint).showelt2d =
    ut_realloc_1d_int ((*pPrint).showelt2d, Mesh2D.EltQty + 1);

  (*pPrint).showelt = ut_realloc_1d_int ((*pPrint).showelt, Mesh.EltQty + 1);
  (*pPrint).showelt1dallqty = 0;

  if (strcmp (argv[(*pi)], "-showpoly") == 0)
  {
    sprintf ((*pPrint).showtype, "tess");

    (*pi)++;
    ut_array_1d_int_set ((*pPrint).showpoly + 1, Geo.PolyQty, 0);

    if (strcmp (argv[(*pi)], "all") == 0)
      ut_array_1d_int_set ((*pPrint).showpoly + 1, Geo.PolyQty, 1);

    else if (argv[(*pi)][0] == '@')
    {
      file = ut_file_open (argv[(*pi)] + 1, "r");
      while (fscanf (file, "%d", &id) != EOF)
	(*pPrint).showpoly[id] = 1;
      ut_file_close (file, argv[(*pi)] + 1, "r");
    }
    else
    {
      int var_qty = 7;
      char **vars = ut_alloc_2d_char (var_qty, 15);
      double *vals = ut_alloc_1d (var_qty);
      int status;
      double res = 0;

      sprintf (vars[0], "cenx");
      sprintf (vars[1], "ceny");
      sprintf (vars[2], "cenz");
      sprintf (vars[3], "volume");
      sprintf (vars[4], "true");
      sprintf (vars[5], "body");
      sprintf (vars[6], "id");

      for (i = 1; i <= Geo.PolyQty; i++)
      {
	ut_array_1d_memcpy (vals, 3, Geo.CenterCoo[i]);
	neut_geo_poly_volume (Geo, i, &(vals[3]));
	vals[4] = Geo.PolyTrue[i];
	vals[5] = Geo.PolyBody[i];
	vals[6] = i;
#if HAVE_LIBMATHEVAL == 1
	status = ut_math_eval (argv[(*pi)], var_qty, vars, vals, &res);
#else
  ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
  abort ();
#endif
	if (status == -1)
	  abort ();
	(*pPrint).showpoly[i] = ut_num_equal (res, 1, 1e-6);
      }

      ut_free_2d_char (vars, var_qty);
      ut_free_1d (vals);
    }

    (*pPrint).showpoly[0] = ut_array_1d_int_sum ((*pPrint).showpoly + 1,
						 Geo.PolyQty);
  }

  else if (strcmp (argv[(*pi)], "-showfaceinter") == 0)
  {
    (*pi)++;
    sscanf (argv[(*pi)], "%d", &((*pPrint).showfaceinter));
  }

  else if (strcmp (argv[(*pi)], "-showedge") == 0)
  {
    sprintf ((*pPrint).showtype, "tess");

    (*pi)++;
    ut_array_1d_int_set ((*pPrint).showedge + 1, Geo.EdgeQty, 0);
    (*pPrint).showedge[0] = -1;

    if (strcmp (argv[(*pi)], "all") == 0)
      ut_array_1d_int_set ((*pPrint).showedge + 1, Geo.EdgeQty, 1);

    else if (argv[(*pi)][0] == '@')
    {
      file = ut_file_open (argv[(*pi)] + 1, "r");
      while (fscanf (file, "%d", &id) != EOF)
	(*pPrint).showedge[id] = 1;
      ut_file_close (file, argv[(*pi) + 1], "r");
    }
    else
    {
      int var_qty = 10;
      char **vars = ut_alloc_2d_char (var_qty, 15);
      double *vals = ut_alloc_1d (var_qty);
      int status;
      double res;
      int *poly = ut_alloc_1d_int (1000);

      sprintf (vars[0], "cenx");
      sprintf (vars[1], "ceny");
      sprintf (vars[2], "cenz");
      sprintf (vars[3], "length");
      sprintf (vars[4], "true");
      sprintf (vars[5], "body");
      sprintf (vars[6], "id");
      sprintf (vars[7], "poly_true");
      sprintf (vars[8], "poly_body");
      sprintf (vars[9], "poly_shown");

      for (i = 1; i <= Geo.EdgeQty; i++)
      {
	neut_geo_edge_centre (Geo, i, vals);
	vals[3] = Geo.EdgeLength[i];
	vals[4] = neut_geo_edge_true (Geo, Geo.PolyTrue, i);
	vals[5] = neut_geo_edge_true (Geo, Geo.PolyBody, i);
	vals[6] = i;
	vals[7] = neut_geo_edge_polytruelevelmax (Geo, Geo.PolyTrue, i);
	vals[8] = neut_geo_edge_polybodylevelmax (Geo, Geo.PolyBody, i);
	vals[9] = 0;
	neut_geo_edge_poly (Geo, i, poly);
	for (j = 1; j <= poly[0]; j++)
	  if ((*pPrint).showpoly[poly[j]] == 1)
	  {
	    vals[9] = 1;
	    break;
	  }

#if HAVE_LIBMATHEVAL == 1
	status = ut_math_eval (argv[(*pi)], var_qty, vars, vals, &res);
#else
  ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
  abort ();
#endif
	if (status == -1)
	  abort ();
	(*pPrint).showedge[i] = ut_num_equal (res, 1, 1e-6);
      }

      ut_free_2d_char (vars, var_qty);
      ut_free_1d (vals);
      ut_free_1d_int (poly);
    }

    (*pPrint).showedge[0] = ut_array_1d_int_sum ((*pPrint).showedge + 1,
						 Geo.EdgeQty);
  }

  // show elements -----------------------------------------------------
  else if (strcmp (argv[(*pi)], "-showelt") == 0)
  {
    sprintf ((*pPrint).showtype, "elt");

    (*pi)++;
    ut_array_1d_int_set ((*pPrint).showelt + 1, Mesh.EltQty, 0);

    if (strcmp (argv[(*pi)], "all") == 0)
      ut_array_1d_int_set ((*pPrint).showelt + 1, Mesh.EltQty, 1);

    else if (argv[(*pi)][0] == '@')
    {
      file = ut_file_open (argv[(*pi)] + 1, "r");
      while (fscanf (file, "%d", &id) != EOF)
	(*pPrint).showelt[id] = 1;
      ut_file_close (file, argv[(*pi) + 1], "r");
    }
    else
    {
      int var_qty = 8;
      char **vars = ut_alloc_2d_char (var_qty, 15);
      double *vals = ut_alloc_1d (var_qty);
      int status;
      double res;

      sprintf (vars[0], "cenx");
      sprintf (vars[1], "ceny");
      sprintf (vars[2], "cenz");
      sprintf (vars[3], "volume");
      sprintf (vars[4], "elset_true");
      sprintf (vars[5], "elset_body");
      sprintf (vars[6], "id");
      sprintf (vars[7], "elset_id");

      for (i = 1; i <= Mesh.EltQty; i++)
      {
	neut_mesh_eltcentre (Mesh, Nodes, i, vals);
	neut_mesh_elt_volume (Nodes, Mesh, i, &(vals[3]));
	vals[4] = Geo.PolyTrue[Mesh.EltElset[i]];
	vals[5] = Geo.PolyBody[Mesh.EltElset[i]];
	vals[6] = i;
	vals[7] = Mesh.EltElset[i];

	int done = 0;
	for (j = 0; j < var_qty; j++)
	  if (strcmp (argv[(*pi)], vars[j]) == 0)
	  {
	    res = vals[j];
	    done = 1;
	    status = 1;
	    break;
	  }

	if (done == 0 && strcmp (argv[(*pi)], "(-ceny+cenz)<0.4") == 0)
	{
	  res = (-vals[1]+vals[2] < 0.4) ? 1: 0;
	  done = 1;
	  status = 1;
	}

	if (done == 0)
	{
#if HAVE_LIBMATHEVAL == 1
	status = ut_math_eval (argv[(*pi)], var_qty, vars, vals, &res);
#else
  ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
  abort ();
#endif
	if (status == -1)
	  abort ();
	}
	(*pPrint).showelt[i] = ut_num_equal (res, 1, 1e-6);
      }

      ut_free_2d_char (vars, var_qty);
      ut_free_1d (vals);
    }

    (*pPrint).showelt[0] = ut_array_1d_int_sum ((*pPrint).showelt + 1,
						Mesh.EltQty);
  }

  else if (strcmp (argv[(*pi)], "-showelt1d") == 0)
  {
    (*pi)++;
    ut_array_1d_int_set ((*pPrint).showelt1d + 1, Mesh1D.EltQty, 0);

    if (strcmp (argv[(*pi)], "all") == 0)
      ut_array_1d_int_set ((*pPrint).showelt1d + 1, Mesh1D.EltQty, 1);

    else if (argv[(*pi)][0] == '@')
    {
      file = ut_file_open (argv[(*pi)] + 1, "r");
      while (fscanf (file, "%d", &id) != EOF)
	(*pPrint).showelt1d[id] = 1;
      ut_file_close (file, argv[(*pi) + 1], "r");
    }
    else
    {
      int var_qty = 8;
      char **vars = ut_alloc_2d_char (var_qty, 100);
      double *vals = ut_alloc_1d (var_qty);
      int status;
      double res;
      int *elt1delts3d = ut_alloc_1d_int (1000);
      int elt1delt3dqty;

      sprintf (vars[0], "cenx");
      sprintf (vars[1], "ceny");
      sprintf (vars[2], "cenz");
      sprintf (vars[3], "length");
      sprintf (vars[4], "elset_true");
      sprintf (vars[5], "elset_body");
      sprintf (vars[6], "id");
      sprintf (vars[7], "elt3d_shown");

      for (i = 1; i <= Mesh1D.EltQty; i++)
      {
	neut_mesh_eltcentre (Mesh, Nodes, i, vals);
	neut_mesh_eltlength (Nodes, Mesh, i, &(vals[3]));
	vals[4] = Geo.PolyTrue[Mesh.EltElset[i]];
	vals[5] = Geo.PolyBody[Mesh.EltElset[i]];
	vals[6] = i;

	vals[7] = 0;
	neut_mesh_elt1delts3d (Mesh1D, i, Mesh2D, Mesh, elt1delts3d,
			       &elt1delt3dqty);
	for (j = 0; j < elt1delt3dqty; j++)
	  if ((*pPrint).showelt[elt1delts3d[j]] == 1)
	  {
	    vals[7] = 1;
	    break;
	  }

	int done = 0;
	for (j = 0; j < var_qty; j++)
	  if (strcmp (argv[(*pi)], vars[j]) == 0)
	  {
	    res = vals[j];
	    done = 1;
	    status = 1;
	    break;
	  }

	if (done == 0)
	{
#if HAVE_LIBMATHEVAL == 1
	  status = ut_math_eval (argv[(*pi)], var_qty, vars, vals, &res);
#else
	  ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
	  abort ();
#endif
	}

	if (status == -1)
	  abort ();

	(*pPrint).showelt1d[i] = ut_num_equal (res, 1, 1e-6);
      }

      ut_free_2d_char (vars, var_qty);
      ut_free_1d (vals);
      ut_free_1d_int (elt1delts3d);
    }

    (*pPrint).showelt1d[0] = ut_array_1d_int_sum ((*pPrint).showelt1d + 1,
						  Mesh1D.EltQty);
  }

  else if (strcmp (argv[(*pi)], "-showelt1din") == 0)
  {
    int* nelts = ut_alloc_1d_int (1000);
    int* cnodes = ut_alloc_1d_int (20);
    int cnodeqty;
    int* celts = ut_alloc_1d_int (20);
    int* nodepair = ut_alloc_1d_int (2);
    int celtqty;

    (*pPrint).showelt1dallqty = 0;
    (*pPrint).showelt1dall = ut_alloc_1d_pint (1);
    for (i = 1; i <= Mesh.EltQty; i++)
      if ((*pPrint).showelt[i] == 1)
      {
	neut_mesh_elt_neighelts (Mesh, i, nelts + 1, &(nelts[0])); 

	for (j = 1; j <= nelts[0]; j++)
	  if ((*pPrint).showelt[nelts[j]] == 1)
	    if (Mesh.EltElset[i] != Mesh.EltElset[nelts[j]])
	    {
	      cnodeqty = 0;
	      for (k = 0; k < 4; k++)
		if (ut_array_1d_int_eltpos (Mesh.EltNodes[i], 4,
		      Mesh.EltNodes[nelts[j]][k]) != -1)
		  cnodes[cnodeqty++] = Mesh.EltNodes[nelts[j]][k];

	      if (cnodeqty > 3 || cnodeqty < 1)
		abort ();
	      
	      for (k = 0; k < cnodeqty; k++)
	      {
		nodepair[0] = cnodes[k];
		for (l = k + 1; l < cnodeqty; l++)
		{
		  nodepair[1] = cnodes[l];
		  neut_nodes_commonelts (Mesh, nodepair, 2, celts, &celtqty);
		  for (m = 0; m < celtqty; m++)
		    if ((*pPrint).showelt[celts[m]] == 0)
		    {
		      int record = 1;
		      for (n = 0; n < (*pPrint).showelt1dallqty; n++)
			if ((*pPrint).showelt1dall[n][0] == nodepair[0]
			 && (*pPrint).showelt1dall[n][1] == nodepair[1])
			{
			  record = 0;
			  break;
			}

		      if (record == 1)
		      {
			(*pPrint).showelt1dallqty++;
			(*pPrint).showelt1dall =
			  ut_realloc_2d_int_addline
			  ((*pPrint).showelt1dall,
			   (*pPrint).showelt1dallqty, 2);

			(*pPrint).showelt1dall[(*pPrint).showelt1dallqty - 1][0] = nodepair[0];
			(*pPrint).showelt1dall[(*pPrint).showelt1dallqty - 1][1] = nodepair[1];
		      }
		    }
		}
	      }
	    }
      }

    ut_free_1d_int (nelts);
    ut_free_1d_int (cnodes);
    ut_free_1d_int (celts);
    ut_free_1d_int (nodepair);
  }

  // show element sets -------------------------------------------------
  else if (strcmp (argv[(*pi)], "-showelset") == 0)
  {
    sprintf ((*pPrint).showtype, "elt");

    ut_array_1d_int_set ((*pPrint).showelt + 1, Mesh.EltQty, 0);
    (*pi)++;

    if (strcmp (argv[(*pi)], "all") == 0)
      ut_array_1d_int_set ((*pPrint).showelt + 1, Mesh.EltQty, 1);
    else if (argv[(*pi)][0] == '@')
    {
      int elsetqty = ut_file_nbwords (argv[(*pi)] + 1);

      file = ut_file_open (argv[(*pi)] + 1, "r");
      for (i = 0; i < elsetqty; i++)
      {
	fscanf (file, "%d", &id);
	if (id > Mesh.ElsetQty)
	{
	  ut_print_message (2, 0, "Elset id exceeds the maximum.\n");
	  abort ();
	}
	for (j = 1; j <= Mesh.Elsets[id][0]; j++)
	  (*pPrint).showelt[Mesh.Elsets[id][j]] = 1;
      }
      ut_file_close (file, argv[(*pi)] + 1, "r");
    }
    else
    {
      int var_qty = 8;
      char **vars = ut_alloc_2d_char (var_qty, 10);
      double *vals = ut_alloc_1d (var_qty);
      int status;
      double res = 0;

      sprintf (vars[0], "cenx");
      sprintf (vars[1], "ceny");
      sprintf (vars[2], "cenz");
      sprintf (vars[3], "vol");
      sprintf (vars[4], "true");
      sprintf (vars[5], "body");
      sprintf (vars[6], "id");
      sprintf (vars[7], "eltqty");

      for (i = 1; i <= Mesh.ElsetQty; i++)
      {
	neut_mesh_elset_centre (Nodes, Mesh, i, vals);
	neut_mesh_elset_volume (Nodes, Mesh, i, &(vals[3]));
	vals[4] = Geo.PolyTrue[i];
	vals[5] = Geo.PolyBody[i];
	vals[6] = i;
	vals[7] = Mesh.Elsets[i][0];

	status = -1;
	for (j = 0; j < var_qty; j++)
	  if (strcmp (argv[(*pi)], vars[j]) == 0)
	  {
	    res = vals[j];
	    status = 0;
	  }

	if (status == -1)
	{
#if HAVE_LIBMATHEVAL == 1
	status = ut_math_eval (argv[(*pi)], var_qty, vars, vals, &res);
#else
	ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
	abort ();
#endif
	}

	if (status == -1)
	  abort ();

	if (ut_num_equal (res, 1, 1e-6))
	  for (j = 1; j <= Mesh.Elsets[i][0]; j++)
	    (*pPrint).showelt[Mesh.Elsets[i][j]] = 1;
      }

      ut_free_2d_char (vars, var_qty);
      ut_free_1d (vals);
    }

    (*pPrint).showelt[0] = ut_array_1d_int_sum ((*pPrint).showelt + 1,
						Mesh.EltQty);
  }

  // error -------------------------------------------------------------
  /*
  else
  {
    ut_print_message (2, 0, "Unknown command %s\n", argv[(*pi)]);
    fflush (stdout);
    abort ();
  }
  */

  ut_free_1d (coo);

  return;
}
