/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_debug.h"

void
neut_debug_nodes (FILE* file, struct NODES Nodes)
{
  fprintf (file, "====== Beginning of Nodes ================================\n");
  fprintf (file, "Nodes:\n");
  fprintf (file, "NodeQty = %d\n", Nodes.NodeQty);

  if (Nodes.NodeQty > 0)
  {
    fprintf (file, "NodeCoo =\n");
    ut_array_2d_fprintf (file, Nodes.NodeCoo + 1, Nodes.NodeQty, 3, "%f");
    fprintf (file, "NodeCl =\n");
    if (Nodes.NodeCl == NULL)
      fprintf (file, "null\n");
    else
      ut_array_1d_fprintf (file, Nodes.NodeCl + 1, Nodes.NodeQty, "%f\n");
  }
  
  fprintf (file, "====== End of Nodes ======================================\n");
  return;
  
  return;
}

void
neut_debug_mesh (FILE* file, struct MESH Mesh)
{
  int i;
  int eltnodeqty = neut_elt_nodeqty ("tri", Mesh.Dimension, Mesh.EltOrder);

  fprintf (file, "====== Beginning of Mesh =================================\n");
  fprintf (file, "Mesh:\n");
  fprintf (file, "Dimension = %d\n", Mesh.Dimension);
  fprintf (file, "EltOrder = %d\n", Mesh.EltOrder);
  fprintf (file, "EltQty = %d\n", Mesh.EltQty);
  if (Mesh.EltQty > 0)
  {
    fprintf (file, "EltNodes = \n");
    ut_array_2d_int_fprintf (file, Mesh.EltNodes + 1, Mesh.EltQty, eltnodeqty, "%d");
    fprintf (file, "ElsetQty = %d\n", Mesh.ElsetQty);
    fprintf (file, "Elsets = (quantity then ids of elements)\n");
    for (i = 1; i <= Mesh.ElsetQty; i++)
      ut_array_1d_int_fprintf (file, Mesh.Elsets[i], Mesh.Elsets[i][0] + 1, "%d");
  }

  fprintf (file, "====== End of Mesh =======================================\n");
  return;
}

void
neut_debug_fod (FILE* file, int** FoD)
{
  int i;

  fprintf (file, "====== Beginning of FoD ==================================\n");
  fprintf (file, "FoD = (quantity then ids of faces)\n");
  for (i = 1; i <= 6; i++)
    ut_array_1d_int_fprintf (file, FoD[i], FoD[i][0] + 1, "%d");

  fprintf (file, "====== End of FoD ========================================\n");

  return;
}

void
neut_debug_fodnodes (FILE* file, int** FoDNodes)
{
  int i;

  fprintf (file, "====== Beginning of FoDNodes =============================\n");
  fprintf (file, "FoDNodes = (quantity then ids of faces)\n");
  for (i = 1; i <= 6; i++)
    ut_array_1d_int_fprintf (file, FoDNodes[i], FoDNodes[i][0] + 1, "%d");

  fprintf (file, "====== End of FoDNodes ===================================\n");

  return;
}


void
neut_debug_geo (FILE* file, struct GEO Geo)
{
  int i;

  fprintf (file, "====== Beginning of Geo ==================================\n");
  fprintf (file, "VerQty = %d\n", Geo.VerQty);
  fprintf (file, "EdgeQty = %d\n", Geo.EdgeQty);
  fprintf (file, "FaceQty = %d\n", Geo.FaceQty);
  fprintf (file, "PolyQty = %d\n", Geo.PolyQty);

  fprintf (file, "== Vertices =================\n");
  fprintf (file, "[id] VerCoo =\n");
  if (Geo.VerCoo == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.VerQty; i++)
    {
      fprintf (file, "%d ", i);
      ut_array_1d_fprintf (file, Geo.VerCoo[i], 3, "%f");
    }

  fprintf (file, "[id] VerEdgeQty then VerEdgeNb =\n");
  if (Geo.VerEdgeQty == NULL || Geo.VerEdgeNb == NULL)
    fprintf (file, "one is NULL\n");
  else 
    for (i = 1; i <= Geo.VerQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.VerEdgeQty[i]);
      ut_array_1d_int_fprintf (file, Geo.VerEdgeNb[i], Geo.VerEdgeQty[i], "%d");
    }

  fprintf (file, "[id] VerBound =\n");
  if (Geo.VerBound == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.VerQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.VerEdgeQty[i]);
      ut_array_1d_int_fprintf (file, Geo.VerBound[i], 4, "%d");
    }
  
  fprintf (file, "[id] VerState =\n");
  if (Geo.VerState == NULL)
    fprintf (file, "is NULL\n");
  else
  for (i = 1; i <= Geo.VerQty; i++)
    fprintf (file, "%d %d\n", i, Geo.VerState[i]);

  fprintf (file, "== Edges =================\n");

  fprintf (file, "[id] EdgeVerNb =\n");
  if (Geo.EdgeVerNb == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.EdgeQty; i++)
      fprintf (file, "%d %d %d\n", i, Geo.EdgeVerNb[i][0], Geo.EdgeVerNb[i][1]);

  fprintf (file, "[id] EdgeFaceQty then Nb =\n");
  if (Geo.EdgeFaceQty == NULL || Geo.EdgeFaceNb == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.EdgeQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.EdgeFaceQty[i]);
      ut_array_1d_int_fprintf (file, Geo.EdgeFaceNb[i], Geo.EdgeFaceQty[i], "%d");
    }

  fprintf (file, "[id] EdgeLength =\n");
  if (Geo.EdgeLength == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.EdgeQty; i++)
      fprintf (file, "%d %f\n", i, Geo.EdgeLength[i]);

  fprintf (file, "[id] EdgeState =\n");
  if (Geo.EdgeState == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.EdgeQty; i++)
      fprintf (file, "%d %d\n", i, Geo.EdgeState[i]);

  fprintf (file, "[id] EdgeDel =\n");
  if (Geo.EdgeDel == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.EdgeQty; i++)
      fprintf (file, "%d %d\n", i, Geo.EdgeDel[i]);

  fprintf (file, "== Faces =================\n");

  fprintf (file, "[id] FacePoly =\n");
  if (Geo.FacePoly == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
      fprintf (file, "%d %d %d\n", i, Geo.FacePoly[i][0], Geo.FacePoly[i][1]);

  fprintf (file, "[id] FaceVerQty then Nb =\n");
  if (Geo.FaceVerQty == NULL || Geo.FaceVerNb == NULL)
    fprintf (file, "one is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.FaceVerQty[i]);
      ut_array_1d_int_fprintf (file, Geo.FaceVerNb[i] + 1, Geo.FaceVerQty[i], "%d");
    }

  fprintf (file, "[id] FaceEdgeQty then Nb =\n");
  if (Geo.FaceVerQty == NULL || Geo.FaceEdgeNb == NULL)
    fprintf (file, "one is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.FaceVerQty[i]);
      ut_array_1d_int_fprintf (file, Geo.FaceEdgeNb[i] + 1, Geo.FaceVerQty[i], "%d");
    }

  fprintf (file, "[id] FaceEdgeQty then Ori =\n");
  if (Geo.FaceVerQty == NULL || Geo.FaceEdgeOri == NULL)
    fprintf (file, "one is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
    {
      fprintf (file, "%d %d ", i, Geo.FaceVerQty[i]);
      ut_array_1d_int_fprintf (file, Geo.FaceEdgeOri[i] + 1, Geo.FaceVerQty[i], "%d");
    }

  fprintf (file, "[id] FaceEq =\n");
  if (Geo.FaceEq == NULL)
    fprintf (file, "is NULL\n");
  else
  for (i = 1; i <= Geo.FaceQty; i++)
    ut_array_1d_fprintf (file, Geo.FaceEq[i], 4, "%f");

  fprintf (file, "[id] FaceState =\n");
  if (Geo.FaceState == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
      fprintf (file, "%d %d\n", i, Geo.FaceState[i]);

  fprintf (file, "[id] FacePt =\n");
  if (Geo.FacePt == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
      fprintf (file, "%d %d\n", i, Geo.FacePt[i]);

  fprintf (file, "[id] FacePtCoo =\n");
  if (Geo.FacePtCoo == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
    {
      fprintf (file, "%d ", i);
      ut_array_1d_fprintf (file, Geo.FacePtCoo[i], 3, "%f");
    }

  fprintf (file, "[id] FaceFF =\n");
  if (Geo.FaceFF == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
      fprintf (file, "%d %f\n", i, Geo.FaceFF[i]);

  fprintf (file, "[id] FaceBound =\n");
  if (Geo.FaceBound == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.FaceQty; i++)
      fprintf (file, "%d %d\n", i, Geo.FaceBound[i]);

  fprintf (file, "== Polys =================\n");

  fprintf (file, "[id] CenterCoo =\n");
  if (Geo.CenterCoo == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.PolyQty; i++)
    {
      fprintf (file, "%d ", i);
      ut_array_1d_fprintf (file, Geo.CenterCoo[i], 3, "%f");
    }

  fprintf (file, "[id] PolyFaceQty then Nb =\n");
  if (Geo.PolyFaceQty == NULL || Geo.PolyFaceNb == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.PolyQty; i++)
    {
      fprintf (file, "%d %d\n", i, Geo.PolyFaceQty[i]);
      ut_array_1d_int_fprintf (file, Geo.PolyFaceNb[i] + 1, Geo.PolyFaceQty[i], "%d");
    }

  fprintf (file, "[id] PolyFaceQty then Ori =\n");
  if (Geo.PolyFaceQty == NULL || Geo.PolyFaceOri == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.PolyQty; i++)
    {
      fprintf (file, "%d %d\n", i, Geo.PolyFaceQty[i]);
      ut_array_1d_int_fprintf (file, Geo.PolyFaceOri[i] + 1, Geo.PolyFaceQty[i], "%d");
    }

  fprintf (file, "[id] PolyTrue =\n");
  if (Geo.PolyTrue == NULL)
    fprintf (file, "is NULL\n");
  else
    for (i = 1; i <= Geo.PolyQty; i++)
    {
      fprintf (file, "%d %d\n", i, Geo.PolyTrue[i]);
    }

  fprintf (file, "====== End of Geo ========================================\n");

  return;
}


void
neut_debug_germset (FILE* file, struct GERMSET GermSet)
{
  int i;

  fprintf (file, "morpho = %s\n", GermSet.morpho);
  fprintf (file, "N = %d\n", GermSet.N);
  fprintf (file, "N1d = %d\n", GermSet.N1d);
  fprintf (file, "Id = %d\n", GermSet.Id);
  fprintf (file, "Random = %ld\n", GermSet.Random);

  fprintf (file, "GermsCoo =\n");
  for (i = 1; i <= GermSet.N; i++)
    ut_array_1d_fprintf (file, GermSet.GermsCoo[i], 3, "%f");

  fprintf (file, "ttype = %d\n", GermSet.ttype);
  fprintf (file, "NDensity = %f\n", GermSet.NDensity);
  fprintf (file, "randomize = %f\n", GermSet.randomize);
  fprintf (file, "randomize2 = %d\n", GermSet.randomize2);

  return;
}
