/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_geo.h"

void
neut_geo_set_zero (struct GEO* pGeo)
{
  (*pGeo).VerQty = 0;
  (*pGeo).FaceQty = 0;
  (*pGeo).EdgeQty = 0;
  (*pGeo).PolyQty = 0;
  (*pGeo).DomainSize = NULL;
  (*pGeo).VerCoo = NULL;
  (*pGeo).VerEdgeQty = NULL;
  (*pGeo).VerEdgeNb = NULL;
  (*pGeo).VerBound = NULL;
  (*pGeo).VerState = NULL;
  (*pGeo).EdgeVerNb = NULL;
  (*pGeo).EdgeFaceQty = NULL;
  (*pGeo).EdgeFaceNb = NULL;
  (*pGeo).EdgeLength = NULL;
  (*pGeo).EdgeState = NULL;
  (*pGeo).EdgeDel = NULL;
  (*pGeo).FacePoly = NULL;
  (*pGeo).FaceVerQty = NULL;
  (*pGeo).FaceVerNb = NULL;
  (*pGeo).FaceEdgeNb = NULL;
  (*pGeo).FaceEdgeOri = NULL;
  (*pGeo).FaceEq = NULL;
  (*pGeo).FaceState = NULL;
  (*pGeo).FacePt = NULL;
  (*pGeo).FacePtCoo = NULL;
  (*pGeo).FaceFF = NULL;
  (*pGeo).FaceBound = NULL;
  (*pGeo).PolyFaceQty = NULL;
  (*pGeo).PolyFaceNb = NULL;
  (*pGeo).PolyFaceOri = NULL;
  (*pGeo).PolyTrue = NULL;
  (*pGeo).PolyBody = NULL;
  (*pGeo).CenterCoo = NULL;

  return;
}
  
struct GEO
neut_geo_alloc (int VerQty, int EdgeQty, int FaceQty, int PolyQty)
{
  struct GEO Geo;

  Geo.VerQty  = VerQty;
  Geo.EdgeQty = EdgeQty;
  Geo.FaceQty = FaceQty;
  Geo.PolyQty = PolyQty;

  Geo.VerCoo = ut_alloc_2d (Geo.VerQty + 1, 3);
  Geo.VerEdgeQty = ut_alloc_1d_int (Geo.VerQty + 1);
  Geo.VerEdgeNb = ut_alloc_2d_int (Geo.VerQty + 1, 20);	/* ? */
  Geo.VerBound = ut_alloc_2d_int (Geo.VerQty + 1, 4);
  Geo.VerState = ut_alloc_1d_int (Geo.VerQty + 1);	/* ini to 0 */

  Geo.EdgeVerNb = ut_alloc_2d_int (Geo.EdgeQty + 1, 2);
  Geo.EdgeFaceQty = ut_alloc_1d_int (Geo.EdgeQty + 1);
  Geo.EdgeFaceNb = ut_alloc_2d_int (Geo.EdgeQty + 1, 10);
  Geo.EdgeLength = ut_alloc_1d (Geo.EdgeQty + 1);
  Geo.EdgeState = ut_alloc_1d_int (Geo.EdgeQty + 1);
  Geo.EdgeDel = ut_alloc_1d_int (Geo.EdgeQty + 1);

  Geo.FacePoly = ut_alloc_2d_int (Geo.FaceQty + 1, 2);
  Geo.FaceVerQty = ut_alloc_1d_int (Geo.FaceQty + 1);
  Geo.FaceVerNb = ut_alloc_1d_pint (Geo.FaceQty + 1);
  Geo.FaceEdgeNb = ut_alloc_1d_pint (Geo.FaceQty + 1);
  Geo.FaceEdgeOri = ut_alloc_1d_pint (Geo.FaceQty + 1);
  Geo.FaceEq = ut_alloc_2d (Geo.FaceQty + 1, 4);
  Geo.FaceState = ut_alloc_1d_int (Geo.FaceQty + 1);
  Geo.FacePt = ut_alloc_1d_int (Geo.FaceQty + 1);
  Geo.FacePtCoo = ut_alloc_2d (Geo.FaceQty + 1, 3);
  Geo.FaceFF = ut_alloc_1d (Geo.FaceQty + 1);
  Geo.FaceBound = ut_alloc_1d_int (Geo.FaceQty + 1);

  Geo.CenterCoo = ut_alloc_2d (Geo.PolyQty + 1, 3);
  Geo.PolyFaceQty = ut_alloc_1d_int (Geo.PolyQty + 1);
  Geo.PolyFaceNb = ut_alloc_1d_pint (Geo.PolyQty + 1);
  Geo.PolyFaceOri = ut_alloc_1d_pint (Geo.PolyQty + 1);

  Geo.DomainSize = ut_alloc_2d (3, 2);
  
  Geo.PolyTrue = NULL;
  Geo.PolyBody = NULL;

  return Geo;
}

void
neut_geo_free (struct GEO* pGeo)
{
  ut_free_2d ((*pGeo).VerCoo, (*pGeo).VerQty + 1);
  ut_free_1d_int ((*pGeo).VerEdgeQty);
  ut_free_2d_int ((*pGeo).VerEdgeNb, (*pGeo).VerQty + 1);
  ut_free_2d_int ((*pGeo).VerBound, (*pGeo).VerQty + 1);
  ut_free_1d_int ((*pGeo).VerState);

  ut_free_2d_int ((*pGeo).EdgeVerNb, (*pGeo).EdgeQty + 1);
  ut_free_1d_int ((*pGeo).EdgeFaceQty);
  ut_free_2d_int ((*pGeo).EdgeFaceNb, (*pGeo).EdgeQty + 1);
  ut_free_1d     ((*pGeo).EdgeLength);
  ut_free_1d_int ((*pGeo).EdgeState);
  ut_free_1d_int ((*pGeo).EdgeDel);

  ut_free_2d_int ((*pGeo).FacePoly, (*pGeo).FaceQty + 1);
  ut_free_1d_int ((*pGeo).FaceVerQty);
  ut_free_2d_int ((*pGeo).FaceVerNb, (*pGeo).FaceQty + 1);
  ut_free_2d_int ((*pGeo).FaceEdgeNb, (*pGeo).FaceQty + 1);
  ut_free_2d_int ((*pGeo).FaceEdgeOri, (*pGeo).FaceQty + 1);
  ut_free_2d     ((*pGeo).FaceEq, (*pGeo).FaceQty + 1);
  ut_free_1d_int ((*pGeo).FaceState);
  ut_free_1d_int ((*pGeo).FacePt);
  ut_free_2d     ((*pGeo).FacePtCoo, (*pGeo).FaceQty + 1);
  ut_free_1d     ((*pGeo).FaceFF);
  ut_free_1d_int ((*pGeo).FaceBound);

  ut_free_2d     ((*pGeo).CenterCoo, (*pGeo).PolyQty + 1);

  ut_free_1d_int ((*pGeo).PolyFaceQty);
  ut_free_2d_int ((*pGeo).PolyFaceNb, (*pGeo).PolyQty + 1);
  ut_free_2d_int ((*pGeo).PolyFaceOri, (*pGeo).PolyQty + 1);

  ut_free_2d ((*pGeo).DomainSize, 3);
  
  ut_free_1d_int ((*pGeo).PolyTrue);
  ut_free_1d_int ((*pGeo).PolyBody);

  neut_geo_set_zero (pGeo);

  return;
}

int
neut_geo_face_bound (struct GEO Geo, int face)
{
  int i, k, ver, res;
  int *tmp = ut_alloc_1d_int (100);

  for (i = 1; i <= Geo.FaceVerQty[face]; i++)
  {
    ver = Geo.FaceVerNb[face][i];
    for (k = 1; k <= Geo.VerBound[ver][0]; k++)
      tmp[++tmp[0]] = Geo.VerBound[ver][k];
  }
  
  res = 0;
  for (i = 1; i <= 6; i++)
    if (ut_array_1d_int_nbofthisval (tmp + 1, tmp[0], -i) == Geo.FaceVerQty[face])
    {
      res = -i;
      break;
    }
    
  ut_free_1d_int (tmp);

  return res;
}

void
neut_geo_poly_ver (struct GEO Geo, int nb, int* ver)
{
  int i, j, face;

  ver[0] = 0;
  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
  {
    face = Geo.PolyFaceNb[nb][i];
    for (j = 1; j <= Geo.FaceVerQty[face]; j++)
    {
      if (ut_array_1d_int_eltpos (ver + 1, ver[0], Geo.FaceVerNb[face][j]) == -1)
	ver[++ver[0]] = Geo.FaceVerNb[face][j];
    }
  }

  ut_array_1d_int_sort (ver + 1, ver[0]);

  return;
}

void
neut_geo_poly_edge (struct GEO Geo, int nb, int* edge)
{
  int i, j, face;

  edge[0] = 0;
  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
  {
    face = Geo.PolyFaceNb[nb][i];
    for (j = 1; j <= Geo.FaceVerQty[face]; j++)
      if (ut_array_1d_int_eltpos (edge + 1, edge[0], Geo.FaceEdgeNb[face][j]) == -1)
      edge[++edge[0]] = Geo.FaceEdgeNb[face][j];
  }
  
  ut_array_1d_int_sort (edge + 1, edge[0]);

  return;
}

void
neut_geo_edge_poly (struct GEO Geo, int edge, int* poly)
{
  int i, j, face, p; 

  /* the list of poly of an edge is the union of the poly of its faces.
   */

  poly[0] = 0;
  for (i = 0; i < Geo.EdgeFaceQty[edge]; i++)
  {
    face = Geo.EdgeFaceNb[edge][i];

    for (j = 0; j < 2; j++)
    {
      p = Geo.FacePoly[face][j];
      
      if (ut_array_1d_int_eltpos (poly + 1, poly[0], p) == -1)
	poly[++poly[0]] = p;
    }
  }

  ut_array_1d_int_sort (poly + 1, poly[0]);
  
  for (i = -1; i >= -6; i--)
    poly[0] -= ut_array_1d_int_deletencompress (poly + 1, poly[0], i, 1);
  
  return;
}


void
neut_geo_ver_face (struct GEO Geo, int ver, int *face)
{
  int i, j, tmpedge;

  face[0] = 0;
  for (i = 0; i <= Geo.VerEdgeQty[ver] - 1; i++)
  {
    tmpedge = Geo.VerEdgeNb[ver][i];
    for (j = 0; j <= Geo.EdgeFaceQty[tmpedge] - 1; j++)
      face[++face[0]] = Geo.EdgeFaceNb[tmpedge][j];
  }

  ut_array_1d_int_sortncompress (face + 1, face[0], &(face[0]));

  return;
}

void
neut_geo_ver_poly (struct GEO Geo, int ver, int *poly)
{
  int i, j;
  int *faces = ut_alloc_1d_int (1000);
  int face;

  neut_geo_ver_face (Geo, ver, faces);

  poly[0] = 0;
  for (i = 1; i <= faces[0]; i++)
  {
    face = faces[i];
    for (j = 0; j < 2; j++)
      poly[++poly[0]] = Geo.FacePoly[face][j];
  }

  ut_array_1d_int_sortncompress (poly + 1, poly[0], &(poly[0]));

  ut_free_1d_int (faces);

  return;
}

/* True stuff ------------------------------------------------------- */

/* Searching the true polyhedra */
int
neut_geo_poly_true (struct GEO Geo, int nb)
{
  int i, j, k, res;
  double ctov;
  double *coo = ut_alloc_1d (3);
  int* ver = ut_alloc_1d_int (1000);

  neut_geo_poly_ver (Geo, nb, ver);

  res = 1;
  for (i = 1; i <= ver[0]; i++)
  {
    ctov = ut_space_dist (Geo.CenterCoo[nb], Geo.VerCoo[ver[i]]);

    for (j = 0; j < 3; j++)
      for (k = 0; k < 2; k++)
	if (fabs (Geo.VerCoo[ver[i]][j] - Geo.DomainSize[j][k]) < 1.001 * ctov)
	  {
	    res = 0;
	    break;
	  }
  }

  ut_free_1d (coo);
  ut_free_1d_int (ver);

  return res;
}

int
neut_geo_poly_body (struct GEO Geo, int nb)
{
  int i, face, res;

  res = 1;
  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
  {
    face = Geo.PolyFaceNb[nb][i]; 
    if (Geo.FacePoly[face][0] < 0 || Geo.FacePoly[face][1] < 0)
    {
      res = 0;
      break;
    }
  }

  return res;
}

/* returns 0 if not true, 1 if true & 2 if all parent poly are true */
int
neut_geo_edge_true (struct GEO Geo, int *TruePoly, int nb)
{
  int j;
  int res, all;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_edge_poly (Geo, nb, poly);

  res = 0;
  all = 1;
  for (j = 1; j <= poly[0]; j++)
  {
    if (poly[j] > 0)
    {
      if (TruePoly[poly[j]] > 0)
	res = 1;
      else
	all = 0;
    }
  }

  if (res == 1 && all == 1)
    res = 2;

  ut_free_1d_int (poly);

  return res;
}

/* returns 0 if not body, 1 if body & 2 if all parent poly are body */
int
neut_geo_edge_body (struct GEO Geo, int *BodyPoly, int nb)
{
  int j;
  int res, all;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_edge_poly (Geo, nb, poly);

  res = 0;
  all = 1;
  for (j = 1; j <= poly[0]; j++)
  {
    if (poly[j] > 0)
    {
      if (BodyPoly[poly[j]] > 0)
	res = 1;
      else
	all = 0;
    }
  }

  if (res == 1 && all == 1)
    res = 2;

  ut_free_1d_int (poly);

  return res;
}


/* returns 0 if not true, 1 if true & 2 if all parent poly are true */
int
neut_geo_face_true (struct GEO Geo, int *TruePoly, int nb)
{
  int j;
  int res, all;

  res = 0;
  all = 1;
  for (j = 0; j < 2; j++)
  {
    if (Geo.FacePoly[nb][j] > 0)
    {
      if (TruePoly[Geo.FacePoly[nb][j]] > 0)
	res = 1;
      else
	all = 0;
    }
    else
    {
      res = 0;
      break;
    }
  }

  if (res == 1 && all == 1)
    res = 2;

  return res;
}

/* returns 0 if not true, 1 if true & 2 if all parent poly are true */
int
neut_geo_ver_true (struct GEO Geo, int *TruePoly, int nb)
{
  int j;
  int res, all;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_ver_poly (Geo, nb, poly);

  res = 0;
  all = 1;
  for (j = 1; j <= poly[0]; j++)
  {
    if (poly[j] > 0)
    {
      if (TruePoly[poly[j]] > 0)
	res = 1;
      else
	all = 0;
    }
  }

  if (res == 1 && all == 1)
    res = 2;

  ut_free_1d_int (poly);

  return res;
}

int
neut_geo_edge_polytruelevelmax (struct GEO Geo, int *TruePoly, int nb)
{
  int j, res;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_edge_poly (Geo, nb, poly);

  res = 0;
  for (j = 1; j <= poly[0]; j++)
    if (poly[j] > 0)
      res = ut_num_max_int (res, TruePoly[poly[j]]);

  ut_free_1d_int (poly);

  return res;
}

int
neut_geo_edge_polybodylevelmax (struct GEO Geo, int *BodyPoly, int nb)
{
  int j, res;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_edge_poly (Geo, nb, poly);

  res = 0;
  for (j = 1; j <= poly[0]; j++)
    if (poly[j] > 0)
      res = ut_num_max_int (res, BodyPoly[poly[j]]);

  ut_free_1d_int (poly);

  return res;
}

int
neut_geo_face_polybodylevelmax (struct GEO Geo, int nb)
{
  int res = 0;

  if (Geo.FacePoly[nb][0] > 0)
    res = Geo.PolyBody[Geo.FacePoly[nb][0]];
  if (Geo.FacePoly[nb][1] > 0)
    res = ut_num_max_int (res, Geo.PolyBody[Geo.FacePoly[nb][1]]);

  return res;
}

int
neut_geo_ver_polytruelevelmax (struct GEO Geo, int *TruePoly, int nb)
{
  int j, res;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_ver_poly (Geo, nb, poly);

  res = 0;
  for (j = 1; j <= poly[0]; j++)
    if (poly[j] > 0)
      res = ut_num_max_int (res, TruePoly[poly[j]]);

  ut_free_1d_int (poly);

  return res;
}

int
neut_geo_ver_polybodylevelmax (struct GEO Geo, int *BodyPoly, int nb)
{
  int j, res;
  int* poly = ut_alloc_1d_int (100);

  neut_geo_ver_poly (Geo, nb, poly);

  res = 0;
  for (j = 1; j <= poly[0]; j++)
    if (poly[j] > 0)
      res = ut_num_max_int (res, BodyPoly[poly[j]]);

  ut_free_1d_int (poly);

  return res;
}

void
neut_geo_init_polytrue (struct GEO* pGeo)
{
  int i, j, all, face, p;
  int tru, status;
  
  if ((*pGeo).PolyTrue == NULL)
    (*pGeo).PolyTrue = ut_alloc_1d_int ((*pGeo).PolyQty + 1);

  for (i = 1; i <= (*pGeo).PolyQty; i++)
    (*pGeo).PolyTrue[i] = neut_geo_poly_true ((*pGeo), i);

  status = 1;
  for (tru = 2; status == 1; tru++)
  {
    status = 0;
    for (i = 1; i <= (*pGeo).PolyQty; i++)
      if ((*pGeo).PolyTrue[i] == tru - 1)
      {
	all = 1;
	for (j = 1; j <= (*pGeo).PolyFaceQty[i]; j++)
	{
	  face = (*pGeo).PolyFaceNb[i][j];
	  p = ((*pGeo).FacePoly[face][0] != i)? 
		(*pGeo).FacePoly[face][0] : (*pGeo).FacePoly[face][1];

	  if ((*pGeo).PolyTrue[p] < tru - 1)
	  {
	    all = 0;
	    break;
	  }
	}
	if (all == 1)
	{
	  (*pGeo).PolyTrue[i] = tru;
	  status = 1;
	}
      }
  }

  return;
}

void
neut_geo_init_polybody (struct GEO* pGeo)
{
  int i, j, all, face, p;
  int body, status;
  
  if ((*pGeo).PolyBody == NULL)
    (*pGeo).PolyBody = ut_alloc_1d_int ((*pGeo).PolyQty + 1);

  for (i = 1; i <= (*pGeo).PolyQty; i++)
    (*pGeo).PolyBody[i] = neut_geo_poly_body ((*pGeo), i);

  status = 1;
  for (body = 2; status == 1; body++)
  {
    status = 0;
    for (i = 1; i <= (*pGeo).PolyQty; i++)
      if ((*pGeo).PolyBody[i] == body - 1)
      {
	all = 1;
	for (j = 1; j <= (*pGeo).PolyFaceQty[i]; j++)
	{
	  face = (*pGeo).PolyFaceNb[i][j];
	  p = ((*pGeo).FacePoly[face][0] != i)? 
		(*pGeo).FacePoly[face][0] : (*pGeo).FacePoly[face][1];

	  if ((*pGeo).PolyBody[p] < body - 1)
	  {
	    all = 0;
	    break;
	  }
	}
	if (all == 1)
	{
	  (*pGeo).PolyBody[i] = body;
	  status = 1;
	}
      }
  }

  return;
}

void
neut_geo_del_polytrue (struct GEO* pGeo)
{
  ut_free_1d_int ((*pGeo).PolyTrue);

  return;
}

void
neut_geo_del_polybody (struct GEO* pGeo)
{
  ut_free_1d_int ((*pGeo).PolyBody);

  return;
}

void
neut_geo_fod (struct GEO Geo, int*** pFoD)
{
  int i, bound;

  (*pFoD) = ut_alloc_2d_int (7, 1);

  for (i = 1; i <= Geo.FaceQty; i++)
  {
    bound = neut_geo_face_bound (Geo, i);

    if (bound != 0)
    {
      (*pFoD)[-bound][0]++;
      (*pFoD)[-bound] = ut_realloc_1d_int ((*pFoD)[-bound],
					   1 + (*pFoD)[-bound][0]);
      (*pFoD)[-bound][(*pFoD)[-bound][0]] = i;
    }
  }

  return;
}

int
neut_geo_edge_selratio (struct GEO Geo, struct GEOPARA GeoPara, int edge,
                  double* pratio)
{
  double sel;
  
  if (GeoPara.dboundcl < 0)
    sel = GeoPara.sel;
  else 
  {
    int var_qty = 2;
    char** vars  = ut_alloc_2d_char (var_qty, 15);
    double* vals = ut_alloc_1d      (var_qty);
    int status = -1;
    double dbound;

    sprintf (vars[0], "body");
    sprintf (vars[1], "true");
    vals[0] = neut_geo_edge_polybodylevelmax (Geo, Geo.PolyBody, edge);
    vals[1] = neut_geo_edge_polytruelevelmax (Geo, Geo.PolyTrue, edge);

    status = ut_math_eval (GeoPara.dbound, var_qty, vars, vals, &dbound);

    if (status == -1)
      abort ();

    if (dbound == 1)
      sel = GeoPara.dboundsel;           // means on bound
    else
      sel = GeoPara.sel;

    ut_free_2d_char (vars, var_qty);
    ut_free_1d (vals);
  }

  if (Geo.EdgeState[edge] == 0)
  {
    (*pratio) = Geo.EdgeLength[edge] / sel;
    return 0;
  }
  else
    return 1;
}

int
neut_geo_face_area (struct GEO Geo, int face, double* parea)
{
  int i, ver;
  double* p0 = NULL;
  double* p1 = NULL;
  double* p2 = NULL;

  if (face < 1 || face > Geo.FaceQty || Geo.FaceState[face] == -1)
  {
    (*parea) = 0;
    return -1;
  }

  if (Geo.FaceState[face] > 0)
  {
    if (Geo.FacePt[face] == 0)
      p0 = Geo.FacePtCoo[face];
    else
    {
      ver = Geo.FaceVerNb[face][Geo.FacePt[face]];
      p0 = Geo.VerCoo[ver];
    }
  }
  else
  {
    ver = Geo.FaceVerNb[face][1];
    p0 = Geo.VerCoo[ver];
  }

  (*parea) = 0;
  for (i = 1; i < Geo.FaceVerQty[face]; i++)
  {
    p1 = Geo.VerCoo[Geo.FaceVerNb[face][i]];
    p2 = Geo.VerCoo[Geo.FaceVerNb[face][i + 1]];
    (*parea) += ut_space_triangle_area (p0, p1, p2);
  }

  p1 = Geo.VerCoo[Geo.FaceVerNb[face][Geo.FaceVerQty[face]]];
  p2 = Geo.VerCoo[Geo.FaceVerNb[face][1]];
  (*parea) += ut_space_triangle_area (p0, p1, p2);

  return 0;
}

int
neut_geo_volume (struct GEO Geo, double* pvol)
{
  int i;
  double tmp;

  (*pvol) = 0;
  for (i = 1; i <= Geo.PolyQty; i++)
  {
    neut_geo_poly_volume (Geo, i, &tmp);
    (*pvol) += tmp;
  }

  return 0;
}

int
neut_geo_poly_volume (struct GEO Geo, int poly, double* pvol)
{
  int i, ver, f, face;
  double area, h;
  double* p0 = NULL;
  double* p1 = NULL;
  double* p2 = NULL;
  double* proj = ut_alloc_1d (3);
  double* eq = ut_alloc_1d (4);

  // Principle: sum the volumes of the pyramids based on the faces and
  // the poly centre. For the non-plane faces, we consider the
  // individual triangular parts (no approximation)

  if (poly < 1 || poly > Geo.PolyQty)
  {
    (*pvol) = 0;
    return -1;
  }

  (*pvol) = 0;
  for (f = 1; f <= Geo.PolyFaceQty[poly]; f++)
  {
    face = Geo.PolyFaceNb[poly][f];

    if (Geo.FaceState[face] > 0)
    {
      if (Geo.FacePt[face] == 0)
	p0 = Geo.FacePtCoo[face];
      else
      {
	ver = Geo.FaceVerNb[face][Geo.FacePt[face]];
	p0 = Geo.VerCoo[ver];
      }
    }
    else
    {
      ver = Geo.FaceVerNb[face][1];
      p0 = Geo.VerCoo[ver];
    }

    for (i = 1; i < Geo.FaceVerQty[face]; i++)
    {
      p1 = Geo.VerCoo[Geo.FaceVerNb[face][i]];
      p2 = Geo.VerCoo[Geo.FaceVerNb[face][i + 1]];
      area = ut_space_triangle_area (p0, p1, p2);
      if (area > 1e-20)
      {
	ut_array_1d_memcpy (proj, 3, Geo.CenterCoo[poly]);
	ut_space_points_plane (p0, p1, p2, eq);
	ut_space_projpoint_alongonto (proj, eq + 1, eq);
	h = ut_space_dist (Geo.CenterCoo[poly], proj);
	(*pvol) += area * h * 0.3333333333333333333333;
      }
    }

    p1 = Geo.VerCoo[Geo.FaceVerNb[face][Geo.FaceVerQty[face]]];
    p2 = Geo.VerCoo[Geo.FaceVerNb[face][1]];
    area = ut_space_triangle_area (p0, p1, p2);
    if (area > 1e-20)
    {
      ut_array_1d_memcpy (proj, 3, Geo.CenterCoo[poly]);
      ut_space_points_plane (p0, p1, p2, eq);
      ut_space_projpoint_alongonto (proj, eq + 1, eq);
      h = ut_space_dist (Geo.CenterCoo[poly], proj);
      (*pvol) += area * h * 0.3333333333333333333333;
    }
  }

  // don't free p0, p1, p2 (shortcuts)
  ut_free_1d (proj);
  ut_free_1d (eq);

  return 0;
}

void
neut_geo_edge_length (struct GEO *pGeo, int edge)
{
  int ver1, ver2;

  ver1 = (*pGeo).EdgeVerNb[edge][0];
  ver2 = (*pGeo).EdgeVerNb[edge][1];

  (*pGeo).EdgeLength[edge] = ut_space_dist ((*pGeo).VerCoo[ver1],
      (*pGeo).VerCoo[ver2]);

  return;
}

void
neut_geo_init_edgelength (struct GEO *pGeo)
{
  int i;

  ut_free_1d ((*pGeo).EdgeLength);
  (*pGeo).EdgeLength = ut_alloc_1d ((*pGeo).EdgeQty + 1);

  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    neut_geo_edge_length (pGeo, i);

  return;
}

// initializes some variables or a Geo from FacePoly:
// PolyQty, PolyFaceQty & PolyFaceNb
void
neut_geo_init_polystuff_fromfacepoly (struct GEO* pGeo)
{
  int i, j, poly;

  // init PolyFaceQty & PolyFaceNb
  (*pGeo).PolyQty = 0;
  for (i = 1; i <= (*pGeo).FaceQty; i++)
    for (j = 0; j < 2; j++)
      (*pGeo).PolyQty = ut_num_max ((*pGeo).PolyQty, (*pGeo).FacePoly[i][j]);

  (*pGeo).PolyFaceQty = ut_alloc_1d_int ((*pGeo).PolyQty + 1);
  (*pGeo).PolyFaceNb = ut_alloc_2d_int ((*pGeo).PolyQty + 1, 1);

  for (i = 1; i <= (*pGeo).FaceQty; i++)
    for (j = 0; j < 2; j++)
      if ((*pGeo).FacePoly[i][j] > 0)
      {
	poly = (*pGeo).FacePoly[i][j];

	(*pGeo).PolyFaceQty[poly]++;
	(*pGeo).PolyFaceNb[poly] = ut_realloc_1d_int ((*pGeo).PolyFaceNb[poly], (*pGeo).PolyFaceQty[poly] + 1);
	(*pGeo).PolyFaceNb[poly][(*pGeo).PolyFaceQty[poly]] = i;
      }

  return;
}

void
neut_geo_init_edgestuff_fromver (struct GEO* pGeo)
{
  int i, j, edge;

  (*pGeo).EdgeVerNb = ut_alloc_2d_int ((*pGeo).EdgeQty + 1, 2);

  for (i = 1; i <= (*pGeo).VerQty; i++)
    for (j = 0; j < (*pGeo).VerEdgeQty[i]; j++)
    {
      edge = (*pGeo).VerEdgeNb[i][j];
      if ((*pGeo).EdgeVerNb[edge][0] == 0)
	(*pGeo).EdgeVerNb[edge][0] = i;
      else if ((*pGeo).EdgeVerNb[edge][1] == 0)
	(*pGeo).EdgeVerNb[edge][1] = i;
      else
	ut_error_reportbug ();
    }

  neut_geo_init_edgelength (pGeo);
  
  return;
}


void
neut_geo_init_facestuff_fromedge (struct GEO* pGeo)
{
  int i, j, face;

  (*pGeo).FaceVerQty = ut_alloc_1d_int ((*pGeo).FaceQty + 1);
  (*pGeo).FaceEdgeNb = ut_alloc_2d_int ((*pGeo).FaceQty + 1, 1);

  for (i = 1; i <= (*pGeo).EdgeQty; i++)
    for (j = 0; j < (*pGeo).EdgeFaceQty[i]; j++)
    {
      face = (*pGeo).EdgeFaceNb[i][j];
      (*pGeo).FaceVerQty[face]++;
      (*pGeo).FaceEdgeNb[face] = ut_realloc_1d_int ((*pGeo).FaceEdgeNb[face], (*pGeo).FaceVerQty[face] + 1);
      (*pGeo).FaceEdgeNb[face][(*pGeo).FaceVerQty[face]] = i;
    }

  // init FaceState
  (*pGeo).FaceState  = ut_alloc_1d_int ((*pGeo).FaceQty + 1);

  return;
}

void
neut_geo_init_facestuff_fromver (struct GEO* pGeo)
{
  int i, j, k, edge, ver, pos;
  int* perm = NULL;

  // permuting FaceEdgeNb and init FaceEdgeOri & FaceVerNb
  (*pGeo).FaceEdgeOri = ut_alloc_1d_pint ((*pGeo).FaceQty + 1);
  (*pGeo).FaceVerNb  = ut_alloc_1d_pint ((*pGeo).FaceQty + 1);
  for (i = 1; i <= (*pGeo).FaceQty; i++)
  {
    perm = ut_alloc_1d_int ((*pGeo).FaceVerQty[i] + 1);
    perm[1] = 1;
    edge = (*pGeo).FaceEdgeNb[i][1];
    ver = (*pGeo).EdgeVerNb[edge][1];
    // looking for the 2nd, 3rd, ... edges
    for (j = 2; j <= (*pGeo).FaceVerQty[i]; j++)
    {
      // checking every edges (we know it's not the 1st)
      for (k = 2; k <= (*pGeo).FaceVerQty[i]; k++)
	if (k != perm[j - 1])
	{
	  edge = (*pGeo).FaceEdgeNb[i][k];
	  pos = ut_array_1d_int_eltpos ((*pGeo).EdgeVerNb[edge], 2, ver);
	  if (pos != -1)
	  {
	    perm[j] = k;
	    ver = (pos == 0) ? (*pGeo).EdgeVerNb[edge][1]
	                     : (*pGeo).EdgeVerNb[edge][0];
	    break;
	  }
	}

      if (perm[j] == 0)
	ut_error_reportbug ();
    }

    ut_array_1d_permutation_int ((*pGeo).FaceEdgeNb[i], (*pGeo).FaceVerQty[i] + 1, perm);

    ut_free_1d_int (perm);

    // init FaceEdgeOri and FaceVerNb
    (*pGeo).FaceEdgeOri[i] = ut_alloc_1d_int ((*pGeo).FaceVerQty[i] + 1);
    (*pGeo).FaceVerNb[i]   = ut_alloc_1d_int ((*pGeo).FaceVerQty[i] + 1);

    edge = (*pGeo).FaceEdgeNb[i][1];
    ver = (*pGeo).EdgeVerNb[edge][1];
    (*pGeo).FaceEdgeOri[i][1] = 1;
    (*pGeo).FaceVerNb[i][1] = (*pGeo).EdgeVerNb[edge][0];
    (*pGeo).FaceVerNb[i][2] = (*pGeo).EdgeVerNb[edge][1];

    for (j = 2; j <= (*pGeo).FaceVerQty[i]; j++)
    {
      edge = (*pGeo).FaceEdgeNb[i][j];
      if ((*pGeo).EdgeVerNb[edge][0] == ver)
      {
	(*pGeo).FaceEdgeOri[i][j] = 1;
	if (j < (*pGeo).FaceVerQty[i])
	  (*pGeo).FaceVerNb[i][j + 1] = (*pGeo).EdgeVerNb[edge][1];
	ver = (*pGeo).EdgeVerNb[edge][1];
      }
      else if ((*pGeo).EdgeVerNb[edge][1] == ver)
      {
	(*pGeo).FaceEdgeOri[i][j] = -1;
	if (j < (*pGeo).FaceVerQty[i])
	  (*pGeo).FaceVerNb[i][j + 1] = (*pGeo).EdgeVerNb[edge][0];
	ver = (*pGeo).EdgeVerNb[edge][0];
      }
      else
	ut_error_reportbug ();
    }
  }

  // reverse FaceVerNb and FaceEdgeNb and modify FaceEdgeOri
  for (i = 1; i <= (*pGeo).FaceQty; i++)
    if (TestFaceNormal (*pGeo, i) != 0)
    {
      ut_array_1d_int_reverseelts ((*pGeo).FaceEdgeNb[i] + 2,
	                           (*pGeo).FaceVerQty[i] - 1);
      ut_array_1d_int_reverseelts ((*pGeo).FaceVerNb[i] + 2,
	                           (*pGeo).FaceVerQty[i] - 1);
      ut_array_1d_int_reverseelts ((*pGeo).FaceEdgeOri[i] + 2,
	                           (*pGeo).FaceVerQty[i] - 1);
      ut_array_1d_int_scale ((*pGeo).FaceEdgeOri[i] + 1,
			     (*pGeo).FaceVerQty[i] , -1);
    }

  // init FacePt, FacePtCoo, FaceFF, FaceBound
  (*pGeo).FacePt = ut_alloc_1d_int ((*pGeo).FaceQty + 1);
  ut_array_1d_int_set ((*pGeo).FacePt + 1, (*pGeo).FaceQty, -1);

  (*pGeo).FacePtCoo = ut_alloc_2d ((*pGeo).FaceQty + 1, 3);
  (*pGeo).FaceFF = ut_alloc_1d ((*pGeo).FaceQty + 1);
  (*pGeo).FaceBound = ut_alloc_1d_int ((*pGeo).FaceQty + 1);

  for (i = 1; i <= (*pGeo).FaceQty; i++)
  {
    // neut_geo_face_init_ff (pGeo, i);
    (*pGeo).FaceBound[i] = neut_geo_face_bound (*pGeo, i);
  }

  return;
}


void
neut_geo_polys_skinfaces (struct GEO Geo, int* poly, int polyqty, int**
    pfaces, int** pfacepoly, int* pfaceqty)
{
  int i, j, k, pos, p, f;

  // allocation
  (*pfaceqty) = 0;
  for (i = 0; i < polyqty; i++)
  {
    p = poly[i];
    (*pfaceqty) += Geo.PolyFaceQty[p];
  }
  (*pfaces) = ut_alloc_1d_int ((*pfaceqty));
  (*pfacepoly) = ut_alloc_1d_int ((*pfaceqty));

  (*pfaceqty) = 0;
  for (i = 0; i < polyqty; i++)
  {
    p = poly[i];
    for (j = 1; j <= Geo.PolyFaceQty[p]; j++)
    {
      f = Geo.PolyFaceNb[p][j];
      pos = ut_array_1d_int_eltpos (*pfaces, *pfaceqty, f);
      if (pos == -1)
      {
	(*pfaces)[(*pfaceqty)] = f;
	(*pfacepoly)[(*pfaceqty)] = p;
	(*pfaceqty)++;
      }
      else
      {
	for (k = pos; k < (*pfaceqty) - 1; k++)
	{
	  (*pfaces)[k] = (*pfaces)[k + 1];
	  (*pfacepoly)[k] = (*pfacepoly)[k + 1];
	}
	(*pfaceqty)--;
      }
    }
  }
  (*pfaces) = ut_realloc_1d_int ((*pfaces), (*pfaceqty));

  return;
}


void
neut_geo_boundingbox (struct GEO Geo, double** size)
{
  int i, j;

  /* Searching the tessellation bounding box */
  size[0][0] = size[1][0] = size[2][0] = DBL_MAX;
  size[0][1] = size[1][1] = size[2][1] = -DBL_MAX;

  for (i = 1; i <= Geo.VerQty; i++)
    for (j = 0; j < 3; j++)
    {
      size[j][0] = ut_num_min (size[j][0], Geo.VerCoo[i][j]);
      size[j][1] = ut_num_max (size[j][1], Geo.VerCoo[i][j]);
    }

  return;
}

void
neut_geo_init_boundingbox (struct GEO* pGeo)
{
  if ((*pGeo).DomainSize == NULL)
    (*pGeo).DomainSize = ut_alloc_2d (3, 2);

  neut_geo_boundingbox (*pGeo, (*pGeo).DomainSize);

}

double
neut_geo_face_init_ff (struct GEO *pGeo, int face)
{
  int i, j, k;
  int ver1, ver2, ver3;
  double ff = 180;
  double minff = 180;
  double thisff = 180;
  double** eq   = ut_alloc_2d ((*pGeo).FaceVerQty[face] + 1, 4);
  double* angle = ut_alloc_1d ((*pGeo).FaceVerQty[face] + 1);

  /* for face point on each ver of the face, the fff is calculated:
   * fff = maximum angle between 2 triangular parts of the face (see
   * the method of interpolation of the face).
   */
  for (i = 1; i <= (*pGeo).FaceVerQty[face]; i++)
  {
    thisff = 0;

    /* ver i is the face point. */
    ver1 = (*pGeo).FaceVerNb[face][i];

    for (j = 1; j <= (*pGeo).FaceVerQty[face] - 2; j++)
    {
      ver2 =
	(*pGeo).FaceVerNb[face][ut_num_rotpos
				(1, (*pGeo).FaceVerQty[face], i, j)];
      ver3 =
	(*pGeo).FaceVerNb[face][ut_num_rotpos
				(1, (*pGeo).FaceVerQty[face], i, j + 1)];

      ut_space_points_plane ((*pGeo).VerCoo[ver1], (*pGeo).VerCoo[ver2],
		 (*pGeo).VerCoo[ver3], eq[j]);
    }
    for (j = 1; j <= (*pGeo).FaceVerQty[face] - 2; j++)
      for (k = j + 1; k <= (*pGeo).FaceVerQty[face] - 2; k++)
      {
	ff = ut_vector_angle (eq[j] + 1, eq[k] + 1);
	thisff = ut_num_max (thisff, ff);
      }

    if (thisff < minff)
    {
      minff = thisff;
      (*pGeo).FacePt[face] = i;
    }
  }

  // case of the barycenter 
  
  // calculating coo
  ut_array_1d_zero ((*pGeo).FacePtCoo[face], 3);
  for (i = 1; i <= (*pGeo).FaceVerQty[face]; i++)
  {
    ver1 = (*pGeo).FaceVerNb[face][i];
    ut_array_1d_add ((*pGeo).FacePtCoo[face], (*pGeo).VerCoo[ver1], 3, (*pGeo).FacePtCoo[face]);
  }
  ut_array_1d_scale ((*pGeo).FacePtCoo[face], 3, 1./(*pGeo).FaceVerQty[face]);

  thisff = 0;
  for (j = 1; j <= (*pGeo).FaceVerQty[face]; j++)
  {
    ver2 =
      (*pGeo).FaceVerNb[face][ut_num_rotpos (1, (*pGeo).FaceVerQty[face], i, j)];
    ver3 =
      (*pGeo).FaceVerNb[face][ut_num_rotpos
			      (1, (*pGeo).FaceVerQty[face], i, j + 1)];

    ut_space_points_plane ((*pGeo).FacePtCoo[face], (*pGeo).VerCoo[ver2],
	       (*pGeo).VerCoo[ver3], eq[j]);
  }
  for (j = 1; j <= (*pGeo).FaceVerQty[face]; j++)
    for (k = j + 1; k <= (*pGeo).FaceVerQty[face] - 2; k++)
    {
      ff = ut_vector_angle (eq[j] + 1, eq[k] + 1);
      thisff = ut_num_max (thisff, ff);
    }

  if (thisff < minff)
  {
    (*pGeo).FacePt[face] = 0;
    minff = thisff;
  }
  else
  {
    ver1 = (*pGeo).FaceVerNb[face][(*pGeo).FacePt[face]];

    ut_array_1d_memcpy ((*pGeo).FacePtCoo[face], 3, (*pGeo).VerCoo[ver1]);
  }

  (*pGeo).FaceFF[face] = minff;

  ut_free_2d (eq, (*pGeo).FaceVerQty[face] + 1);
  ut_free_1d (angle);

  return (*pGeo).FaceFF[face];
}

void
neut_geo_poly_switch (struct GEO* pGeo, int p1, int p2)
{
  int i, id;

  if (p1 == p2)
    return;

  // CenterCoo
  ut_array_2d_switchlines ((*pGeo).CenterCoo, 3, p1, p2);
  
  // PolyTrue
  if ((*pGeo).PolyTrue != NULL)
    ut_array_1d_int_switch ((*pGeo).PolyTrue, p1, p2);

  // PolyBody
  if ((*pGeo).PolyBody != NULL)
    ut_array_1d_int_switch ((*pGeo).PolyBody, p1, p2);
  
  // PolyFace* (keep FaceQty at the end)
  ut_array_2d_int_switchlines_length ((*pGeo).PolyFaceNb,
			       p1, (*pGeo).PolyFaceQty[p1] + 1,
			       p2, (*pGeo).PolyFaceQty[p2] + 1);

  ut_array_2d_int_switchlines_length ((*pGeo).PolyFaceOri,
			       p1, (*pGeo).PolyFaceQty[p1] + 1,
			       p2, (*pGeo).PolyFaceQty[p2] + 1);
  
  ut_array_1d_int_switch ((*pGeo).PolyFaceQty, p1, p2);

  // FacePoly | the following is to avoid problems when p1 and p2 share
  // a common face
  for (i = 1; i <= (*pGeo).PolyFaceQty[p1]; i++)
  {
    id = (*pGeo).PolyFaceNb[p1][i];
    ut_array_1d_int_findnreplace ((*pGeo).FacePoly[id], 2, p2, -1000);
  }
  for (i = 1; i <= (*pGeo).PolyFaceQty[p2]; i++)
  {
    id = (*pGeo).PolyFaceNb[p2][i];
    ut_array_1d_int_findnreplace ((*pGeo).FacePoly[id], 2, p1, p2);
  }
  for (i = 1; i <= (*pGeo).PolyFaceQty[p1]; i++)
  {
    id = (*pGeo).PolyFaceNb[p1][i];
    ut_array_1d_int_findnreplace ((*pGeo).FacePoly[id], 2, -1000, p1);
  }

  return;
}

void
neut_geo_face_centre (struct GEO Geo, int face, double* centre)
{
  int i, ver;

  ut_array_1d_set (centre, 3, 0);
  for (i = 1; i <= Geo.FaceVerQty[face]; i++)
  {
    ver = Geo.FaceVerNb[face][i];
    ut_array_1d_add (centre, Geo.VerCoo[ver], 3, centre);
  }
  ut_array_1d_scale (centre, 3, 1./Geo.FaceVerQty[face]);

  return;
}

int
neut_geo_edge_centre (struct GEO Geo, int edge, double* pos)
{
  int i;

  for (i = 0; i < 3; i++)
    pos[i] = (Geo.VerCoo[Geo.EdgeVerNb[edge][0]][i]
	    + Geo.VerCoo[Geo.EdgeVerNb[edge][1]][i]) / 2.;
  
  return 0;
}

void
neut_geo_centre (struct GEO Geo, double* centre)
{
  int i;

  if (Geo.DomainSize == NULL)
  {
    printf ("neut_geo_centre: DomainSize is NULL\n");
    abort ();
  }

  for (i = 0; i < 3; i++)
    centre[i] = (Geo.DomainSize[i][1] - Geo.DomainSize[i][0]) / 2;

  return;
}

void
neut_geo_poly_neighpoly (struct GEO Geo, int poly, int* neigh, int* pneighqty)
{
  int i, j, face;

  (*pneighqty) = 0;
  for (i = 1; i <= Geo.PolyFaceQty[poly]; i++)
  {
    face = Geo.PolyFaceNb[poly][i];

    for (j = 0; j < 2; j++)
      if (Geo.FacePoly[face][j] > 0 && Geo.FacePoly[face][j] != poly)
	neigh[(*pneighqty)++] = Geo.FacePoly[face][j];
  }

  return;
}

void
neut_geo_poly_edge_face (struct GEO Geo, int poly, int edge, int* face)
{
  int i, f, qty = 0;

  for (i = 0; i < Geo.EdgeFaceQty[edge]; i++)
  {
    f = Geo.EdgeFaceNb[edge][i];
    if (ut_array_1d_int_eltpos (Geo.FacePoly[f], 2, poly) != -1)
      face[qty++] = f;
  }

  if (qty != 2)
    ut_error_reportbug ();

  return;
}

void
neut_geo_face_edge_ori (struct GEO Geo, int face, int edge, int* pedgeori)
{
  int pos;

  pos = 1 + ut_array_1d_int_eltpos (Geo.FaceEdgeNb[face] + 1,
				    Geo.FaceVerQty[face], edge);
  if (pos == 0)
    (*pedgeori) = -1;
  else
    (*pedgeori) = Geo.FaceEdgeOri[face][pos];

  return;
}

void
neut_geo_poly_face_ori (struct GEO Geo, int poly, int face, int* pfaceori)
{
  int pos;

  pos = 1 + ut_array_1d_int_eltpos (Geo.PolyFaceNb[poly] + 1,
				    Geo.PolyFaceQty[poly], face);
  if (pos == 0)
    (*pfaceori) = -1;
  else
    (*pfaceori) = Geo.PolyFaceOri[poly][pos];

  return;
}

int
neut_geo_expr_polytab (struct GEO Geo, char* expr, int* poly)
{
  int i, id;
  FILE* file;

  if (strcmp (expr, "all") == 0)
    ut_array_1d_int_set (poly + 1, Geo.PolyQty, 1);

  else if (expr[0] == '@')
  {
    file = ut_file_open (expr + 1, "r");
    while (fscanf (file, "%d", &id) != EOF)
      poly[id] = 1;
    ut_file_close (file, expr + 1, "r");
  }
  else
  {
    int var_qty = 7;
    char **vars = ut_alloc_2d_char (var_qty, 15);
    double *vals = ut_alloc_1d (var_qty);
    int status;
    double res;

    sprintf (vars[0], "cenx");
    sprintf (vars[1], "ceny");
    sprintf (vars[2], "cenz");
    sprintf (vars[3], "volume");
    sprintf (vars[4], "true");
    sprintf (vars[5], "body");
    sprintf (vars[6], "id");

    for (i = 1; i <= Geo.PolyQty; i++)
    {
      ut_array_1d_memcpy (vals, 3, Geo.CenterCoo[i]);
      neut_geo_poly_volume (Geo, i, &(vals[3]));
      vals[4] = Geo.PolyTrue[i];
      vals[5] = Geo.PolyBody[i];
      vals[6] = i;
#if HAVE_LIBMATHEVAL == 1
      status = ut_math_eval (expr, var_qty, vars, vals, &res);
#else
ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
abort ();
#endif
      if (status == -1)
	abort ();
      poly[i] = ut_num_equal (res, 1, 1e-6);
    }

    ut_free_2d_char (vars, var_qty);
    ut_free_1d (vals);
  }

  return 0;
}

int
neut_geo_expr_facetab (struct GEO Geo, char* expr, int* face)
{
  int i, id;
  FILE* file;

  if (strcmp (expr, "all") == 0)
    ut_array_1d_int_set (face + 1, Geo.FaceQty, 1);

  else if (expr[0] == '@')
  {
    file = ut_file_open (expr + 1, "r");
    while (fscanf (file, "%d", &id) != EOF)
      face[id] = 1;
    ut_file_close (file, expr + 1, "r");
  }
  else
  {
    int var_qty = 7;
    char **vars = ut_alloc_2d_char (var_qty, 15);
    double *vals = ut_alloc_1d (var_qty);
    int status;
    double res;

    sprintf (vars[0], "cenx");
    sprintf (vars[1], "ceny");
    sprintf (vars[2], "cenz");
    sprintf (vars[3], "volume");
    sprintf (vars[4], "true");
    sprintf (vars[5], "body");
    sprintf (vars[6], "id");

    for (i = 1; i <= Geo.FaceQty; i++)
    {
      neut_geo_face_centre (Geo, i, vals);
      neut_geo_face_area (Geo, i, &(vals[3]));
      vals[4] = neut_geo_face_true (Geo, Geo.PolyTrue, i);
      vals[5] = 0;
      vals[6] = i;

#if HAVE_LIBMATHEVAL == 1
      status = ut_math_eval (expr, var_qty, vars, vals, &res);
#else
ut_print_messagewnc (2, 72, "This capability is not available because this version of Neper has not been compiled with libmatheval.");
abort ();
#endif
      if (status == -1)
	abort ();
      face[i] = ut_num_equal (res, 1, 1e-6);
    }

    ut_free_2d_char (vars, var_qty);
    ut_free_1d (vals);
  }

  return 0;
}

int
neut_geo_poly_skinmesh (struct GEO Geo, int nb,
		        struct NODES* pNodes, struct MESH* pMesh)
{
  int i, j, face;
  int* ver = ut_alloc_1d_int (1000);
  int* verinv = NULL;
  int verinvqty;
  int node0;
  int verqty;
  int* eltnodes = ut_alloc_1d_int (3);

  neut_mesh_set_zero  (pMesh );
  neut_nodes_set_zero (pNodes);
  neut_mesh_addelset (pMesh, NULL, 0);
  (*pMesh).EltOrder = 1;
  (*pMesh).Dimension = 2;
  (*pMesh).EltType = ut_alloc_1d_char (4);
  sprintf ((*pMesh).EltType, "tri");

  // adding poly vertices and calculating the inverse array
  neut_geo_poly_ver (Geo, nb, ver);
  ut_array_1d_int_inv (ver + 1, ver[0], &verinv, &verinvqty);
  ut_array_1d_int_addval (verinv, verinvqty, 1, verinv);

  for (i = 1; i <= ver[0]; i++)
    neut_nodes_addnode (pNodes, Geo.VerCoo[ver[i]], 1);

  // for each face, recording a new node if necessary and recording the
  // elements
  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
  {
    face = Geo.PolyFaceNb[nb][i];

    verqty = Geo.FaceVerQty[face];

    // adding node if defined as the barycenter
    if (Geo.FaceState[face] == 0)
    {
      neut_geo_face_centre (Geo, face, Geo.FacePtCoo[face]);
      neut_nodes_addnode (pNodes, Geo.FacePtCoo[face], 1);
    }
    else if (Geo.FacePt[face] == 0)
    {
      neut_nodes_addnode (pNodes, Geo.FacePtCoo[face], 1);
    }

    // looking for the master node (as recorded in Nodes)
    node0 = (Geo.FacePt[face] != 0) ?
            verinv[Geo.FaceVerNb[face][Geo.FacePt[face]]]
	  : (*pNodes).NodeQty;

    for (j = 1; j <= verqty; j++)
    {
      // if p0 is a face vertex: 
      // if the ver is p0 or the previous vertex, it is skipped
      if (j == Geo.FacePt[face]
	  || ut_num_rotpos (1, verqty, j, 1) == Geo.FacePt[face])
	continue;

      // recording coo of the 1st and 2nd pts
      eltnodes[0] = node0;
      eltnodes[1] = verinv[Geo.FaceVerNb[face][j]];
      eltnodes[2] = verinv[Geo.FaceVerNb[face][ut_num_rotpos (1, verqty, j, 1)]];

      neut_mesh_addelt (pMesh, eltnodes);
      neut_mesh_elset_addelt (pMesh, 1, (*pMesh).EltQty);

      /* Probably this kind of test would be useful here
      // if the 3 points are aligned (on an edge of the domain), skipping.
      // use an angle-criterion instead.
      // 01/10: sounds that this never happens...
      if (ut_space_pointaligned (p1, p2, p0, 1e-6))
	continue;
      */
    }
  }

  neut_mesh_init_eltelset (pMesh, NULL);

  ut_free_1d_int (ver);
  ut_free_1d_int (verinv);
  ut_free_1d_int (eltnodes);

  return 0;
}

int
neut_geo_poly_mesh (struct GEO Geo, int nb,
		    struct NODES* pNodes, struct MESH* pMesh)
{
  int i;
  struct MESH Mesh2D;
  int* eltnodes = ut_alloc_1d_int (4);

  neut_geo_poly_skinmesh (Geo, nb, pNodes, &Mesh2D);
  neut_nodes_addnode (pNodes, Geo.CenterCoo[nb], 1);

  neut_mesh_set_zero (pMesh);
  neut_mesh_addelset (pMesh, NULL, 0);
  (*pMesh).EltOrder = 1;
  (*pMesh).Dimension = 3;
  (*pMesh).EltType = ut_alloc_1d_char (4);

  sprintf ((*pMesh).EltType, "tri");
  for (i = 1; i <= Mesh2D.EltQty; i++)
  {
    ut_array_1d_int_memcpy (eltnodes, 3, Mesh2D.EltNodes[i]);
    eltnodes[3] = (*pNodes).NodeQty;

    neut_mesh_addelt (pMesh, eltnodes);
    neut_mesh_elset_addelt (pMesh, 1, (*pMesh).EltQty);
  }

  neut_mesh_init_eltelset (pMesh, NULL);

  ut_free_1d_int (eltnodes);
  neut_mesh_free (&Mesh2D);

  return 0;
}

int
neut_geo_poly_reg (struct GEO Geo, int nb)
{
  int i;

  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
    if (Geo.FaceState[Geo.PolyFaceNb[nb][i]] != 0)
      return 1;

  return 0;
}

int
neut_geo_poly_std (struct GEO Geo, int nb)
{
  return (neut_geo_poly_reg (Geo, nb) == 0) ? 1 : 0;
}

int
neut_geo_point_inpoly_std (struct GEO Geo, double* coo, int nb)
{
  int i, j, res;
  double* faceeq = ut_alloc_1d (4);

  res = 1;
  for (i = 1; i <= Geo.PolyFaceQty[nb]; i++)
  {
    for (j = 0; j < 4; j++)
      faceeq[j] =
	(double) Geo.PolyFaceOri[nb][i] *
	Geo.FaceEq[Geo.PolyFaceNb[nb][i]][j];

    if (ut_space_planeside_tol (faceeq, coo - 1, 1e-12) > 0)
    {
      res = 0;
      break;
    }
  }

  ut_free_1d (faceeq);

  return res;
}

int
neut_geo_point_inpoly_reg (struct GEO Geo, double* coo, int nb)
{
  int elt, status;
  struct MESH Mesh;
  struct NODES Nodes;

  neut_mesh_set_zero  (&Mesh);
  neut_nodes_set_zero (&Nodes);

  neut_geo_poly_mesh (Geo, nb, &Nodes, &Mesh);

  status = neut_mesh_elset_point_elt (Mesh, Nodes, 1, coo, &elt);

  neut_mesh_free  (&Mesh);
  neut_nodes_free (&Nodes);

  if (status == 0)
    return 1;
  else
    return 0;
}

int
neut_geo_point_inpoly (struct GEO Geo, double* coo, int nb)
{
  if (neut_geo_poly_std (Geo, nb) == 1)
    return neut_geo_point_inpoly_std (Geo, coo, nb);
  else
    return neut_geo_point_inpoly_reg (Geo, coo, nb);
}


// Returns the interpolation (triangles) of a (regularized or not) face
// under the form of a mesh.
int
neut_geo_face_interpolmesh (struct GEO Geo, int face, struct NODES* pN,
    struct MESH* pM)
{
  int i, tqty, ver, ver0, verqty;
  double *p0 = NULL, *p1 = NULL, *p2 = NULL;
  int* nodes = ut_alloc_1d_int (3);
  int* elts = NULL;

  neut_nodes_set_zero (pN);
  neut_mesh_set_zero (pM);
  (*pM).Dimension = 2;
  (*pM).EltOrder = 1;

  verqty = Geo.FaceVerQty[face];
  for (i = 1; i <= verqty; i++)
  {
    ver = Geo.FaceVerNb[face][i];
    neut_nodes_addnode (pN, Geo.VerCoo[ver], 1);
  }
  if (Geo.FacePt[face] == 0)
    neut_nodes_addnode (pN, Geo.FacePtCoo[face], 1);

  ver0 = (Geo.FacePt[face] != 0) ? Geo.FacePt[face] : verqty + 1;
  p0 = Geo.VerCoo[ver0];

  tqty = 0;
  for (i = 1; i <= verqty; i++)
  {
    // if p0 is a face vertex: 
    // if the ver is p0 or the previous vertex, it is skipped
    if (i == Geo.FacePt[face]
	|| ut_num_rotpos (1, verqty, i, 1) == Geo.FacePt[face])
      continue;

    // recording coo of the 1st and 2nd pts
    nodes[0] = ver0;
    nodes[1] = i;
    nodes[2] = ut_num_rotpos (1, verqty, i, 1);
    p1 = Geo.VerCoo[nodes[1]];
    p2 = Geo.VerCoo[nodes[2]];

    neut_mesh_addelt (pM, nodes);

    // if the 3 points are aligned (on an edge of the domain), skipping.
    // use an angle-criterion instead.
    // 01/10: sounds that this never happens...
    if (ut_space_pointaligned (p1, p2, p0, 1e-6))
      continue;

    tqty++;
  }
  elts = ut_alloc_1d_int ((*pM).EltQty);
  for (i = 0; i < (*pM).EltQty; i++)
    elts[i] = i + 1;
  neut_mesh_addelset (pM, elts, (*pM).EltQty);
  
  ut_free_1d_int (elts);
  ut_free_1d_int (nodes);

  return tqty;
}
