/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_geo_fprintf.h"

/* Geometry exportation: head */
void
GEHead (struct GEO Geo, FILE * file)
{
  fprintf (file, "***tess1.9.2\n");
  Geo.PolyQty = Geo.PolyQty; 
  fprintf (file, " **general\n");
  fprintf (file, " %s %d %d\n", Geo.morpho, Geo.Type, Geo.Id);

  return;
}

/* Geometry exportation: foot */
void
GEFoot (FILE * file)
{
  fprintf (file, "***end\n");

  return;
}

/* Geometry exportation: vertex */
void
GEVer (struct GEO Geo, FILE * file)
{
  int i, j;

  fprintf (file, " **vertex\n");
  fprintf (file, " %d\n", Geo.VerQty);

  for (i = 1; i <= Geo.VerQty; i++)
  {
    fprintf (file, " %3d", i);
    for (j = 0; j < 4; j++)
      fprintf (file, " %d", Geo.VerBound[i][j]);
    fprintf (file, "\n    ");
    fprintf (file, " %d", Geo.VerEdgeQty[i]);
    for (j = 0; j < Geo.VerEdgeQty[i]; j++)
      fprintf (file, " %d", Geo.VerEdgeNb[i][j]);
    fprintf (file, "\n    ");
    for (j = 0; j < 3; j++)
      fprintf (file, " %.12f", Geo.VerCoo[i][j]);
    fprintf (file, " %d\n", Geo.VerState[i]);
  }

  return;
}

/* Geometry exportation: edge */
void
GEEdge (struct GEO Geo, FILE * file)
{
  int i, j;

  fprintf (file, " **edge\n");
  fprintf (file, " %d\n", Geo.EdgeQty);

  for (i = 1; i <= Geo.EdgeQty; i++)
  {
    fprintf (file, " %3d", i);
    for (j = 0; j < 2; j++)
      fprintf (file, " %d", Geo.EdgeVerNb[i][j]);
    fprintf (file, "\n");
    fprintf (file, "     %d ", Geo.EdgeFaceQty[i]);
    ut_array_1d_int_fprintf (file, Geo.EdgeFaceNb[i], Geo.EdgeFaceQty[i], "%d");
    fprintf (file, " %d\n", Geo.EdgeState[i]);
  }

  return;
}

/* Geometry exportation: face */
void
GEFace (struct GEO Geo, FILE * file)
{
  int i, j;

  fprintf (file, " **face\n");
  fprintf (file, " %d\n", Geo.FaceQty);

  for (i = 1; i <= Geo.FaceQty; i++)
  {
    fprintf (file, " %3d %d", i, Geo.FaceBound[i]);
    for (j = 0; j < 2; j++)
      fprintf (file, " %d", Geo.FacePoly[i][j]);
    for (j = 0; j < 4; j++)
      fprintf (file, " %.12f", Geo.FaceEq[i][j]);
    fprintf (file, "\n");

    fprintf (file, "    ");
    fprintf (file, " %d", Geo.FaceVerQty[i]);
    for (j = 1; j <= Geo.FaceVerQty[i]; j++)
      fprintf (file, " %d", Geo.FaceVerNb[i][j]);
    fprintf (file, "\n");

    fprintf (file, "      ");
    for (j = 1; j <= Geo.FaceVerQty[i]; j++)
      fprintf (file, " %d", Geo.FaceEdgeOri[i][j] * Geo.FaceEdgeNb[i][j]);
    fprintf (file, "\n");

    fprintf (file, "    ");
    fprintf (file, " %d", Geo.FaceState[i]);
    fprintf (file, " %d ", Geo.FacePt[i]);
    ut_array_1d_fprintf (file, Geo.FacePtCoo[i], 3, "%.12f");
    fprintf (file, "     %.12f", Geo.FaceFF[i]);
    fprintf (file, "\n");
  }

  return;
}

/* Geometry exportation: poly */
void
GEPoly (struct GEO Geo, FILE * file)
{
  int i, j;

  fprintf (file, " **polyhedron\n");
  fprintf (file, " %d\n", Geo.PolyQty);

  for (i = 1; i <= Geo.PolyQty; i++)
  {
    /* polyhedron id */
    fprintf (file, " %3d", i);

    /* polyhedron center */
    for (j = 0; j < 3; j++)
      fprintf (file, " %.12f", Geo.CenterCoo[i][j]);
    fprintf (file, " %d %d", Geo.PolyTrue[i], Geo.PolyBody[i]);
    fprintf (file, "\n    ");

    /* polyhedron faces */
    fprintf (file, " %d", Geo.PolyFaceQty[i]);
    for (j = 1; j <= Geo.PolyFaceQty[i]; j++)
      fprintf (file, " %d", Geo.PolyFaceOri[i][j] * Geo.PolyFaceNb[i][j]);
    fprintf (file, "\n");
  }

  return;
}
