/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neut_geo_scale.h"

/* Deformation of the geoellation.
 */
void
neut_geo_scale (struct GEO *pGeo, double vx, double vy, double vz)
{
  int i, j = 0, numa, numb;
  double norm;
  double* gsize = ut_alloc_1d (3);
  gsize[0] = vx;
  gsize[1] = vy;
  gsize[2] = vz;

  /* TODO alloc should probably not be done */
  (*pGeo).EdgeLength = ut_alloc_1d ((*pGeo).EdgeQty + 1);

  /* changing vertex coordinates */
  for (i = 1; i <= (*pGeo).VerQty; i++)
    for (j = 0; j < 3; j++)
      (*pGeo).VerCoo[i][j] *= gsize[j];

  /* updating edge lengths from the ver positions */
  for (i = 1; i <= (*pGeo).EdgeQty; i++)
  {
    numa = (*pGeo).EdgeVerNb[i][0];
    numb = (*pGeo).EdgeVerNb[i][1];

    (*pGeo).EdgeLength[i] = ut_space_dist ((*pGeo).VerCoo[numa], (*pGeo).VerCoo[numb]);
  }

  /* updating face equations */
  for (i = 1; i <= (*pGeo).FaceQty; i++)
  {
    (*pGeo).FaceEq[i][1] /= gsize[0];
    (*pGeo).FaceEq[i][2] /= gsize[1];
    (*pGeo).FaceEq[i][3] /= gsize[2];
    norm = ut_vector_norm ((*pGeo).FaceEq[i] + 1);
    for (j = 0; j < 4; j++)
      (*pGeo).FaceEq[i][j] /= norm;
  }

  /* changing centre coordinates */
  for (i = 1; i <= (*pGeo).PolyQty; i++)
    for (j = 0; j < 3; j++)
      (*pGeo).CenterCoo[i][j] *= gsize[j];

  /* changing centre coordinates */
  for (i = 1; i <= (*pGeo).FaceQty; i++)
    if ((*pGeo).FaceState[i] > 0 && (*pGeo).FacePt[i] > 0)
      for (j = 0; j < 3; j++)
	(*pGeo).FacePtCoo[i][j] *= gsize[j];

  ut_free_1d (gsize);

  return;
}
