/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"TestVer.h"

/* we check is the number of edge is >= 3
 * if it does not have two identical edges
 * and if the edges are useful.
 */
int
TestVerEdgeQtyNNb (struct GEO Geo, int i, int verbosity)
{
  int j;
  int edge;
  int res;

  if (Geo.VerEdgeQty[i] < 3)
  {
    if (verbosity >= 2)
    {
      ut_print_lineheader (1);
      printf ("ver %d has less than 3 = %d edges\n", i, Geo.VerEdgeQty[i]);
    }
    return 1;
  }

  res = Geo.VerEdgeQty[i] - 1;
  ut_array_1d_int_compress (Geo.VerEdgeNb[i], &res);
  if (res != Geo.VerEdgeQty[i] - 1)
  {
    /*
       ut_print_lineheader(1);
       printf("ver %d has two identical edges\n",i);
     */
    return 1;
  }

  for (j = 0; j <= Geo.VerEdgeQty[i] - 1; j++)
  {
    edge = Geo.VerEdgeNb[i][j];
    if (Geo.EdgeState[edge] == -1)
    {
      /*
         ut_print_lineheader(1);
         printf("ver %d has a useless edge: nb %d\n",i,edge);
       */
      return 1;
    }
  }

  return 0;
}

/* we check the reciprocity of each ver edge */
int
TestVerEdgeReciprocity (struct GEO Geo, int i)
{
  int j;
  int edge;

  for (j = 0; j <= Geo.VerEdgeQty[i] - 1; j++)
  {
    edge = Geo.VerEdgeNb[i][j];
    if (ut_array_1d_int_eltpos (Geo.EdgeVerNb[edge], 2, i) == -1)
    {
      /*
         ut_print_lineheader(2);
         printf("edge %d of ver %d = edge nb %d has not ver %d as ver.\n",j,i,edge,i);
       */
      return 1;
    }
  }


  return 0;
}

int
TestVerBoundNCoo (struct GEO Geo, int i)
{
  int j;

  if (Geo.VerBound[i][0] > 3)
  {
    /*
       ut_print_lineheader(2);
       printf("ver %d has more than 3 = %d bounds!\n",i,Geo.VerBound[i][0]);
     */
    return 1;
  }

  for (j = 1; j <= Geo.VerBound[i][0]; j++)
  {
    if (Geo.VerBound[i][j] != -1 && Geo.VerBound[i][j] != -2
	&& Geo.VerBound[i][j] != -3 && Geo.VerBound[i][j] != -4
	&& Geo.VerBound[i][j] != -5 && Geo.VerBound[i][j] != -6)
    {
      /*
         ut_print_lineheader(2);
         printf("ver %d has a bad bound:\n",i);
         printf("bound[%d]=%d\n",j,Geo.VerBound[i][j]);
       */
      return 1;
    }

    /* The following checking was commented to enable non-unity cubes to be meshed.
     * if there is a problem with that, change the given bounds (eps & 1-eps)

     if((Geo.VerBound[i][j]==-1 && Geo.VerCoo[i][0]>  eps)
     ||(Geo.VerBound[i][j]==-2 && Geo.VerCoo[i][0]<1-eps)
     ||(Geo.VerBound[i][j]==-3 && Geo.VerCoo[i][1]>  eps)
     ||(Geo.VerBound[i][j]==-4 && Geo.VerCoo[i][1]<1-eps)
     ||(Geo.VerBound[i][j]==-5 && Geo.VerCoo[i][2]>  eps)
     ||(Geo.VerBound[i][j]==-6 && Geo.VerCoo[i][2]<1-eps) )
     {
     ut_print_lineheader(2);
     printf("ver %d has incompatible bounds and coos:\n",i);
     printf("bound[%d]=%d and coo %lf %lf %lf\n",j,Geo.VerBound[i][j],\
     Geo.VerCoo[i][0],Geo.VerCoo[i][1],Geo.VerCoo[i][2]);
     return 1;
     }
     */

  }

  return 0;
}

int
TestVerFaceCoplaneity (struct GEO Geo, int ver)
{
  int i, j, k;
  int edge1, edge2;
  int face;
  int common;

  for (i = 0; i <= Geo.VerEdgeQty[ver] - 2; i++)
  {
    edge1 = Geo.VerEdgeNb[ver][i];

    for (j = i + 1; j <= Geo.VerEdgeQty[ver] - 1; j++)
    {
      edge2 = Geo.VerEdgeNb[ver][j];

      /* printf ("i = %d j = %d; edge1 = %d  edge2 = %d\n", i, j, edge1, edge2); */

      /* searching the number of common faces of edge1 and edge2 */
      common = 0;
      for (k = 0; k <= Geo.EdgeFaceQty[edge1] - 1; k++)
      {
	face = Geo.EdgeFaceNb[edge1][k];
	if (ut_array_1d_int_eltpos
	    (Geo.EdgeFaceNb[edge2], Geo.EdgeFaceQty[edge2], face) != -1)
	  common++;
      }

      if (common > 1)
	return 2;
    }
  }

  return 0;
}
