/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_geo_test.h"

int
neut_geo_test (struct GEO Geo)
{
  int i;
  int res = 0;
  int verbosity = 0;

  /* Testing of the vertices */
  if (verbosity >= 4)
    printf ("Testing vertices: ");
  for (i = 1; i <= Geo.VerQty; i++)
    if (Geo.VerState[i] != -1)
    {
      if (verbosity >= 4)
	printf ("%d:", i);
      res = TestVer (Geo, i, verbosity);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("ver %d: nok\n", i);
	return res;
      }
    }
  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the edges */
  if (verbosity >= 4)
    printf ("\nTesting edges: ");
  for (i = 1; i <= Geo.EdgeQty; i++)
    if (Geo.EdgeState[i] == 0)
    {
      if (verbosity >= 4)
	printf ("%d:", i);
      res = TestEdge (Geo, i);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("edge %d: nok\n", i);
	return res;
      }
    }
  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the faces */
  if (verbosity >= 4)
    printf ("\nTesting faces: ");
  for (i = 1; i <= Geo.FaceQty; i++)
    if (Geo.FaceState[i] >= 0)
    {
      if (verbosity >= 4)
	printf ("%d:", i);
      res = TestFace (Geo, i);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("face %d: nok (TestFace returned %d)\n", i, res);
	return res;
      }
    }
  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the polys */
  if (verbosity >= 4)
    printf ("\nTesting polys: ");
  for (i = 1; i <= Geo.PolyQty; i++)
  {
    if (verbosity >= 4)
      printf ("%d:", i);
    res = TestPoly (Geo, i);
    if (res != 0)
    {
      if (verbosity >= 4)
	printf ("poly %d: nok\n", i);
      return res;
    }
  }
  if (verbosity >= 4)
    printf ("\n");

  return res;
}

int
neut_geo_testAroundVer (struct GEO Geo, int ver, int verbosity)
{
  int i;
  int edge, face, poly;
  int res = 0;
  int *faces = ut_alloc_1d_int (1000);
  int *polys = ut_alloc_1d_int (1000);

  if (verbosity >= 4)
    printf ("testing GEO AROUND VER %d\n", ver);

  /* Testing of the vertex */
  if (verbosity >= 4)
    printf ("Testing vertex: ");

  if (Geo.VerState[ver] != -1)
  {
    if (verbosity >= 4)
      printf ("%d:", ver);
    res = TestVer (Geo, ver, verbosity);
    if (res != 0)
    {
      if (verbosity >= 4)
	printf ("ver %d: nok\n", ver);
      return res;
    }
  }

  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the edges */
  if (verbosity >= 4)
    printf ("\nTesting edges: ");

  for (i = 0; i <= Geo.VerEdgeQty[ver] - 1; i++)
  {
    edge = Geo.VerEdgeNb[ver][i];

    if (Geo.EdgeState[edge] == 0)
    {
      if (verbosity >= 4)
	printf ("%d:", edge);
      res = TestEdge (Geo, edge);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("edge %d: nok\n", edge);
	return res;
      }
    }
    else
    {
      res = 1;
      abort ();
      return res;
    }
  }
  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the faces */
  neut_geo_ver_face (Geo, ver, faces);

  if (verbosity >= 4)
    printf ("\nTesting faces: ");

  for (i = 1; i <= faces[0]; i++)
  {
    face = faces[i];

    if (Geo.FaceState[face] >= 0)
    {
      if (verbosity >= 4)
	printf ("%d:", face);
      res = TestFace (Geo, face);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("face %d: nok\n", face);
	return res;
      }
    }
    else
    {
      abort ();
      return 1;
    }
  }

  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the polys */
  neut_geo_ver_poly (Geo, ver, polys);

  if (verbosity >= 4)
    printf ("\nTesting polys: ");
  for (i = 1; i <= polys[0]; i++)
  {
    poly = polys[i];
    if (poly < 0)
      continue;

    if (verbosity >= 4)
      printf ("%d:", i);
    res = TestPoly (Geo, poly);
    if (res != 0)
    {
      if (verbosity >= 4)
	printf ("poly %d: nok\n", poly);
      return res;
    }
  }
  if (verbosity >= 4)
    printf ("\n");

  ut_free_1d_int (faces);
  ut_free_1d_int (polys);

  return res;
}

/* Testing the geometry around a vertex: all the polys around the ver
 * and all entities of the polys: faces, edges & vertices.
 */
int
neut_geo_testAroundVer2 (struct GEO Geo, int ver, int verbosity)
{
  int i, j, k;
  int edge, face, poly;
  int res = 0;
  int *vers = ut_alloc_1d_int (10000);
  int *edges = ut_alloc_1d_int (10000);
  int *faces = ut_alloc_1d_int (10000);
  int *polys = ut_alloc_1d_int (10000);


  if (verbosity >= 4)
    printf ("testing GEO (2) AROUND VER %d (/%d)\n", ver, Geo.VerQty);

  /* Searching polyhedra */
  neut_geo_ver_poly (Geo, ver, polys);

  /* searching faces and edges of all the polyhedra */
  faces[0] = 0;
  edges[0] = 0;
  vers[0] = 0;
  for (i = 1; i <= polys[0]; i++)
  {
    poly = polys[i];
    if (poly < 0)
      continue;

    for (j = 1; j <= Geo.PolyFaceQty[poly]; j++)
    {
      face = Geo.PolyFaceNb[poly][j];

      faces[0]++;
      faces[faces[0]] = face;

      for (k = 1; k <= Geo.FaceVerQty[face]; k++)
      {
	edges[0]++;
	edges[edges[0]] = Geo.FaceEdgeNb[face][k];

	vers[0]++;
	vers[vers[0]] = Geo.FaceVerNb[face][k];
      }
    }
  }

  gsl_sort_int (faces + 1, 1, faces[0]);
  ut_array_1d_int_compress (faces + 1, &(faces[0]));

  gsl_sort_int (edges + 1, 1, edges[0]);
  ut_array_1d_int_compress (edges + 1, &(edges[0]));

  gsl_sort_int (vers + 1, 1, vers[0]);
  ut_array_1d_int_compress (vers + 1, &(vers[0]));

  /* Testing of the vertex */
  if (verbosity >= 4)
    printf ("Testing vertex: ");

  for (i = 1; i <= vers[0]; i++)
  {
    ver = vers[i];

    if (Geo.VerState[ver] != -1)
    {
      if (verbosity >= 4)
	printf ("%d:", ver);
      res = TestVer (Geo, ver, 0);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("ver %d: nok\n", ver);
	ut_free_1d_int (vers);
	ut_free_1d_int (edges);
	ut_free_1d_int (faces);
	ut_free_1d_int (polys);
	return res;
      }
    }
  }

  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the edges */
  if (verbosity >= 4)
    printf ("\nTesting edges: ");

  for (i = 0; i <= edges[0]; i++)
  {
    edge = edges[i];

    if (Geo.EdgeState[edge] == 0)
    {
      if (verbosity >= 4)
	printf ("%d:", edge);
      res = TestEdge (Geo, edge);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("edge %d: nok\n", edge);
	ut_free_1d_int (vers);
	ut_free_1d_int (edges);
	ut_free_1d_int (faces);
	ut_free_1d_int (polys);
	return res;
      }
    }
  }
  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the faces */

  if (verbosity >= 4)
    printf ("\nTesting faces: ");

  for (i = 1; i <= faces[0]; i++)
  {
    face = faces[i];

    if (Geo.FaceState[face] >= 0)
    {
      if (verbosity >= 4)
	printf ("%d:", face);
      res = TestFace (Geo, face);
      if (res != 0)
      {
	if (verbosity >= 4)
	  printf ("face %d: nok\n", face);
	ut_free_1d_int (vers);
	ut_free_1d_int (edges);
	ut_free_1d_int (faces);
	ut_free_1d_int (polys);
	return res;
      }
    }
  }

  if (verbosity >= 4)
    printf ("\n");

  /* Testing of the polys */

  if (verbosity >= 4)
    printf ("\nTesting polys: ");

  for (i = 1; i <= polys[0]; i++)
  {
    poly = polys[i];

    if (poly < 0)
      continue;

    if (verbosity >= 4)
      printf ("%d:", i);
    res = TestPoly (Geo, poly);
    if (res != 0)
    {
      if (verbosity >= 4)
	printf ("poly %d: nok\n", poly);
	ut_free_1d_int (vers);
	ut_free_1d_int (edges);
	ut_free_1d_int (faces);
	ut_free_1d_int (polys);
      return res;
    }
  }
  if (verbosity >= 4)
    printf ("\n");

  ut_free_1d_int (vers);
  ut_free_1d_int (edges);
  ut_free_1d_int (faces);
  ut_free_1d_int (polys);

  return res;
}
