/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#ifndef STRUCTGERMSET_H
#define STRUCTGERMSET_H

struct GERMSET
{
  char* morpho;
  
  /* N: number of germs */
  int N;
  int N1d;
  /* Id: for a given N, identifier of the germ random distribution */
  int Id;
  /* Random: long integer number calculated from (N,Id):
   * Random is 8 numbers long ; N is put at the left side, Id at the
   * right side. If N and Id numbers cross, they are added.
   *                          Random=XXXXXXXX
   * N=123  and Id=12     --> Random=12300012
   * N=3000 and Id=65433  --> Random=30065433
   * N=3333 and Id=88888  --> Random=33418888
   */
  long Random;
  /* GermsCoo contains the coo of all the germs.
   * GermsCoo[i][j] contains the jth coordinate (j=1,2,3) of germ i
   * (i=1,...,N).
   */
  double **GermsCoo;
  /* ttype is the type of tessellation
   * 0: standard tessellation
   * 1: periodic tessellation (useful for -MM only)
   * 2: sd type tessellation (useful for -MM only)
   */
  int ttype;

  /* NDensity is useful for module -MM with -ttype 2;
   * it is the mean germ quantity per unit volume.
   * (N can differ from NDensity in sd type tessellations)
   */
  double NDensity;

  double randomize;
  int randomize2;
};
typedef struct GERMSET GERMSET;

#endif /* STRUCTGERMSET_H */
