/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_tess_fprintf.h"

/* Tessellation exportation: head
 */
void
TEHead (struct TESS Tess, FILE * file)
{
  fprintf (file, "***tess1.9\n");
  fprintf (file, " **general\n");
  fprintf (file, " %s %d %d %d\n", Tess.morpho, Tess.Type, Tess.PolyQty, Tess.Id);

  return;
}

/* Tessellation exportation: foot
 */
void
TEFoot (FILE * file)
{
  fprintf (file, "***end\n");

  return;
}

/* Tessellation exportation: vertex
 */
void
TEVer (struct TESS Tess, FILE * file)
{
  int i, j;

  fprintf (file, " **vertex\n");
  fprintf (file, " %d\n", Tess.VerQty);

  for (i = 1; i <= Tess.VerQty; i++)
  {
    fprintf (file, " %3d", i);
    for (j = 0; j <= 3; j++)
      fprintf (file, " %d", Tess.VerGerm[i][j]);
    fprintf (file, "\n    ");
    for (j = 0; j <= 3; j++)
      fprintf (file, " %d", Tess.VerEdge[i][j]);
    fprintf (file, "\n    ");
    for (j = 0; j <= 2; j++)
      fprintf (file, " %.12f", Tess.VerCoo[i][j]);
    fprintf (file, "\n");
  }

  return;
}

/* Tessellation exportation: edge
 */
void
TEEdge (struct TESS Tess, FILE * file)
{
  int i, j;

  fprintf (file, " **edge\n");
  fprintf (file, " %d\n", Tess.EdgeQty);

  for (i = 1; i <= Tess.EdgeQty; i++)
  {
    fprintf (file, " %3d", i);
    for (j = 0; j <= 1; j++)
      fprintf (file, " %d", Tess.EdgeVerNb[i][j]);
    fprintf (file, "\n");
  }

  return;
}

/* Tessellation exportation: face
 */
void
TEFace (struct TESS Tess, FILE * file)
{
  int i, j;

  fprintf (file, " **face\n");
  fprintf (file, " %d\n", Tess.FaceQty);

  for (i = 1; i <= Tess.FaceQty; i++)
  {
    fprintf (file, " %3d", i);
    for (j = 0; j <= 1; j++)
      fprintf (file, " %d", Tess.FacePoly[i][j]);
    for (j = 0; j <= 3; j++)
      fprintf (file, " %.12f", Tess.FaceEq[i][j]);
    fprintf (file, "\n    ");
    fprintf (file, " %d", Tess.FaceVerQty[i]);
    for (j = 1; j <= Tess.FaceVerQty[i]; j++)
      fprintf (file, " %d", Tess.FaceVerNb[i][j]);
    fprintf (file, "\n      ");
    for (j = 1; j <= Tess.FaceVerQty[i]; j++)
      fprintf (file, " %d", Tess.FaceEdgeOri[i][j] * Tess.FaceEdgeNb[i][j]);
    fprintf (file, "\n");
  }

  return;
}

/* Tessellation exportation: poly
 */
void
TEPoly (struct TESS Tess, FILE * file)
{
  int i, j;

  fprintf (file, " **polyhedron\n");
  fprintf (file, " %d\n", Tess.PolyQty);

  for (i = 1; i <= Tess.PolyQty; i++)
  {
    /* polyhedron id */
    fprintf (file, " %3d", i);

    /* polyhedron center */
    for (j = 0; j < 3; j++)
      fprintf (file, " %.12f", Tess.CenterCoo[i][j]);
    fprintf (file, " %d %d", Tess.PolyTrue[i], Tess.PolyBody[i]);
    fprintf (file, "\n    ");

    /* polyhedron vertices */
    fprintf (file, " %d", Tess.PolyVerQty[i]);
    for (j = 1; j <= Tess.PolyVerQty[i]; j++)
      fprintf (file, " %d", Tess.PolyVerNb[i][j]);
    fprintf (file, "\n    ");

    /* polyhedron edges */
    fprintf (file, " %d", Tess.PolyEdgeQty[i]);
    for (j = 1; j <= Tess.PolyEdgeQty[i]; j++)
      fprintf (file, " %d", Tess.PolyEdgeNb[i][j]);
    fprintf (file, "\n    ");

    /* polyhedron faces */
    fprintf (file, " %d", Tess.PolyFaceQty[i]);
    for (j = 1; j <= Tess.PolyFaceQty[i]; j++)
      fprintf (file, " %d", Tess.PolyFaceOri[i][j] * Tess.PolyFaceNb[i][j]);
    fprintf (file, "\n");
  }

  return;
}

/* Tessellation exportation: domain
 */
void
TEDomain (struct TESS Tess, FILE * file)
{
  int i;

  fprintf (file, " **domain\n");

  for (i = 0; i <= 7; i++)
    fprintf (file, " %3d", Tess.VerOfDomain[i]);
  fprintf (file, "\n");

  return;
}
