/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neut_tess_fscanf.h"

/* Tessellation importation: head  */
void
TIHead (struct TESS *pTess, FILE * file)
{
  char* version = ut_alloc_1d_char (100);

  fscanf (file, "***tess%s", version);

  if (strcmp (version, "1.4") == 0)
  {
    (*pTess).Type = 0;
    (*pTess).Id = -2;
  }
  else if (strcmp (version, "1.8") == 0 || strcmp (version, "1.9") == 0)
  {
    ut_file_skip (file, 1);
    (*pTess).morpho = ut_alloc_1d_char (10);
    fscanf (file, "%s", ((*pTess).morpho));
    fscanf (file, "%d", &((*pTess).Type));
    fscanf (file, "%d", &((*pTess).PolyQty));
    fscanf (file, "%d", &((*pTess).Id));
  }
  else
  {
    ut_print_message (2, 0, "Input file is not a valid tessellation file.\n");
    printf ("Unknown / unsupported version %s\n", version);
    abort ();
  }

  ut_free_1d_char (version);

  return;
}

/* Tessellation importation: vertices
 */
void
TIVer (struct TESS *pTess, FILE * file)
{
  TIVerHead (file);
  TIVerQty (pTess, file);
  TIVerProp (pTess, file);

  return;
}

/* Tessellation importation: edges
 */
void
TIEdge (struct TESS *pTess, FILE * file)
{
  TIEdgeHead (file);
  TIEdgeQty (pTess, file);
  TIEdgeProp (pTess, file);

  return;
}

/* Tessellation importation: faces
 */
void
TIFace (struct TESS *pTess, FILE * file)
{
  TIFaceHead (file);
  TIFaceQty (pTess, file);
  TIFaceProp (pTess, file);

  return;
}

/* Tessellation importation: poly
 */
void
TIPoly (struct TESS *pTess, FILE * file)
{
  TIPolyHead (file);
  TIPolyQty (pTess, file);
  TIPolyProp (pTess, file);

  return;
}

/* Tessellation importation: domain
 */
void
TIDomain (struct TESS *pTess, FILE * file)
{
  int i;

  TIDomainHead (file);

  (*pTess).VerOfDomain = ut_alloc_1d_int (8);

  for (i = 0; i <= 7; i++)
    fscanf (file, "%d", &(*pTess).VerOfDomain[i]);

  return;
}

/* Tessellation importation: foot
 */
void
TIFoot (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**end") != 0)
    abort ();

  return;
}
