/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include "neut_tess_fscanf.h"

void
TIVerHead (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**vertex") != 0)
    abort ();

  return;
}

void
TIVerQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).VerQty);

  return;
}

void
TIVerProp (struct TESS *pTess, FILE * file)
{
  int i;
  int ver;

  (*pTess).VerGerm = ut_alloc_2d_int ((*pTess).VerQty + 1, 4);
  (*pTess).VerEdge = ut_alloc_2d_int ((*pTess).VerQty + 1, 4);
  (*pTess).VerCoo = ut_alloc_2d ((*pTess).VerQty + 1, 3);

  for (i = 1; i <= (*pTess).VerQty; i++)
  {
    fscanf (file, "%d", &ver);
    TIVerParGerm (pTess, ver, file);
    TIVerParEdge (pTess, ver, file);
    TIVerCoo (pTess, ver, file);
  }

  return;
}

void
TIEdgeHead (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**edge") != 0)
    abort ();

  return;
}

void
TIEdgeQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).EdgeQty);

  return;
}

void
TIEdgeProp (struct TESS *pTess, FILE * file)
{
  int i;
  int edge;

  (*pTess).EdgeVerNb = ut_alloc_2d_int ((*pTess).EdgeQty + 1, 2);

  for (i = 1; i <= (*pTess).EdgeQty; i++)
  {
    fscanf (file, "%d", &edge);
    TIEdgeVerNb (pTess, edge, file);
  }

  return;
}


void
TIFaceHead (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**face") != 0)
    abort ();

  return;
}


void
TIFaceQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).FaceQty);

  return;
}


void
TIFaceProp (struct TESS *pTess, FILE * file)
{
  int i;
  int face;

  (*pTess).FacePoly = ut_alloc_2d_int ((*pTess).FaceQty + 1, 2);
  (*pTess).FaceEq = ut_alloc_2d ((*pTess).FaceQty + 1, 4);
  (*pTess).FaceVerQty = ut_alloc_1d_int ((*pTess).FaceQty + 1);
  (*pTess).FaceVerNb = ut_alloc_1d_pint ((*pTess).FaceQty + 1);
  (*pTess).FaceEdgeNb = ut_alloc_1d_pint ((*pTess).FaceQty + 1);
  (*pTess).FaceEdgeOri = ut_alloc_1d_pint ((*pTess).FaceQty + 1);

  for (i = 1; i <= (*pTess).FaceQty; i++)
  {
    fscanf (file, "%d", &face);
    TIFacePoly (pTess, face, file);
    TIFaceEq (pTess, face, file);
    TIFaceVerNEdgeProp (pTess, face, file);
  }

  return;
}

void
TIPolyHead (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**polyhedron") != 0)
    abort ();

  return;
}


void
TIPolyQty (struct TESS *pTess, FILE * file)
{
  fscanf (file, "%d", &(*pTess).PolyQty);

  return;
}

void
TIPolyProp (struct TESS *pTess, FILE * file)
{
  int i;
  int poly;

  (*pTess).CenterCoo = ut_alloc_2d ((*pTess).PolyQty + 1, 3);
  (*pTess).PolyVerQty = ut_alloc_1d_int ((*pTess).PolyQty + 1);
  (*pTess).PolyVerNb = ut_alloc_1d_pint ((*pTess).PolyQty + 1);
  (*pTess).PolyEdgeQty = ut_alloc_1d_int ((*pTess).PolyQty + 1);
  (*pTess).PolyEdgeNb = ut_alloc_1d_pint ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceQty = ut_alloc_1d_int ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceNb = ut_alloc_1d_pint ((*pTess).PolyQty + 1);
  (*pTess).PolyFaceOri = ut_alloc_1d_pint ((*pTess).PolyQty + 1);
  (*pTess).PolyTrue    = ut_alloc_1d_int ((*pTess).PolyQty + 1);
  (*pTess).PolyBody    = ut_alloc_1d_int ((*pTess).PolyQty + 1);

  for (i = 1; i <= (*pTess).PolyQty; i++)
  {
    fscanf (file, "%d", &poly);
    TICenterCoo (pTess, poly, file);
    fscanf (file, "%d", &((*pTess).PolyTrue[i]));
    fscanf (file, "%d", &((*pTess).PolyBody[i]));
    TIPolyVerProp (pTess, poly, file);
    TIPolyEdgeProp (pTess, poly, file);
    TIPolyFaceProp (pTess, poly, file);
  }

  return;
}

void
TIDomainHead (FILE * file)
{
  if (ut_file_string_scanncomp (file, "**domain") != 0)
    abort ();

  return;
}
