/* This file is part of the 'neper' program. */
/* Copyright (C) 2003-2011, Romain Quey. */
/* See the COPYING file in the top-level directory. */

#include"neut_tess_geo.h"

/* neut_tess_geotializing the properties of the vertices */
void
neut_tess_geoVer (struct TESS Tess, struct GEO *pGeo)
{
  neut_tess_geoVerCoo (Tess, pGeo);
  neut_tess_geoVerEdge (Tess, pGeo);
  neut_tess_geoVerBound (Tess, pGeo);
  neut_tess_geoVerState (pGeo);

  return;
}

/* neut_tess_geotializing the properties of the edges */
void
neut_tess_geoEdge (struct TESS Tess, struct GEO *pGeo)
{
  neut_tess_geoEdgeVerNb (Tess, pGeo);
  neut_tess_geoEdgeFace (Tess, pGeo);
  neut_geo_init_edgelength (pGeo);
  neut_tess_geoEdgeState (pGeo);
  neut_tess_geoEdgeDel (pGeo);

  return;
}

/* neut_tess_geotializing the properties of the faces */
void
neut_tess_geoFace (struct TESS Tess, struct GEO *pGeo)
{
  neut_tess_geoFacePoly (Tess, pGeo);
  neut_tess_geoFaceVer (Tess, pGeo);
  neut_tess_geoFaceEdge (Tess, pGeo);
  neut_tess_geoFaceFF  (pGeo); 
  neut_tess_geoFaceState (pGeo); 
  neut_tess_geoFacePt (pGeo); 
  neut_tess_geoFaceEq (Tess, pGeo);
  neut_tess_geoFaceBound (pGeo);

  return;
}

/* neut_tess_geotializing the properties of the polyhedra */
void
neut_tess_geoPoly (struct TESS Tess, struct GEO *pGeo)
{
  (*pGeo).CenterCoo = ut_alloc_2d (Tess.PolyQty + 1, 3);
  (*pGeo).PolyTrue  = ut_alloc_1d_int (Tess.PolyQty + 1);
  (*pGeo).PolyBody  = ut_alloc_1d_int (Tess.PolyQty + 1);
  
  ut_array_2d_memcpy ((*pGeo).CenterCoo, Tess.PolyQty + 1, 3, Tess.CenterCoo);

  ut_array_1d_int_memcpy ((*pGeo).PolyTrue, Tess.PolyQty + 1, Tess.PolyTrue);
  ut_array_1d_int_memcpy ((*pGeo).PolyBody, Tess.PolyQty + 1, Tess.PolyBody);

  neut_tess_geoPolyFace (Tess, pGeo);

  return;
}

/* neut_tess_geoDomainSize ini the domain size */
void
neut_tess_geodomainsize (struct TESS Tess, struct GEO *pGeo)
{
  int i, j;
  double **coo = ut_alloc_2d (3, 8);

  (*pGeo).DomainSize = ut_alloc_2d (3, 2);

  for (i = 0; i < 3; i++)
  {
    for (j = 0; j < 8; j++)
      coo[i][j] = Tess.VerCoo[Tess.VerOfDomain[j]][i];

    /* lower bounds are supposed to be 0, 0, 0. */
    (*pGeo).DomainSize[i][0] = ut_array_1d_min (coo[i], 8);
    (*pGeo).DomainSize[i][1] = ut_array_1d_max (coo[i], 8);
  }

  ut_free_2d (coo, 3);

  return;
}
