/**********************************************************************
 * Orilib - routines for orientation manipulation.                    *
 * Copyright (C) 2007, 2008, 2009 Romain Quey                         *
 *                                                                    *
 * This program is free software: you can redistribute it and/or      *
 * modify it under the terms of the GNU General Public License as     *
 * published by the Free Software Foundation, either version 3 of the *
 * License, or (at your option) any later version.                    *
 *                                                                    *
 * This program is distributed in the hope that it will be useful,    *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
 * General Public License for more details.                           *
 *                                                                    *
 * You should have received a copy of the GNU General Public License  *
 * along with this program.  If not, see                              *
 * <http://www.gnu.org/licenses/>.                                    *
 *                                                                    *
 * project homepage: http://orilib.sourceforge.net                    *
 **********************************************************************/
#include"ol_des.h"

double *
ol_R_alloc ()
{
  double *R = ut_alloc_1d (3);

  return R;
}

void
ol_R_free (double *R)
{
  if (!R)
    return;
  else
    free (R);

  return;
}

void
ol_R_set_zero (double *R)
{
  ut_array_1d_zero (R, 3);

  return;
}

void
ol_R_set_id (double *R)
{
  ut_array_1d_zero (R, 3);

  return;
}

void
ol_R_set_this (double *R,
double R1, 
double R2, 
double R3)
{
  R[0] = R1;
  R[1] = R2;
  R[2] = R3;

  return;
}


void
ol_R_memcpy (double *Rsrc, double *Rdest)
{
  ut_array_1d_memcpy (Rdest, 3, Rsrc);

  return;
}

void
ol_rtheta_R (double *r, double theta, double *R)
{
  ol_rtheta_R_rad (r, theta * OL_PI / 180, R);

  return;
}

void
ol_rtheta_R_rad (double *r, double theta, double *R)
{
  int i;

  for (i = 0; i <= 2; i++)
    R[i] = r[i] * tan (theta / 2);

  return;
}

void
ol_R_rtheta (double *R, double *r, double *ptheta)
{
  ol_R_rtheta_rad (R, r, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_R_rtheta_rad (double *R, double *r, double *ptheta)
{
  int i;
  double norm;

  norm = ut_array_1d_norm (R, 3);

  (*ptheta) = 2 * atan (norm);

  if ((*ptheta) < OL_EPS_DEG)
  {
    r[0] = 1;
    r[1] = 0;
    r[2] = 0;
  }
  else
    for (i = 0; i <= 2; i++)
      r[i] = R[i] / norm;

  return;
}

void
ol_R_r (double *R, double *r)
{
  double theta;

  ol_R_rtheta_rad (R, r, &theta);

  return;
}

void
ol_R_theta (double *R, double *ptheta)
{
  ol_R_theta_rad (R, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_R_theta_rad (double *R, double *ptheta)
{
  (*ptheta) = 2 * atan (ut_array_1d_norm (R, 3));

  return;
}

void
ol_g_R (double **g, double *R)
{
  int i;
  double *r;
  double theta;

  r = ol_r_alloc ();

  ol_g_rtheta_rad (g, r, &theta);

  for (i = 0; i <= 2; i++)
    R[i] = r[i] * tan (theta / 2);

  ol_r_free (r);

  return;
}

void
ol_R_g (double *R, double **g)
{
  int i;
  double norm;
  double *r;
  double theta;

  r = ol_r_alloc ();

  /* Calculation of the angle/axis pair from R */
  norm = ut_num_sqrt (ut_num_p2 (R[0]) + ut_num_p2 (R[1]) + ut_num_p2 (R[2]));
  theta = 2 * atan (norm);

  for (i = 0; i <= 2; i++)
    r[i] = R[i] / norm;

  ol_rtheta_g_rad (r, theta, g);

  ol_r_free (r);

  return;
}

int
ol_R_fscanf (FILE * file, double *R)
{
  if (fscanf (file, "%lf%lf%lf", &R[0], &R[1], &R[2]) == 3)
    return 1;
  else
    return 0;
}

int
ol_R_fprintf (FILE * file, double *R, char *format)
{
  fprintf (file, format, R[0]);
  fprintf (file, " ");
  fprintf (file, format, R[1]);
  fprintf (file, " ");
  fprintf (file, format, R[2]);
  return fprintf (file, "\n");
}

void
ol_R_e (double *R, double *e)
{
  double *q = ol_q_alloc ();

  ol_R_q (R, q);
  ol_q_e (q, e);

  ol_q_free (q);

  return;
}

void
ol_R_e_rad (double *R, double *e)
{
  double *q = ol_q_alloc ();

  ol_R_q (R, q);
  ol_q_e_rad (q, e);

  ol_q_free (q);

  return;
}

double *
ol_e_alloc ()
{
  double *e = ut_alloc_1d (3);

  return e;
}

void
ol_e_free (double *e)
{
  if (!e)
    return;
  else
    free (e);

  return;
}

void
ol_e_set_zero (double *e)
{
  ut_array_1d_zero (e, 3);

  return;
}

void
ol_e_set_id (double *e)
{
  ut_array_1d_zero (e, 3);

  return;
}

void
ol_e_set_this (double *e, double phi1, double phi, double phi2)
{
  e[0] = phi1;
  e[1] = phi;
  e[2] = phi2;

  return;
}

void
ol_e_memcpy (double *esrc, double *edest)
{
  ut_array_1d_memcpy (edest, 3, esrc);

  return;
}

void
ol_g_e (double **g, double *e)
{
  ol_g_e_rad (g, e);

  ol_e_rad2deg (e, e);

  return;
}

void
ol_g_e_rad (double **g, double *e)
{
  e[1] = ut_num_acos (g[2][2]);

  if (fabs (e[1]) > OL_EPS_DEG && fabs (e[1] - OL_PI) > OL_EPS_DEG)
  {
    e[0] = atan2 (g[2][0], -g[2][1]);
    e[2] = atan2 (g[0][2], g[1][2]);
  }
  else
  {
    e[0] = atan2 (g[0][1], g[0][0]);
    e[2] = 0;
  }

  return;
}

void
ol_e_g (double *e, double **g)
{
  double *e2;

  e2 = ol_e_alloc ();
  ol_e_deg2rad (e, e2);

  ol_e_g_rad (e2, g);

  ol_e_free (e2);

  return;
}

void
ol_e_g_rad (double *e, double **g)
{
  g[0][0] = cos (e[0]) * cos (e[2]) - sin (e[0]) * sin (e[2]) * cos (e[1]);
  g[0][1] = sin (e[0]) * cos (e[2]) + cos (e[0]) * sin (e[2]) * cos (e[1]);
  g[0][2] = sin (e[2]) * sin (e[1]);

  g[1][0] = -cos (e[0]) * sin (e[2]) - sin (e[0]) * cos (e[2]) * cos (e[1]);
  g[1][1] = -sin (e[0]) * sin (e[2]) + cos (e[0]) * cos (e[2]) * cos (e[1]);
  g[1][2] = cos (e[2]) * sin (e[1]);

  g[2][0] = sin (e[0]) * sin (e[1]);
  g[2][1] = -cos (e[0]) * sin (e[1]);
  g[2][2] = cos (e[1]);

  return;
}

void
ol_e_e (double *e, double *e2)
{
  ol_e_deg2rad (e, e2);
  ol_e_e_rad (e2, e2);
  ol_e_rad2deg (e2, e2);

  return;
}

void
ol_e_e_rad (double *e, double *e2)
{
  int i;

  for (i = 0; i < 3; i++)
    e2[i] = e[i];

  /* ranging phi within [0,(2*OL_PI)] */
  e2[1] -= ((int) (e2[1] / (2 * OL_PI))) * (2 * OL_PI);
  if (e2[1] < 0)
    e2[1] += (2 * OL_PI);

  /* if phi>OL_PI, applying reflection */
  /* phi becomes within [0,OL_PI] */
  if (e2[1] > OL_PI)
  {
    e2[1] = (2 * OL_PI) - e2[1];
    e2[0] += OL_PI;
    e2[2] += OL_PI;
  }

  /* treating the std case where phi != 0 */
  if (fabs (e2[1]) > OL_EPS_DEG && fabs (e2[1] - OL_PI) > OL_EPS_DEG)
  {
    /* ranging phi2 within [0,(2*OL_PI)] */
    e2[2] -= ((int) (e2[2] / (2 * OL_PI))) * (2 * OL_PI);
    if (e2[2] < 0)
      e2[2] += (2 * OL_PI);
  }
  /* treating degeneracy: phi = 0: phi1 += phi2 and phi2 = 0. */
  else if (fabs (e2[1]) > OL_EPS_DEG)
  {
    e2[0] += e2[2];
    e2[2] = 0;
  }
  else				/* the same at phi = 180 */
  {
    e2[0] -= e2[2];
    e2[2] = 0;
  }

  /* ranging phi1 within [0,(2*OL_PI)] */
  e2[0] -= ((int) (e2[0] / (2 * OL_PI))) * (2 * OL_PI);
  if (e2[0] < 0)
    e2[0] += (2 * OL_PI);

  return;
}

void
ol_e_deg2rad (double *e, double *e2)
{
  int i;

  for (i = 0; i <= 2; i++)
    ol_theta_deg2rad (e[i], &(e2[i]));

  return;
}

void
ol_e_rad2deg (double *e, double *e2)
{
  int i;

  for (i = 0; i < 3; i++)
    ol_theta_rad2deg (e[i], &(e2[i]));

  return;
}

int
ol_e_fprintf (FILE * file, double *e, char *format)
{
  fprintf (file, format, e[0]);
  fprintf (file, " ");
  fprintf (file, format, e[1]);
  fprintf (file, " ");
  fprintf (file, format, e[2]);
  return fprintf (file, "\n");
}

int
ol_e_fscanf (FILE * file, double *e)
{
  if (fscanf (file, "%lf%lf%lf", &e[0], &e[1], &e[2]) == 3)
    return 1;
  else
    return 0;
}

void
ol_e_R (double *e, double *R)
{
  double *q = ol_q_alloc ();

  ol_e_q (e, q);
  ol_q_R (q, R);

  ol_q_free (q);

  return;
}

void
ol_e_R_rad (double *e, double *R)
{
  double *q = ol_q_alloc ();

  ol_e_q_rad (e, q);
  ol_q_R (q, R);

  ol_q_free (q);

  return;
}

void
ol_e_rtheta (double *e, double *r, double *ptheta)
{
  double *q = ol_q_alloc ();

  ol_e_q (e, q);
  ol_q_rtheta (q, r, ptheta);

  ol_q_free (q);

  return;
}

void
ol_e_rtheta_rad (double *e, double *r, double *ptheta)
{
  double *q = ol_q_alloc ();

  ol_e_q_rad (e, q);
  ol_q_rtheta_rad (q, r, ptheta);

  ol_q_free (q);

  return;
}

void
ol_e_r (double *e, double *r)
{
  double *q = ol_q_alloc ();

  ol_e_q (e, q);
  ol_q_r (q, r);

  ol_q_free (q);

  return;
}

void
ol_e_theta_rad (double *e, double *ptheta)
{
  double *q = ol_q_alloc ();

  ol_e_q_rad (e, q);
  ol_q_theta_rad (q, ptheta);

  ol_q_free (q);

  return;
}

void
ol_e_theta (double *e, double *ptheta)
{
  ol_e_theta_rad (e, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_e_er_rad (double* e, double* er)
{
  er[0] = e[0] - OL_PI / 2.;
  er[1] = e[1];
  er[2] = e[2] + OL_PI / 2.;
  ol_e_e_rad (er, er);

  return;
}

void
ol_e_ek_rad (double* e, double* ek)
{
  ek[0] = e[0] - OL_PI / 2.;
  ek[1] = e[1];
  ek[2] = OL_PI / 2. - e[2];
  ol_e_e_rad (ek, ek);

  return;
}

void
ol_er_e_rad (double* er, double* e)
{
  e[0] = er[0] + OL_PI / 2.;
  e[1] = er[1];
  e[2] = er[2] - OL_PI / 2.;
  ol_e_e_rad (e, e);

  return;
}

void
ol_ek_e_rad (double* ek, double* e)
{
  e[0] = ek[0] + OL_PI / 2.;
  e[1] = ek[1];
  e[2] = OL_PI / 2. - ek[2];
  ol_e_e_rad (e, e);

  return;
}

void
ol_e_er (double* e, double* er)
{
  ol_e_deg2rad (e , er);
  ol_e_er_rad  (er, er);
  ol_e_rad2deg (er, er);

  return;
}

void
ol_e_ek (double* e, double* ek)
{
  ol_e_deg2rad (e , ek);
  ol_e_ek_rad  (ek, ek);
  ol_e_rad2deg (ek, ek);

  return;
}

void
ol_er_e (double* er, double* e)
{
  ol_e_deg2rad (er, e);
  ol_er_e_rad  (e , e);
  ol_e_rad2deg (e , e);

  return;
}

void
ol_ek_e (double* ek, double* e)
{
  ol_e_deg2rad (ek, e);
  ol_ek_e_rad  (e , e);
  ol_e_rad2deg (e , e);

  return;
}

double **
ol_g_alloc ()
{
  double **g = ut_alloc_2d (3, 3);

  g[0][0] = 1;
  g[1][1] = 1;
  g[2][2] = 1;

  return g;
}

void
ol_g_free (double **g)
{
  if (!g)
    return;
  else
    ut_free_2d (g, 3);

  return;
}

void
ol_g_set_zero (double **g)
{
  ut_array_2d_zero (g, 3, 3);

  return;
}

void
ol_g_set_id (double **g)
{
  ut_array_2d_zero (g, 3, 3);
  g[0][0] = 1;
  g[1][1] = 1;
  g[2][2] = 1;

  return;
}

void
ol_g_set_this (double **g, double g11, double g12, double g13, double g21, double g22, double g23, double g31, double g32, double g33)
{
  g[0][0] = g11;
  g[0][1] = g12;
  g[0][2] = g13;
  g[1][0] = g21;
  g[1][1] = g22;
  g[1][2] = g23;
  g[2][0] = g31;
  g[2][1] = g32;
  g[2][2] = g33;

  return;
}

void
ol_g_memcpy (double **gsrc, double **gdest)
{
  ut_array_2d_memcpy (gdest, 3, 3, gsrc);

  return;
}

int
ol_g_fscanf (FILE * file, double **g)
{
  if (fscanf (file, "%lf%lf%lf%lf%lf%lf%lf%lf%lf",
	      &g[0][0], &g[0][1], &g[0][2],
	      &g[1][0], &g[1][1], &g[1][2],
	      &g[2][0], &g[2][1], &g[2][2]) == 9)
    return 1;
  else
    return 0;
}

int
ol_g_fprintf (FILE * file, double **g, char *format)
{
  int i, j, res;

  for (i = 0; i < 3; i++)
  {
    for (j = 0; j < 3; j++)
    {
      fprintf (file, format, g[i][j]);
      fprintf (file, " ");
    }
    res = fprintf (file, "\n");
  }

  return res;
}

int *
ol_m_alloc ()
{
  int *m = ut_alloc_1d_int (6);

  m[2] = 1;
  m[3] = 1;

  return m;
}

void
ol_m_free (int *m)
{
  if (!m)
    return;
  else
    free (m);

  return;
}

void
ol_m_set_zero (int *m)
{
  ut_array_1d_int_zero (m, 6);

  return;
}

void
ol_m_set_id (int *m)
{
  ut_array_1d_int_zero (m, 6);
  m[2] = 1;
  m[3] = 1;

  return;
}

void
ol_m_set_this (int *m, int h, int k, int l, int u, int v, int w)
{
  m[0] = h;
  m[1] = k;
  m[2] = l;
  m[3] = u;
  m[4] = v;
  m[5] = w;

  return;
}

void
ol_m_memcpy (int *msrc, int *mdest)
{
  ut_array_1d_int_memcpy (mdest, 6, msrc);

  return;
}

void
ol_m_g (int *m, double **g)
{
  double M, N;

  M =
    sqrt ((double) m[0] * (double) m[0] + (double) m[1] * (double) m[1] +
	  (double) m[2] * (double) m[2]);
  N =
    sqrt ((double) m[3] * (double) m[3] + (double) m[4] * (double) m[4] +
	  (double) m[5] * (double) m[5]);

  g[0][0] = (double) m[3] / N;
  g[1][0] = (double) m[4] / N;
  g[2][0] = (double) m[5] / N;

  g[0][1] =
    ((double) m[1] * (double) m[5] - (double) m[2] * (double) m[4]) / (M * N);
  g[1][1] =
    ((double) m[2] * (double) m[3] - (double) m[0] * (double) m[5]) / (M * N);
  g[2][1] =
    ((double) m[0] * (double) m[4] - (double) m[1] * (double) m[3]) / (M * N);

  g[0][2] = (double) m[0] / M;
  g[1][2] = (double) m[1] / M;
  g[2][2] = (double) m[2] / M;

  return;
}

void
ol_g_m (double **g, char* crysym, int maxid, int maxres, int **allm, double *mstat,
	int *pqty)
{
  ol_g_m_quality (g, crysym, maxid, maxres, 0.5, allm, mstat, pqty);

  return;
}

void
ol_g_m_quality (double **g, char* crysym, int maxid, int maxres, double alpha, int **allm, double *mstat, int *pqty)
{
  int i, j, test, pos, hklpos, uvwpos, ID;
  double A;
  double *v = ut_alloc_1d (3);
  int *vsgn = ut_alloc_1d_int (3);
  double *vp = ut_alloc_1d (3);
  double *a = ut_alloc_1d (3);
  int *qty = ut_alloc_1d_int (2);

  int ***id = ut_alloc_3d_int (2, 5 * maxid, 3);	/* empirical */
  double *theta, *mq;
  int *Sig;
  int *srt;

  int *m = ol_m_alloc ();
  double **g2 = ol_g_alloc ();
  int scalprod, resqty;

  /*** Building a set of potential miller indices. ***/
  for (ID = 0; ID <= 1; ID++)
  {
    if (ID == 0)		/* Searching hkl */
      for (i = 0; i < 3; i++)
      {
	v[i] = fabs (g[i][2]);
	vsgn[i] = ut_num_sgn (g[i][2]);
      }
    else			/* Searching uvw */
      for (i = 0; i < 3; i++)
      {
	v[i] = fabs (g[i][0]);
	vsgn[i] = ut_num_sgn (g[i][0]);
      }

    qty[ID] = 0;
    ut_array_1d_zero (vp, 3);
    do
    {
      for (i = 0; i < 3; i++)
	a[i] = (floor (vp[i] + 0.5) + 0.5) / v[i];
      A = ut_array_1d_min (a, 3);
      /* A is the number by which to multiply v to get the
       * next id (one index incremented) */

      for (i = 0; i < 3; i++)
      {
	vp[i] = A * (1 + 1e-6) * v[i];
	id[ID][qty[ID]][i] = ut_num_d2ri (vp[i]);
      }

      test = 1;			/* if test = 1, recording under conditions; else, no */
      for (j = 0; j < qty[ID]; j++)	/* checking whether not the same as previously */
	if (ut_vector_int_angle (id[ID][j], id[ID][qty[ID]]) < 1e-3)
	{
	  test = 0;
	  break;
	}

      if (test == 1 && ut_array_1d_int_max (id[ID][qty[ID]], 3) <= maxid)
	qty[ID]++;
    }
    while (ut_array_1d_max (vp, 3) <= maxid + .5);

    for (i = 0; i < qty[ID]; i++)
      for (j = 0; j < 3; j++)
	id[ID][i][j] *= vsgn[j];
  }
  /*** Built a set of potential miller indices. ***/

  theta = ut_alloc_1d (qty[0] * qty[1]);
  Sig = ut_alloc_1d_int (qty[0] * qty[1]);
  mq = ut_alloc_1d (qty[0] * qty[1]);
  srt = ut_alloc_1d_int (qty[0] * qty[1]);

  resqty = qty[0] * qty[1];
  for (i = 0; i < qty[0]; i++)
  {
    ut_array_1d_int_memcpy (m, 3, id[0][i]);

    for (j = 0; j < qty[1]; j++)
    {
      pos = i * qty[1] + j;

      ut_array_1d_int_memcpy (m + 3, 3, id[1][j]);

      scalprod = m[0] * m[3] + m[1] * m[4] + m[2] * m[5];
      if (scalprod != 0)
	resqty--;

      ol_m_g (m, g2);
      ol_g_g_disori (g, g2, crysym, &(theta[pos]));

      Sig[pos] = (fabs (m[0]) + fabs (m[1]) + fabs (m[2])) +
	(fabs (m[3]) + fabs (m[4]) + fabs (m[5]));

      mq[pos] =
	(1 - alpha) * (double) Sig[pos] + alpha * theta[pos] +
	(double) (fabs (scalprod) * 1e7);
      /* if scalprod != 0, mq is set to a very high value to reject the
       * set of indices. */
    }
  }

  ut_array_1d_sort_index (mq, qty[0] * qty[1], srt);

  (*pqty) = ut_num_min_int (resqty, maxres);
  for (i = 0; i < (*pqty); i++)
  {
    pos = srt[i];
    hklpos = pos / qty[1];
    uvwpos = pos % qty[1];

    ut_array_1d_int_memcpy (allm[i], 3, id[0][hklpos]);
    ut_array_1d_int_memcpy (allm[i] + 3, 3, id[1][uvwpos]);
    mstat[i] = mq[pos];
  }

  /* freeing memory */
  ut_free_1d (v);
  ut_free_1d_int (vsgn);
  ut_free_1d (vp);
  ut_free_1d (a);
  ut_free_1d_int (qty);

  ut_free_3d_int (id, 2, 5 * maxid);
  ut_free_1d (theta);
  ut_free_1d_int (Sig);
  ut_free_1d (mq);
  ut_free_1d_int (srt);

  ol_m_free (m);
  ol_g_free (g2);

  return;
}

void
ol_e_m (double *e, char* crysym, int maxid, int maxres, int **allm, double *mstat,
	int *pqty)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_m_quality (g, crysym, maxid, maxres, 0.5, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_e_m_quality (double *e, char* crysym, int maxid, int maxres, double alpha, int **allm, double *mstat, int *pqty)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_m_quality (g, crysym, maxid, maxres, alpha, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_rtheta_m (double *r, double theta, char* crysym, int maxid, int maxres, int **allm, double *mstat, int *pqty)
{
  double **g = ol_g_alloc ();
  ol_rtheta_g (r, theta, g);

  ol_g_m_quality (g, crysym, maxid, maxres, 0.5, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_rtheta_m_quality (double *r, double theta, char* crysym, int maxid, int maxres, double alpha, int **allm, double *mstat, int *pqty)
{
  double **g = ol_g_alloc ();
  ol_rtheta_g (r, theta, g);

  ol_g_m_quality (g, crysym, maxid, maxres, alpha, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_R_m (double *R, char* crysym, int maxid, int maxres, int **allm, double *mstat,
	int *pqty)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_m_quality (g, crysym, maxid, maxres, 0.5, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_R_m_quality (double *R, char* crysym, int maxid, int maxres, double alpha, int **allm, double *mstat, int *pqty)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_m_quality (g, crysym, maxid, maxres, alpha, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_q_m (double *q, char* crysym, int maxid, int maxres, int **allm, double *mstat,
	int *pqty)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_m_quality (g, crysym, maxid, maxres, 0.5, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

void
ol_q_m_quality (double *q, char* crysym, int maxid, int maxres, double alpha, int **allm, double *mstat, int *pqty)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_m_quality (g, crysym, maxid, maxres, alpha, allm, mstat, pqty);

  ol_g_free (g);

  return;
}

int
ol_m_fscanf (FILE * file, int *m)
{
  if (fscanf (file, "%d%d%d%d%d%d",
	      &m[0], &m[1], &m[2], &m[3], &m[4], &m[5]) == 6)
    return 1;
  else
    return 0;
}

int
ol_m_fprintf (FILE * file, int *m, char *format)
{
  int i;

  for (i = 0; i <= 5; i++)
  {
    fprintf (file, format, m[i]);
    fprintf (file, " ");
  }
  return fprintf (file, "\n");
}

void
ol_m_e (int *m, double *e)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_e (g, e);

  ol_g_free (g);

  return;
}

void
ol_m_e_rad (int *m, double *e)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_e_rad (g, e);

  ol_g_free (g);

  return;
}

void
ol_m_q (int *m, double *q)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_q (g, q);

  ol_g_free (g);

  return;
}

void
ol_m_rtheta (int *m, double *r, double *ptheta)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_rtheta (g, r, ptheta);

  ol_g_free (g);

  return;
}

void
ol_m_rtheta_rad (int *m, double *r, double *ptheta)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_rtheta_rad (g, r, ptheta);

  ol_g_free (g);

  return;
}

void
ol_m_r (int *m, double *r)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_r (g, r);

  ol_g_free (g);

  return;
}

void
ol_m_theta_rad (int *m, double *ptheta)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_theta_rad (g, ptheta);

  ol_g_free (g);

  return;
}

void
ol_m_theta (int *m, double *ptheta)
{
  ol_m_theta_rad (m, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}



void
ol_m_R (int *m, double *R)
{
  double **g = ol_g_alloc ();

  ol_m_g (m, g);
  ol_g_R (g, R);

  ol_g_free (g);

  return;
}

int *
ol_pole_alloc (void)
{
  int *pole = ut_alloc_1d_int (3);

  return pole;
}

int *
ol_hpole_alloc (void)
{
  int *hpole = ut_alloc_1d_int (4);

  return hpole;
}

double *
ol_vect_alloc (void)
{
  double *vect = ut_alloc_1d (3);

  return vect;
}

double *
ol_p_alloc (void)
{
  double *p = ut_alloc_1d (2);

  return p;
}

void
ol_pole_free (int *pole)
{
  if (!pole)
    return;
  else
    ut_free_1d_int (pole);

  return;
}

void
ol_hpole_free (int *hpole)
{
  if (!hpole)
    return;
  else
    ut_free_1d_int (hpole);

  return;
}

void
ol_vect_free (double *vect)
{
  if (!vect)
    return;
  else
    ut_free_1d (vect);

  return;
}

void
ol_p_free (double *p)
{
  if (!p)
    return;
  else
    ut_free_1d (p);

  return;
}

int
ol_pole_fprintf (FILE * file, int *pole, char *format)
{
  fprintf (file, format, pole[0]);
  fprintf (file, " ");
  fprintf (file, format, pole[1]);
  fprintf (file, " ");
  fprintf (file, format, pole[2]);
  return fprintf (file, "\n");
}

int
ol_pole_fscanf (FILE * file, int *pole)
{
  if (fscanf (file, "%d%d%d", &pole[0], &pole[1], &pole[2]) == 3)
    return 1;
  else
    return 0;
}

int
ol_hpole_fprintf (FILE * file, int *hpole, char *format)
{
  fprintf (file, format, hpole[0]);
  fprintf (file, " ");
  fprintf (file, format, hpole[1]);
  fprintf (file, " ");
  fprintf (file, format, hpole[2]);
  fprintf (file, " ");
  fprintf (file, format, hpole[3]);
  return fprintf (file, "\n");
}

int
ol_hpole_fscanf (FILE * file, int *hpole)
{
  return fscanf (file, "%d%d%d%d", &hpole[0], &hpole[1], &hpole[2],
		 &hpole[3]);
}

int
ol_vect_fprintf (FILE * file, double *vect, char *format)
{
  fprintf (file, format, vect[0]);
  fprintf (file, " ");
  fprintf (file, format, vect[1]);
  fprintf (file, " ");
  fprintf (file, format, vect[2]);
  return fprintf (file, "\n");
}

int
ol_vect_fscanf (FILE * file, double *vect)
{
  return fscanf (file, "%lf%lf%lf", &vect[0], &vect[1], &vect[2]);
}

int
ol_p_fprintf (FILE * file, double *p, char *format)
{
  fprintf (file, format, p[0]);
  fprintf (file, " ");
  fprintf (file, format, p[1]);
  return fprintf (file, "\n");
}

int
ol_p_fscanf (FILE * file, double *p)
{
  return fscanf (file, "%lf%lf", &p[0], &p[1]);
}

void
ol_pole_vect (int *m, double *vect)
{
  int i;
  double norm;

  norm = ut_array_1d_norm_int (m, 3);

  for (i = 0; i < 3; i++)	/* cannot copy using ut_; int to double ... */
    vect[i] = (double) m[i];
  ut_array_1d_scale (vect, 3, 1 / norm);

  return;
}

void
ol_hpole_vect (int *hm, double *vect)
{
  int i, j;
  double norm;
  double **a = ut_alloc_2d (4, 3);

  /* setting the non-zero terms only */
  a[0][0] = sqrt (3) / 2;
  a[0][1] = -.5;
  a[1][1] = 1;
  a[2][0] = -sqrt (3) / 2;
  a[2][1] = -.5;
  a[3][2] = 1;

  ut_array_1d_zero (vect, 3);
  for (i = 0; i < 3; i++)
    for (j = 0; j < 4; j++)
      vect[i] += (double) hm[j] * a[j][i];

  norm = ut_array_1d_norm (vect, 3);
  ut_array_1d_scale (vect, 3, 1 / norm);

  ut_free_2d (a, 4);

  return;
}

void
ol_pole_set_zero (int *pole)
{
  ut_array_1d_int_zero (pole, 3);

  return;
}

void
ol_pole_set_this (int *pole, int x, int y, int z)
{
  pole[0] = x;
  pole[1] = y;
  pole[2] = z;

  return;
}

void
ol_pole_memcpy (int *polesrc, int *poledest)
{
  ut_array_1d_int_memcpy (poledest, 3, polesrc);

  return;
}

void
ol_vect_set_zero (double *vect)
{
  ut_array_1d_zero (vect, 3);

  return;
}

void
ol_vect_set_this (double *vect, double x, double y, double z)
{
  vect[0] = x;
  vect[1] = y;
  vect[2] = z;

  return;
}

void
ol_vect_memcpy (double *vectsrc, double *vectdest)
{
  ut_array_1d_memcpy (vectdest, 3, vectsrc);

  return;
}

void
ol_p_set_zero (double *p)
{
  ut_array_1d_zero (p, 2);

  return;
}

void
ol_p_set_this (double *p, double x, double y)
{
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_p_memcpy (double *psrc, double *pdest)
{
  ut_array_1d_memcpy (pdest, 2, psrc);

  return;
}

void
ol_vect_stproj_rad (double *vect, double *p)
{
  double alpha, beta;

  /* Calculation of alpha and beta */

  alpha = ut_num_acos (vect[2]);

  if (fabs (alpha) < 1e-6)	/* beta is undefined; choosing 0 */
    beta = 0;
  else
    beta = atan2 (vect[1] / sin (alpha), vect[0] / sin (alpha));

  if (alpha > OL_PI / 2)
  {
    alpha = OL_PI - alpha;
    beta += OL_PI;
  }

  /* calculation of p
   * 0: Op; 1: beta (angle from RD)
   */
  p[0] = tan (alpha / 2);
  p[1] = beta;

  return;
}

void
ol_vect_stproj (double *vect, double *p)
{
  ol_vect_stproj_rad (vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_vect_stprojxy (double *vect, double *p)
{
  double x, y;

  ol_vect_stproj_rad (vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_vect_eaproj (double *vect, double *p)
{
  ol_vect_eaproj_rad (vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_vect_eaprojxy (double *vect, double *p)
{
  double x, y;

  ol_vect_eaproj_rad (vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_vect_eaproj_rad (double *vect, double *p)
{
  ol_vect_stproj_rad (vect, p);

  p[0] = sqrt (2) * sin (atan (p[0]));

  return;
}

void
ol_g_pf_stproj (double **g, double *vect, double *p)
{
  ol_g_pf_stproj_rad (g, vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_pf_stprojxy (double **g, double *vect, double *p)
{
  double x, y;

  ol_g_pf_stproj_rad (g, vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_g_pf_stproj_rad (double **g, double *vect, double *p)
{
  int i, j;
  double *tmp = ol_vect_alloc ();

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      tmp[i] += g[j][i] * vect[j];	/* [j][i] == using the inverse of g */

  ol_vect_stproj_rad (tmp, p);

  ol_vect_free (tmp);

  return;
}

void
ol_g_pf_eaproj (double **g, double *vect, double *p)
{
  ol_g_pf_eaproj_rad (g, vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_pf_eaprojxy (double **g, double *vect, double *p)
{
  double x, y;

  ol_g_pf_eaproj_rad (g, vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}


void
ol_g_pf_eaproj_rad (double **g, double *vect, double *p)
{
  ol_g_pf_stproj_rad (g, vect, p);

  p[0] = sqrt (2) * sin (atan (p[0]));

  return;
}

void
ol_g_ipf_stproj (double **g, double *vect, double *p)
{
  ol_g_ipf_stproj_rad (g, vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_ipf_stprojxy (double **g, double *vect, double *p)
{
  double x, y;

  ol_g_ipf_stproj_rad (g, vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_g_ipf_stproj_rad (double **g, double *vect, double *p)
{
  int i, j;
  double *tmp = ol_vect_alloc ();

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      tmp[i] += g[i][j] * vect[j];

  ol_vect_stproj_rad (tmp, p);

  ol_vect_free (tmp);

  return;
}

void
ol_g_ipf_eaproj (double **g, double *vect, double *p)
{
  ol_g_ipf_eaproj_rad (g, vect, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_ipf_eaprojxy (double **g, double *vect, double *p)
{
  double x, y;

  ol_g_ipf_eaproj_rad (g, vect, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_g_ipf_eaproj_rad (double **g, double *vect, double *p)
{
  ol_g_ipf_stproj_rad (g, vect, p);

  p[0] = sqrt (2) * sin (atan (p[0]));

  return;
}

void
ol_g_ipf_crysym_stproj (double **g, double *vect, char* crysym, double *p)
{
  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_ipf_crysym_stprojxy (double **g, double *vect, char* crysym, double *p)
{
  double x, y;

  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

/* could really be improved for cpu efficiency */
/* Does this work for crystals other than cubic??????? */
void
ol_g_ipf_crysym_stproj_rad (double **g, double *vect, char* crysym, double *p)
{
  int i, j;
  double alpha, beta;
  double *tmp = ut_alloc_1d (3);
  double **gsym = ol_g_alloc ();
  double *v = ol_vect_alloc ();
  double *weight = ol_e_alloc ();
  int qty = ol_crysym_qty (crysym);

  /* Calculation of alpha and beta */

  /* Choosing the right symmetry */
  for (i = 1; i <= qty; i++)
  {
    ol_g_crysym (g, crysym, i, gsym);
    ol_g_ipf_stprojxy (gsym, vect, p);
    ol_stprojxy_vect (p, v);
    if (ol_vect_ipfweight (v, weight) == 0)
      break;
  }

  for (i = 0; i < 3; i++)
    for (j = 0; j < 3; j++)
      tmp[i] += gsym[i][j] * vect[j];

  alpha = ut_num_acos (tmp[2]);

  if (fabs (alpha) < 1e-6)	/* beta is undefined; choosing 0 */
    beta = 0;
  else
    beta = atan2 (tmp[1] / sin (alpha), tmp[0] / sin (alpha));

  if (alpha > OL_PI / 2)
  {
    alpha = OL_PI - alpha;
    beta += OL_PI;
  }

  /* calculation of p
   * 0: Op; 1: beta (angle from RD)
   */
  p[0] = tan (alpha / 2);
  p[1] = beta;

  ut_free_1d (tmp);
  ol_g_free (gsym);
  ol_vect_free (v);
  ol_e_free (weight);

  return;
}

void
ol_g_ipf_crysym_eaproj (double **g, double *vect, char* crysym, double *p)
{
  ol_g_ipf_crysym_eaproj_rad (g, vect, crysym, p);
  ol_theta_rad2deg (p[1], &(p[1]));

  return;
}

void
ol_g_ipf_crysym_eaprojxy (double **g, double *vect, char* crysym, double *p)
{
  double x, y;

  ol_g_ipf_crysym_eaproj_rad (g, vect, crysym, p);

  x = p[0] * cos (p[1]);
  y = p[0] * sin (p[1]);
  p[0] = x;
  p[1] = y;

  return;
}

void
ol_g_ipf_crysym_eaproj_rad (double **g, double *vect, char* crysym, double *p)
{
  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);

  p[0] = sqrt (2) * sin (atan (p[0]));

  return;
}

void
ol_stproj_vect_rad (double *p, double *vect)
{
  double alpha = 2 * atan (p[0]);

  vect[0] = sin (alpha) * cos (p[1]);
  vect[1] = sin (alpha) * sin (p[1]);
  vect[2] = cos (alpha);

  return;
}

void
ol_eaproj_vect_rad (double *p, double *vect)
{
  double alpha = 2 * asin (p[0] / sqrt (2));

  vect[0] = sin (alpha) * cos (p[1]);
  vect[1] = sin (alpha) * sin (p[1]);
  vect[2] = cos (alpha);

  return;
}

void
ol_stproj_vect (double *p, double *vect)
{
  double *pcpy = ol_p_alloc ();

  ol_p_memcpy (p, pcpy);
  ol_theta_deg2rad (pcpy[1], &(pcpy[1]));
  ol_stproj_vect_rad (pcpy, vect);

  ol_p_free (pcpy);

  return;
}

void
ol_eaproj_vect (double *p, double *vect)
{
  double *pcpy = ol_p_alloc ();

  ol_p_memcpy (p, pcpy);
  ol_theta_deg2rad (pcpy[1], &(pcpy[1]));
  ol_eaproj_vect_rad (pcpy, vect);

  ol_p_free (pcpy);

  return;
}

void
ol_stprojxy_vect (double *p, double *vect)
{
  double *psh = ol_p_alloc ();

  psh[0] = sqrt (p[0] * p[0] + p[1] * p[1]);

  if (psh[0] > 1e-12)
    psh[1] = atan2 (p[1] / psh[0], p[0] / psh[0]);
  else
    psh[1] = 0;

  ol_stproj_vect_rad (psh, vect);

  ol_p_free (psh);

  return;
}

void
ol_eaprojxy_vect (double *p, double *vect)
{
  double *psh = ol_p_alloc ();

  psh[0] = sqrt (p[0] * p[0] + p[1] * p[1]);

  if (psh[0] > 1e-12)
    psh[1] = atan2 (p[1] / psh[0], p[0] / psh[0]);
  else
    psh[1] = 0;

  ol_eaproj_vect_rad (psh, vect);

  ol_p_free (psh);

  return;
}

int
ol_vect_ipfweight (double *v, double *weight)
{
  double *v1 = ol_vect_alloc ();
  double *v2 = ol_vect_alloc ();
  double *v3 = ol_vect_alloc ();
  double *r = ol_r_alloc ();
  double **g = ol_g_alloc ();
  double t, theta;
  int test = 0;

  ol_e_set_zero (weight);

  if (v[0] > v[1])
    test = -1;

  if (test != -1)
  {
    /* calculating v1 = rotated vector to (001)-(011) about (01-1) */
    ol_r_set_this (r, 0, 1 / sqrt (2), -1 / sqrt (2));

    ol_theta_rad2deg (atan2 (sqrt (2) * v[0], (v[1] + v[2])), &theta);

    ol_rtheta_g (r, theta, g);
    ol_g_vect_vect (g, v, v1);
/* computing theta3 = angle to reach segment (001)-(111) by rotation
   * about r (theta3 has a negative value) 

  * theta3 is the solution of equation,
   * cos(theta3).v0 - sin(theta3).(v1+v2)/sqrt(2)
   = cos(theta3).(v1+v2)/2 + sin(theta3).v0/sqrt(2) + (v1-v2)/2
   which is equivalent to,
   t^2 .2(v2-v0) + t (-2sqrt(2).(v0+v1+v2)) + 2(v0-v1) = 0
   with t = tan (theta3 / 2)
   (( using cos (theta3) = (1 - t^2) / (1 + t^2) ))
   ((       sin (theta3) =     2t    / (1 + t^2) ))

  a = 2 * (v[2] - v[0]);
  b = -2 * sqrt(2) * (v[0] + v[1] + v[2]);
  c = 2 * (v[0] - v[1]);

  if (fabs (a) < 1e-6)
    t = - c / b;
  else
  {
    delta = b*b - 4 * a * c;
    t = (-b - sqrt(delta)) / (2 * a);
  }

  ol_theta_rad2deg (2 * atan (t), &thetab);

  ol_rtheta_g (r, thetab, g);
  ol_g_vect_vect (g, v, v1b);

  w = 1 - acos (v1[2]) / (OL_PI / 4);
  wb = 1 - acos(v1b[2]) / acos(1/sqrt(3));
  ratio = theta / (theta + thetab);
  weight[0] = w; */
    weight[0] = 1 - acos (v1[2]) / (OL_PI / 4);

    if (weight[0] < 0 && weight[0] > -OL_EPS)
      weight[0] = 0;
    if (weight[0] > 1 && weight[0] < 1 + OL_EPS)
      weight[0] = 1;
    if (weight[0] < 0 || weight[0] > 1)
      test = -1;
  }

  if (test != -1)
  {
    /* calculating v2 = rotated vector to (001)-(011) about (-110) */

    ol_r_set_this (r, -1 / sqrt (2), 1 / sqrt (2), 0);

    if (fabs (v[1]) < OL_EPS)
      t = -v[0] / (sqrt (2) * v[2]);
    else
      t =
	(sqrt (2) * v[2] -
	 sqrt (2 * v[2] * v[2] + 4 * v[0] * v[1])) / (2 * v[1]);

    ol_theta_rad2deg (2 * atan (t), &theta);

    ol_rtheta_g (r, -theta, g);
    ol_g_vect_vect (g, v, v2);
    weight[1] = ut_num_acos (v2[2]) / (OL_PI / 4);
    if (weight[1] < 0 && weight[1] > -OL_EPS)
      weight[1] = 0;
    if (weight[1] > 1 && weight[1] < 1 + OL_EPS)
      weight[1] = 1;
    if (weight[1] < 0 || weight[1] > 1)
      test = -1;
  }

  if (test != -1)
  {
    /* rotation to (001)--(111) */
    /* calculating v3 = rotated vector to (011)-(111) about (-100) */

    ol_r_set_this (r, -1, 0, 0);

    ol_theta_rad2deg (atan2 (v[1] - v[2], v[1] + v[2]), &theta);

    ol_rtheta_g (r, theta, g);
    ol_g_vect_vect (g, v, v3);

    weight[2] =
      1 - acos ((v3[0] + v3[1] + v3[2]) / sqrt (3)) / acos (2 / sqrt (6));
    if (weight[2] < 0 && weight[2] > -OL_EPS)
      weight[2] = 0;
    if (weight[2] > 1 && weight[2] < 1 + OL_EPS)
      weight[2] = 1;
    if (weight[2] < 0 || weight[2] > 1)
      test = -1;
  }

  ol_g_free (g);
  ol_vect_free (v1);
  ol_vect_free (v2);
  ol_vect_free (v3);
  ol_r_free (r);

  return test;
}

void
ol_q_pf_stproj (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_pf_stprojxy (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_pf_stproj_rad (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_pf_eaproj (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_pf_eaprojxy (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_pf_eaproj_rad (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_pf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_stproj (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_stprojxy (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_stproj_rad (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_eaproj (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_eaprojxy (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_eaproj_rad (double *q, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_stproj (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_stproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_stprojxy (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_stprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_stproj_rad (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_eaproj (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_eaproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_eaprojxy (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_eaprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_q_ipf_crysym_eaproj_rad (double *q, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_q_g (q, g);

  ol_g_ipf_crysym_eaproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_stproj (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_stprojxy (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_stproj_rad (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_eaproj (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_eaprojxy (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_pf_eaproj_rad (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_pf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_stproj (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_stprojxy (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_stproj_rad (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_eaproj (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_eaprojxy (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_eaproj_rad (double *e, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_stproj (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_stproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_stprojxy (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_stprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_stproj_rad (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_eaproj (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_eaproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_eaprojxy (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_eaprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_e_ipf_crysym_eaproj_rad (double *e, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_e_g (e, g);

  ol_g_ipf_crysym_eaproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_stproj (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_stprojxy (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_stproj_rad (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_eaproj (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_eaprojxy (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_pf_eaproj_rad (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_pf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_stproj (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_stproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_stprojxy (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_stprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_stproj_rad (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_stproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_eaproj (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_eaproj (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_eaprojxy (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_eaprojxy (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_eaproj_rad (double *R, double *vect, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_eaproj_rad (g, vect, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_stproj (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_stproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_stprojxy (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_stprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_stproj_rad (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_stproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_eaproj (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_eaproj (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_eaprojxy (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_eaprojxy (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

void
ol_R_ipf_crysym_eaproj_rad (double *R, double *vect, char* crysym, double *p)
{
  double **g = ol_g_alloc ();
  ol_R_g (R, g);

  ol_g_ipf_crysym_eaproj_rad (g, vect, crysym, p);

  ol_g_free (g);

  return;
}

double *
ol_q_alloc ()
{
  double *q = ut_alloc_1d (4);

  q[0] = 1;

  return q;
}

void
ol_q_free (double *q)
{
  if (!q)
    return;
  else
    free (q);

  return;
}

void
ol_q_set_zero (double *q)
{
  ut_array_1d_zero (q, 4);

  return;
}

void
ol_q_set_id (double *q)
{
  q[0] = 1;
  ut_array_1d_zero (q + 1, 3);

  return;
}

void
ol_q_set_unit (double *q)
{
  double norm = sqrt (q[0] * q[0] + q[1] * q[1] + q[2] * q[2] + q[3] * q[3]);
  ut_array_1d_scale (q, 4, 1 / norm);

  return;
}


void
ol_q_set_this (double *q, double rho, double lambda, double mu, double nu)
{
  q[0] = rho;
  q[1] = lambda;
  q[2] = mu;
  q[3] = nu;

  return;
}

void
ol_q_memcpy (double *qsrc, double *qdest)
{
  ut_array_1d_memcpy (qdest, 4, qsrc);

  return;
}

void
ol_g_q (double **g, double *q)
{
  int i, m;

  q[0] = 0.5 * ut_num_sqrt (g[0][0] + g[1][1] + g[2][2] + 1);

  if (fabs (q[0]) > OL_EPS)
  {
    q[1] = (g[1][2] - g[2][1]) / (4 * q[0]);
    q[2] = (g[2][0] - g[0][2]) / (4 * q[0]);
    q[3] = (g[0][1] - g[1][0]) / (4 * q[0]);
  }
  else
  {
    for (i = 0; i <= 2; i++)
      q[i + 1] = ut_num_sqrt ((g[i][i] + 1) / 2);

    m = 1 + ut_array_1d_max_index (q + 1, 3);
    for (i = 1; i <= 3; i++)
      if (i != m)
	q[i] *= ut_num_sgn (g[i - 1][m - 1]);
  }

  return;
}

void
ol_q_g (double *q, double **g)
{
  g[0][0] =
    ut_num_p2 (q[0]) + ut_num_p2 (q[1]) - ut_num_p2 (q[2]) - ut_num_p2 (q[3]);
  g[1][1] =
    ut_num_p2 (q[0]) - ut_num_p2 (q[1]) + ut_num_p2 (q[2]) - ut_num_p2 (q[3]);
  g[2][2] =
    ut_num_p2 (q[0]) - ut_num_p2 (q[1]) - ut_num_p2 (q[2]) + ut_num_p2 (q[3]);

  g[0][1] = 2 * (q[1] * q[2] + q[0] * q[3]);
  g[0][2] = 2 * (q[1] * q[3] - q[0] * q[2]);

  g[1][0] = 2 * (q[1] * q[2] - q[0] * q[3]);
  g[1][2] = 2 * (q[2] * q[3] + q[0] * q[1]);

  g[2][0] = 2 * (q[1] * q[3] + q[0] * q[2]);
  g[2][1] = 2 * (q[2] * q[3] - q[0] * q[1]);

  return;
}

void
ol_rtheta_q (double *r, double theta, double *q)
{
  ol_rtheta_q_rad (r, theta * OL_PI / 180, q);

  return;
}

void
ol_rtheta_q_rad (double *r, double theta, double *q)
{
  int i;

  q[0] = cos (theta / 2);

  for (i = 0; i <= 2; i++)
    q[i + 1] = r[i] * sin (theta / 2);

  return;
}

void
ol_q_rtheta (double *q, double *r, double *ptheta)
{
  ol_q_rtheta_rad (q, r, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_q_rtheta_rad (double *q, double *r, double *ptheta)
{
  int i;

  (*ptheta) = 2 * ut_num_acos (q[0]);

  if ((*ptheta) < OL_EPS_DEG)
  {
    r[0] = 1;
    r[1] = 0;
    r[2] = 0;
  }
  else
    for (i = 0; i <= 2; i++)
      r[i] = q[i + 1] / sin ((*ptheta) / 2);

  return;
}

void
ol_q_r (double *q, double *r)
{
  double theta;

  ol_q_rtheta_rad (q, r, &theta);
}

void
ol_q_R (double *q, double *R)
{
  double *r = ol_r_alloc ();
  double theta;

  ol_q_rtheta_rad (q, r, &theta);
  ol_rtheta_R_rad (r, theta, R);

  ol_r_free (r);

  return;
}

void
ol_R_q (double *R, double *q)
{
  double *r = ol_r_alloc ();
  double theta;

  ol_R_rtheta_rad (R, r, &theta);
  ol_rtheta_q_rad (r, theta, q);

  ol_r_free (r);

  return;
}

void
ol_q_theta_rad (double *q, double *ptheta)
{
  (*ptheta) = 2 * ut_num_acos (q[0]);

  return;
}

void
ol_q_theta (double *q, double *ptheta)
{
  ol_q_theta_rad (q, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_e_q (double *e, double *q)
{
  double *e2;

  e2 = ol_e_alloc ();
  ol_e_deg2rad (e, e2);

  ol_e_q_rad (e2, q);

  ol_e_free (e2);

  return;
}

void
ol_e_q_rad (double *e, double *q)
{
  q[0] = cos (e[1] / 2) * cos ((e[0] + e[2]) / 2);
  q[1] = sin (e[1] / 2) * cos ((e[2] - e[0]) / 2);
  q[2] = sin (e[1] / 2) * sin ((e[0] - e[2]) / 2);
  q[3] = cos (e[1] / 2) * sin ((e[0] + e[2]) / 2);

  return;
}

void
ol_q_e (double *q, double *e)
{
  ol_q_e_rad (q, e);

  ol_e_rad2deg (e, e);

  return;
}

void
ol_q_e_rad (double *q, double *e)
{
  e[1] = 2 * atan2 (sqrt (q[1] * q[1] + q[2] * q[2]),
		    sqrt (q[0] * q[0] + q[3] * q[3]));

  if (fabs (e[1]) > OL_EPS_DEG && fabs (OL_PI - e[1]) > OL_EPS_DEG)
  {
    e[0] = atan2 (q[3], q[0]) + atan2 (q[2], q[1]);
    e[2] = atan2 (q[3], q[0]) - atan2 (q[2], q[1]);
  }
  else if (fabs (e[1]) < OL_EPS_DEG)
  {
    e[0] = 2 * acos (q[0]);
    e[2] = 0;
  }
  else
  {
    e[0] = 2 * acos (q[1]);
    e[2] = 0;
  }


  return;
}

void
ol_q_q (double *q1, double *q2)
{
  int i;

  ol_q_memcpy (q1, q2);

  for (i = 0; i < 4; i++)
    if (fabs (q2[i]) > OL_EPS)
    {
      if (q2[i] < 0)
	ut_array_1d_scale (q2, 4, -1);

      break;
    }

  return;
}

int
ol_q_fprintf (FILE * file, double *q, char *format)
{
  int i;

  for (i = 0; i <= 3; i++)
  {
    fprintf (file, format, q[i]);
    fprintf (file, " ");
  }
  return fprintf (file, "\n");

}

int
ol_q_fscanf (FILE * file, double *q)
{
  if (fscanf (file, "%lf%lf%lf%lf", &q[0], &q[1], &q[2], &q[3]) == 4)
    return 1;
  else
    return 0;
}

double *
ol_r_alloc ()
{
  double *r = ut_alloc_1d (3);

  r[0] = 1;

  return r;
}

void
ol_r_free (double *r)
{
  if (!r)
    return;
  else
    free (r);

  return;
}

void
ol_r_set_zero (double *r)
{
  ut_array_1d_zero (r, 3);

  return;
}

void
ol_r_set_id (double *r)
{
  r[0] = 1;
  r[1] = 0;
  r[2] = 0;

  return;
}

void
ol_r_set_unit (double *r)
{
  double norm = ut_vector_norm (r);
  ut_array_1d_scale (r, 3, 1 / norm);

  return;
}


void
ol_r_set_this (double *r, double r1, double r2, double r3)
{
  r[0] = r1;
  r[1] = r2;
  r[2] = r3;

  return;
}

void
ol_rtheta_set_zero (double *r, double *ptheta)
{
  ut_array_1d_zero (r, 3);
  (*ptheta) = 0;

  return;
}

void
ol_rtheta_set_id (double *r, double *ptheta)
{
  r[0] = 1;
  r[1] = 0;
  r[2] = 0;
  (*ptheta) = 0;

  return;
}

void
ol_rtheta_set_this (double *r, double *ptheta, double r1, double r2, double r3, double thetaval)
{
  r[0] = r1;
  r[1] = r2;
  r[2] = r3;
  (*ptheta) = thetaval;

  return;
}

void
ol_theta_deg2rad (double theta, double *ptheta2)
{
  (*ptheta2) = theta * .01745329251994329576;	/* OL_PI/180 */

  return;
}

void
ol_theta_rad2deg (double theta, double *ptheta2)
{
  (*ptheta2) = theta * 57.29577951308232087721;	/* 180/OL_PI */

  return;
}

void
ol_r_memcpy (double *rsrc, double *rdest)
{
  ut_array_1d_memcpy (rdest, 3, rsrc);

  return;
}

void
ol_rtheta_memcpy (double *rsrc, double thetasrc, double *rdest,
		  double *pthetadest)
{
  ut_array_1d_memcpy (rdest, 3, rsrc);
  (*pthetadest) = thetasrc;

  return;
}

int
ol_g_rtheta (double **g, double *r, double *ptheta)
{
  int status;
  
  status = ol_g_rtheta_rad (g, r, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return status;
}

int
ol_g_r (double **g, double *r)
{
  double theta;

  return ol_g_rtheta_rad (g, r, &theta);
}


int
ol_g_rtheta_rad (double **g, double *r, double *ptheta)
{
  int i, m;
  int status;

  (*ptheta) = ut_num_acos ((g[0][0] + g[1][1] + g[2][2] - 1) / 2);

  status = 0;
  if ((*ptheta) < OL_EPS_RAD)
  {
    r[0] = 1;
    r[1] = 0;
    r[2] = 0;
  }
  else if ((*ptheta) < (1 - OL_EPS_RAD) * OL_PI)
  {
    if (ut_num_equal (g[1][2], g[2][1], OL_EPS) != 1
     || ut_num_equal (g[1][2], g[2][1], OL_EPS) != 1
     || ut_num_equal (g[1][2], g[2][1], OL_EPS) != 1)
      status = -1;

    r[0] = (g[1][2] - g[2][1]) / (2 * sin (*ptheta));
    r[1] = (g[2][0] - g[0][2]) / (2 * sin (*ptheta));
    r[2] = (g[0][1] - g[1][0]) / (2 * sin (*ptheta));
  }
  else
  {
    r[0] = ut_num_sqrt ((g[0][0] + 1) / 2);
    r[1] = ut_num_sqrt ((g[1][1] + 1) / 2);
    r[2] = ut_num_sqrt ((g[2][2] + 1) / 2);

    m = ut_array_1d_max_index (r, 3);
    for (i = 0; i < 3; i++)
      if (i != m)
	r[i] *= ut_num_sgn (g[i][m]);
  }

  return -1;
}

void
ol_g_theta (double **g, double *ptheta)
{
  ol_g_theta_rad (g, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_g_theta_rad (double **g, double *ptheta)
{
  (*ptheta) = ut_num_acos ((g[0][0] + g[1][1] + g[2][2] - 1) / 2);

  return;
}

void
ol_rtheta_g (double *r, double theta, double **g)
{
  ol_rtheta_g_rad (r, theta * OL_PI / 180, g);

  return;
}

void
ol_rtheta_g_rad (double *r, double theta, double **g)
{
  double C, S;
  C = cos (theta);
  S = sin (theta);

  g[0][0] = r[0] * r[0] * (1 - C) + C;
  g[0][1] = r[0] * r[1] * (1 - C) + r[2] * S;
  g[0][2] = r[0] * r[2] * (1 - C) - r[1] * S;

  g[1][0] = r[1] * r[0] * (1 - C) - r[2] * S;
  g[1][1] = r[1] * r[1] * (1 - C) + C;
  g[1][2] = r[1] * r[2] * (1 - C) + r[0] * S;

  g[2][0] = r[2] * r[0] * (1 - C) + r[1] * S;
  g[2][1] = r[2] * r[1] * (1 - C) - r[0] * S;
  g[2][2] = r[2] * r[2] * (1 - C) + C;

  return;
}

int
ol_rtheta_fprintf (FILE * file, double *r, double theta, char *format)
{
  fprintf (file, format, r[0]);
  fprintf (file, " ");
  fprintf (file, format, r[1]);
  fprintf (file, " ");
  fprintf (file, format, r[2]);
  fprintf (file, " ");
  fprintf (file, format, theta);
  return fprintf (file, "\n");
}

int
ol_r_fprintf (FILE * file, double *r, char *format)
{
  fprintf (file, format, r[0]);
  fprintf (file, " ");
  fprintf (file, format, r[1]);
  fprintf (file, " ");
  fprintf (file, format, r[2]);
  return fprintf (file, "\n");

}

int
ol_theta_fprintf (FILE * file, double theta, char *format)
{
  fprintf (file, format, theta);
  return fprintf (file, "\n");
}

int
ol_rtheta_fscanf (FILE * file, double *r, double *ptheta)
{
  if (fscanf (file, "%lf%lf%lf%lf", &r[0], &r[1], &r[2], ptheta) == 4)
    return 1;
  else
    return 0;
}

int
ol_r_fscanf (FILE * file, double *r)
{
  if (fscanf (file, "%lf%lf%lf", &r[0], &r[1], &r[2]) == 3)
    return 1;
  else
    return 0;
}

int
ol_theta_fscanf (FILE * file, double *ptheta)
{
  if (fscanf (file, "%lf", ptheta) == 1)
    return 1;
  else
    return 0;
}

void
ol_rtheta_rtheta_rad (double *r1, double theta, double *r2, double *ptheta2)
{
  ol_r_memcpy (r1, r2);

  (*ptheta2) = theta;

  /* ranging theta within [0,(2*OL_PI)] */
  while (*ptheta2 < 0)
    *ptheta2 += 2 * OL_PI;
  while (*ptheta2 > 2 * OL_PI)
    *ptheta2 -= 2 * OL_PI;

  /* ranging theta within [0, OL_PI] */
  if ((*ptheta2) > OL_PI)
  {
    (*ptheta2) -= OL_PI;
    ut_array_1d_scale (r2, 3, -1);
  }

  return;
}

void
ol_rtheta_rtheta (double *r1, double theta, double *r2, double *ptheta2)
{
  ol_rtheta_rtheta_rad (r1, theta * OL_PI / 180, r2, ptheta2);
  ol_theta_rad2deg (*ptheta2, ptheta2);

  return;
}

void
ol_rtheta_e (double *r, double theta, double *e)
{
  double *q = ol_q_alloc ();

  ol_rtheta_q (r, theta, q);
  ol_q_e (q, e);

  ol_q_free (q);
}

void
ol_rtheta_e_rad (double *r, double theta, double *e)
{
  double *q = ol_q_alloc ();

  ol_rtheta_q_rad (r, theta, q);
  ol_q_e_rad (q, e);

  ol_q_free (q);
}
