/**********************************************************************
 * Orilib - routines for orientation manipulation.                    *
 * Copyright (C) 2007, 2008, 2009 Romain Quey                         *
 *                                                                    *
 * This program is free software: you can redistribute it and/or      *
 * modify it under the terms of the GNU General Public License as     *
 * published by the Free Software Foundation, either version 3 of the *
 * License, or (at your option) any later version.                    *
 *                                                                    *
 * This program is distributed in the hope that it will be useful,    *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of     *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  *
 * General Public License for more details.                           *
 *                                                                    *
 * You should have received a copy of the GNU General Public License  *
 * along with this program.  If not, see                              *
 * <http://www.gnu.org/licenses/>.                                    *
 *                                                                    *
 * project homepage: http://orilib.sourceforge.net                    *
 **********************************************************************/
#include"ol_gen.h"
#define frand() ((double) rand() / (RAND_MAX+1.0))

void
ol_nb_e (double n1, double n2, double n3, double *e)
{
  ol_nb_e_rad (n1, n2, n3, e);
  ol_e_rad2deg (e, e);

  return;
}

void
ol_nb_e_rad (double n1, double n2, double n3, double *e)
{
  e[0] = 2 * OL_PI * n1;
  e[1] = ut_num_acos (2 * n2 - 1);
  e[2] = 2 * OL_PI * n3;

  return;
}

void
ol_srand_e_rad (int seed, int qty, double **e)
{
  int i;
  srand (seed);

  for (i = 0; i < qty; i++)
    ol_nb_e_rad (frand (), frand (), frand (), e[i]);

  return;
}

void
ol_srand_e (int seed, int qty, double **e)
{
  int i;
  srand (seed);

  for (i = 0; i < qty; i++)
    ol_nb_e (frand (), frand (), frand (), e[i]);

  return;
}

void
ol_nb_r (double n1, double n2, double *r)
{
  double alpha, beta;

  if (n1 < 0 || n1 > 1 || n2 < 0 || n2 > 1)
    abort ();

  alpha = acos (2 * n1 - 1);
  beta = 2 * OL_PI * n2;

  r[0] = sin (alpha) * cos (beta);
  r[1] = sin (alpha) * sin (beta);
  r[2] = cos (alpha);

  return;
}

void
ol_nb_max_theta_rad (double n4, double thetamax, double *ptheta)
{
  if (thetamax < 0)
    abort ();
  else if (thetamax > OL_PI)
    thetamax = OL_PI;

  (*ptheta) = thetamax * pow (n4, 1. / 3.);

  return;
}

void
ol_nb_max_theta (double n4, double thetamax, double *ptheta)
{
  ol_nb_max_theta_rad (n4, thetamax * OL_PI / 180, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_nb_max_rtheta_rad (double n1, double n2, double n3,
		      double thetamax, double *r, double *ptheta)
{
  ol_nb_r (n1, n2, r);
  ol_nb_max_theta_rad (n3, thetamax, ptheta);

  return;
}

void
ol_nb_max_rtheta (double n1, double n2, double n3,
		  double thetamax, double *r, double *ptheta)
{
  ol_nb_r (n1, n2, r);
  ol_nb_max_theta (n3, thetamax, ptheta);

  return;
}

void
ol_nb_max_theta_mis_rad (double n4, double thetamax, double *ptheta)
{
  if (thetamax < 0)
    abort ();
  else if (thetamax > OL_PI)
    thetamax = OL_PI;

  (*ptheta) = thetamax * n4;

  return;
}

void
ol_nb_max_theta_mis (double n4, double thetamax, double *ptheta)
{
  ol_nb_max_theta_mis_rad (n4, thetamax * OL_PI / 180, ptheta);
  ol_theta_rad2deg (*ptheta, ptheta);

  return;
}

void
ol_nb_max_rtheta_mis_rad (double n1, double n2, double n3,
			  double thetamax, double *r, double *ptheta)
{
  ol_nb_r (n1, n2, r);
  ol_nb_max_theta_mis_rad (n3, thetamax, ptheta);

  return;
}

void
ol_nb_max_rtheta_mis (double n1, double n2, double n3,
		      double thetamax, double *r, double *ptheta)
{
  ol_nb_r (n1, n2, r);
  ol_nb_max_theta_mis (n3, thetamax, ptheta);

  return;
}
